proc pixelStringLength {str {fnt default} } {
    label .templ -text $str -font $fnt
    set rw [winfo reqwidth .templ]
    destroy .templ
    return $rw
}

proc ::tk::MouseWheel {wFired X Y D {shifted 0}} {
    # Set event to check based on call
    set evt "<[expr {$shifted?{Shift-}:{}}]MouseWheel>"
    # do not double-fire in case the class already has a binding
    if {[bind [winfo class $wFired] $evt] ne ""} { return }
    # obtain the window the mouse is over
    set w [winfo containing $X $Y]
    # if we are outside the app, try and scroll the focus widget
    if {![winfo exists $w]} { catch {set w [focus]} }
    if {[winfo exists $w]} {
	if {[bind $w $evt] ne ""} {
	    # Awkward ... this widget has a MouseWheel binding, but to
	    # trigger successfully in it, we must give it focus.
	    catch {focus} old
	    if {$w ne $old} { focus $w }
	    event generate $w $evt -rootx $X -rooty $Y -delta $D
	    if {$w ne $old} { focus $old }
	    return
	}
	# aqua and x11/win32 have different delta handling
	if {[tk windowingsystem] ne "aqua"} {
	    set delta [expr {- ($D / 30)}]
	} else {
	    set delta [expr {- ($D)}]
	}
	# scrollbars have different call conventions
	if {[string match "*Scrollbar" [winfo class $w]]} {
	    catch {tk::ScrollByUnits $w \
		       [string index [$w cget -orient] 0] $delta}
	} else {
	    set cmd [list $w [expr {$shifted ? "xview" : "yview"}] \
			 scroll $delta units]
	    # Walking up to find the proper widget handles cases like
	    # embedded widgets in a canvas
	    while {[catch $cmd] && [winfo toplevel $w] ne $w} {
		set w [winfo parent $w]
	    }
	}
    }
}

bind all <MouseWheel> [list ::tk::MouseWheel %W %X %Y %D 0]
bind all <Shift-MouseWheel> [list ::tk::MouseWheel %W %X %Y %D 1]
if {[tk windowingsystem] eq "x11"} {
    # Support for mousewheels on Linux/Unix commonly comes through
    # mapping the wheel to the extended buttons.
    bind all <4> [list ::tk::MouseWheel %W %X %Y 120]
    bind all <5> [list ::tk::MouseWheel %W %X %Y -120]
}

proc save_matrix {fp id rows cols} {
	for {set i 1} {$i<=$rows} {incr i} {
		set cmd [set ::$id\_window]
		puts $fp "\$$id\_window set row $i,1 \{[$cmd get $i,1 $i,$cols]\}"
	}
}

proc SafeScilabEval {cmd} {
        global ERR ERRTEXT CMD
        set ERR ""
        set CMD $cmd
        ScilabEval "cmd=TCL_GetVar('CMD');safe_execstr(cmd)"
}

proc jogloop {f var min max command dt style} {
	set flag "flag$var" 
	set inc "inc$var"
	global $var $flag $inc
	eval $command
	if {[expr $$var+$$inc]>$max} {
		if {$style=="once"} {
			$f.play configure -state normal;
			$f.redo configure -state normal;
			$f.loop configure -state normal;
			return
		} elseif {$style=="replay"} {
			set $var $min
		}  elseif {$style=="loop"} {
			set $var [expr $$var-$$inc]
			set $inc [expr -$$inc]
		}
		set $flag [after $dt "jogloop {$f} {$var} {$min} {$max} {$command} {$dt} {$style}"]
	} elseif {[expr $$var+$$inc]<$min} {
		set $var [expr $$var-$$inc]
		set $inc [expr -$$inc]
		set $flag [after $dt "jogloop {$f} {$var} {$min} {$max} {$command} {$dt} {$style}"]
	} else {
		set $var [expr $$var+$$inc]
		set $flag [after $dt "jogloop {$f} {$var} {$min} {$max} {$command} {$dt} {$style}"]
	}
}

proc jog {f var min max increment command dt} {

	set flag "flag$var" 
	set inc "inc$var"
	global $flag $inc $var
	
    	set $flag ""
    
	frame $f
#	set $var $min
	set $inc $increment
	Button $f.play -relief flat -width 24 -height 24 -bd 0 -image playImage -command "\
		set $inc $increment;\
		$f.play configure -state disabled;\
		$f.redo configure -state disabled;\
		$f.loop configure -state disabled;\
		jogloop $f $var $min $max {$command} {$dt} once;"
	Button $f.redo -relief flat -width 24 -height 24 -bd 0 -image redoImage -command "\
		$f.play configure -state disabled;\
		$f.redo configure -state disabled;\
		$f.loop configure -state disabled;\
		jogloop $f $var $min $max {$command} {$dt} replay;"
	Button $f.loop -relief flat -width 24 -height 24 -bd 0 -image reloadImage -command "\
		$f.play configure -state disabled;\
		$f.redo configure -state disabled;\
		$f.loop configure -state disabled;\
		jogloop $f $var $min $max {$command} {$dt} loop;"
	Button $f.pause -relief flat -width 24 -height 24 -bd 0 -image pauseImage -command "\
		after cancel $$flag; $f.play configure -state normal;$f.redo configure -state normal;$f.loop configure -state normal"
	pack $f.play $f.redo $f.loop $f.pause -side left
}

proc runScilab {bidon} {
	global ok wait lastcmd
	if {$ok == 1} {
		set wait 0
		set ok 0
		ScilabEval exec(xmllab_run,-1)
	} else {
	set wait 1
	}
}

proc mess { mess } {
  ScilabEval "messagebox('$mess')"
 }

if {[catch {set xmllab}]} {
	set xmllab $env(xmllab)
}

set fontsize 10
option add *font {Helvetica 10 roman}
if {$tcl_platform(os)=="Windows NT"} {
set fontsize 8
option add *font {Helvetica 8 roman}
}

source [file join $xmllab bin tcl env.tcl]
source [file join $xmllab bin tcl xmllablib.tcl]

set ::keep no

lappend auto_path $xmllab_lib [file join $SCI tcl]
 
package require BWidget

image create photo saveImage -file [file join $xmllab noarch images filesave.gif]
image create photo loadImage -file [file join $xmllab noarch images  fileopen.gif]
image create photo deleteImage -file [file join $xmllab noarch images editdelete.gif]
image create photo homeImage -file [file join $xmllab noarch images gohome.gif]
image create photo playImage -file [file join $xmllab noarch images media-play.gif]
image create photo pauseImage -file [file join $xmllab noarch images media-pause.gif]
image create photo redoImage -file [file join $xmllab noarch images redo.gif]
image create photo reloadImage -file [file join $xmllab noarch images reload.gif]
image create photo tearoffImage -file [file join $xmllab noarch images editcopy.gif]

catch {font create defaultFont -family Helvetica -slant roman -size $fontsize -weight normal}
catch {font create mainFont -family Helvetica -slant roman -size $fontsize -weight normal}
catch {font create menuFont -family Helvetica -slant roman -size $fontsize -weight normal}
catch {font create fixedFont -family courier -slant roman -size $fontsize -weight normal}
catch {font create boldFont -family Helvetica -slant roman -size $fontsize -weight bold}
catch {font create bigFont -family Helvetica -slant roman -size $fontsize -weight bold}

