proc xmllab {argc argv} {

global env tcl_platform xmllab_bin xmllab lang TMPDIR

if !$argc {mess "usage : xmllab file or xmllab file.xml"; return 1}

if {[catch {set xmllab}]} {
	set xmllab $env(xmllab)
}
set xmllab [string map {"\\" "/"} $xmllab]

set errorCode NONE

set sciflag "-nw"
set run 0
set web no
set scriptname ""
set havesource  no
set screenshots no
set valid 0
set k 0
set job ""
set referer ""
set webimages ""
set images ""
set logfile ""
set serverroot ""
set css ""
set amp &
set appname NONAME
set mastername NONAME
set lang DEFAULT
set debug no
set timing no
set widget plain
set shell no

while {$k<$argc} {
	set arg [lindex $argv $k] 
	if {$arg=="-lang"} {
        incr k
        set lang [lindex $argv $k]
	} elseif {$arg=="-shell"} {
        set shell yes 
	} elseif {$arg=="-appname"} {
        incr k
        set appname [lindex $argv $k]    
	} elseif {$arg=="-w"} {
        incr k
        set widget [lindex $argv $k] 
	} elseif {$arg=="-referer"} {
        incr k
        set referer [lindex $argv $k] 
	} elseif {$arg=="-css"} {
        incr k
        set css [lindex $argv $k] 
    } elseif {$arg=="-serverroot"} {
        incr k
        set serverroot [lindex $argv $k] 
    } elseif {$arg=="-webimages"} {
        incr k
        set webimages [lindex $argv $k]
    } elseif {$arg=="-images"} {
        incr k
        set images [lindex $argv $k]
    } elseif {$arg=="-logfile"} {
    	 incr k
        set logfile [lindex $argv $k]
	} elseif {$arg=="-mastername"} {
        incr k
        set mastername [lindex $argv $k]    
     } elseif {$arg=="-run"} {
	    set havesource yes
    	set run 1
    } elseif {$arg=="-c"} {
	    set sciflag "-nb"
    } elseif {$arg=="-screenshots"} {
    	set run 1
        set screenshots yes
	} elseif {$arg=="-web"} {
		set web yes
    	set run 1
    } elseif {$arg=="-scriptname"} {
        incr k
        set scriptname [lindex $argv $k] 
    } elseif {$arg=="-valid"} {
    	set valid 1
    } elseif {$arg=="-helper"} {
    	set havesource yes
    } elseif {$arg=="-debug"} {
	    set debug "yes" 
    } elseif {$arg=="-timing"} {
        incr k
	    set timing [lindex $argv $k] 
    } else {
        set job $arg	
    }
    incr k
}

if {$job==""} {
	mess "xmllab : missing argument"; 
	return 1
}

set rootname [file rootname $job]
set ext [file extension $job]
set filename [file tail $rootname]
set realfilename [file tail $rootname]
set dirname [file dirname $rootname]

if {[file pathtype $dirname]=="relative"} {
	if {$dirname=="."} {
		set dirname ""
	}
	set dirname [file join [pwd] $dirname]
}

if ![file isfile $job] {
	if {$ext==""} {set ext .xml}
	set source $rootname$ext
	if ![file isfile $source] {mess "$job, $rootname$ext : file not found"; return 1}
} else {
	set source $job
}

if $run {
    set source [file join $dirname $filename$ext]
    if {$screenshots=="no"} {
	    set filename [clock seconds]
    }
    set outputname [file join $TMPDIR $filename]
    set outputdir $TMPDIR
} else {
    if {[file writable [pwd]]} {
	set outputname [file join [pwd] $filename]
        set outputdir [pwd]
    } else {
	mess "You don't have write access in current directory"
        return 1
    }
}

catch {exec unzip -o -d $TMPDIR $source } mess
if [string match "*signature not found*" $mess] {
} else {
    set source [file join $TMPDIR content.xml]
    set dirname $TMPDIR
}

set xsltproc xsltproc
set xmllint xmllint

set env(XML_CATALOG_FILES) "file:///[string map {" " "%20" "\\" "/"} $xmllab]dtd/catalog.xml"

catch {exec $xsltproc $xmllab/xsl/fileversion.xsl $source} fileversion 
catch {exec $xsltproc $xmllab/xsl/publicid.xsl $source} publicid
catch {exec $xsltproc $xmllab/xsl/systemuri.xsl $source} systemuri

if {$fileversion == "1.6"} {
    if {[catch {exec $xsltproc --novalid\
                     --stringparam web $web\
                     --stringparam file $outputname\
                     --stringparam publicid [string map {1.6 1.7} $publicid]\
                     --stringparam systemuri [string map {1.6 1.7} $systemuri]\
                     $xmllab/xsl/convert1.7.xsl $source} mess]} {
	    if {$errorCode!="NONE"} {
    	    mess $mess
		    return 1;
	    } elseif {$run==0} {
    	    mess $mess            
        }
    }
    file copy -force $source $outputname.xml.1.6
    file rename -force  $outputname-converted.xml  $outputname.xml	
    set fileversion "1.7"
    set source $outputname.xml
} elseif {$fileversion != "1.7"} {
    mess "Cannot convert XMLlab version $fileversion files\nUse XMLlab 1.6 instead"
    return 1;
}

# Validation

if {[catch {exec $xmllint -noout --valid $source} mess]} {
	if {$errorCode!="NONE"} {
    	mess $mess
		return 1;
	}
}


if $valid { # Validate only
	return 0
}

if {[catch {exec $xsltproc --stringparam appname $appname\
	       --stringparam shell $shell\
	       --stringparam web $web\
	       --stringparam xmllab $xmllab\
	       --stringparam widget $widget\
           --stringparam timing $timing\
	       --stringparam scriptname $scriptname\
	       --stringparam logfile $logfile\
           --stringparam webimages $webimages\
           --stringparam images $images\
		   --stringparam css $css\
           --stringparam serverroot $serverroot\
		   --stringparam referer $referer\
		   --stringparam mastername $mastername\
           --stringparam screenshots $screenshots\
           --stringparam language $lang\
           --stringparam os $tcl_platform(os)\
           --stringparam havesource $havesource\
	       --stringparam file $filename\
	       --stringparam outputdir $outputdir\
           --stringparam outputname $outputname\
	       --stringparam sourcename $source\
	       --stringparam utf8file [utf8 $filename]\
	       --stringparam utf8dirname [utf8 $dirname]\
	       --stringparam utf8sourcename [utf8 $source]\
	       --stringparam utf8outputdir [utf8 $TMPDIR]\
           --stringparam utf8outputname [utf8 $outputname]\
	        $xmllab/xsl/simulation.xsl $source} mess]} {
	mess $mess
	if {$errorCode!="NONE"} {
		return 1;
	}
}       

if {[catch {exec  $xsltproc $xmllab/xsl/sciml2sci.xsl $outputname.sciml > $outputname.sce} mess]} {
	mess $mess
	if {$errorCode!="NONE"} {
		return 1;
	}
}

if {[catch {exec $xsltproc --stringparam file $filename\
            --stringparam os $tcl_platform(os)\
            --stringparam web $web\
            --stringparam dirname [utf8 $dirname]\
            --stringparam outputname [utf8 $outputname.tk]\
            --stringparam screenshots $screenshots\
            $xmllab/xsl/tkml2tk.xsl $outputname.tkml > $outputname.tk} mess]} {
	mess $mess
	if {$errorCode!="NONE"} {
		return 1;
	}
}


# files to be copied side by side with the sce and tk files

set files [exec $xsltproc $xmllab/xsl/images_and_files.xsl $source]

if {[string length $files]>0} {
	foreach file $files {
		catch {file copy -force [file join $dirname $file] [file join $outputdir $file]}
	}
}

if {$web=="yes"} {
    set files [exec $xsltproc $xmllab/xsl/images.xsl $source]
	foreach file $files {
		catch {file copy -force [file join $dirname $file] [file join $serverroot $webimages $file]}
	}
}


if {[file exists [file join $dirname $realfilename.sci] ]} {	
	file copy -force [file join $dirname $realfilename.sci] [file join $outputdir $filename.sci]
}

if {[file exists [file join $dirname $realfilename.mod] ]} {
	file copy -force [file join $dirname $realfilename.mod] [file join $outputdir $filename.mod]
}

if {$debug=="no"} {
	file delete $outputname.sciml
	file delete $outputname.tkml
}

if $argc==1 {
	return 0
}

if $run {
  if {[catch {ScilabEval 0}]} { # If we are outside Scilab
    switch $tcl_platform(os) {
      "Windows NT" {
        if {[catch {exec cmd /c start /min "" "$env(SCILAB)\\bin\\scilex" -nb -e exec(\"$outputname.sce\",-1),xmllab_loop(%t) &} mess]} {
          mess $mess
          return 1
        }
      }
      "Linux" -
      "SunOS" -
      "Darwin" {
        set cmd "exec $env(SCILAB)/../../bin/scilab $sciflag -nb -e exec(\"$outputname.sce\",-1),xmllab_loop(%t) > /dev/null &"
        if {[catch {eval $cmd} mess]} {
          mess $mess
        }
		  }
		}
  } else { # If we were called from within Scilab
    
    if {$::keep=="yes"} {
  		ScilabEval exec(\"$outputname.sce\",-1)
    } else {
      ScilabEval exec(\"$outputname.sce\",-1),xmllab_loop(%f)
    }
  }
}
return 0
}


proc xmllab_unpack {argc argv} {

	global env tcl_platform xmllab_bin xmllab lang TMPDIR

	if !$argc {mess "usage : unpack file or unpack file.xml"; return 1}

	set k 0
	set job ""
	set outputfile ""
	set outputdir "."
  set shell no

	while {$k<$argc} {
		set arg [lindex $argv $k] 
		if {$arg=="-d"} {
    		incr k
			set outputdir [lindex $argv $k] 		
		} elseif {$arg=="-shell"} {
			set shell yes 		
		} else {
    		set job $arg	
		}
		incr k
	}

	if {$job==""} {
		mess "unpack : missing argument"; 
		return 1
	}

	if {[catch {set xmllab}]} {
		set xmllab $env(xmllab)
	}

	if {[catch {set lang}]} {
		set lang $env(lang)
	}

    set xsltproc xsltproc

    set rootname [file rootname $job]
    set ext [file extension $job]
    set filename [file tail $rootname]
    set dirname [file dirname $rootname]

    if {[file pathtype $dirname]=="relative"} {
	    if {$dirname=="."} {
		    set dirname ""
	    }
	    set dirname [file join [pwd] $dirname]
    }

    set job [file join $dirname $filename]

    if ![file isfile $job] {
	    if {$ext==""} {set ext .xmllab}
	    set source $job$ext
	    if ![file isfile $source] {mess "$job, $job$ext : file not found"; return 1}
    } else {
	    set source $job
    }

    if {[catch {exec unzip -o  -d $outputdir $source} mess]} {
        mess $mess
        if {$errorCode!="NONE"} {
	        return 1;
        }
    }
    file rename -force [file join $outputdir content.xml] [file join $outputdir $filename.xml]
    return 0
}



proc xmllab_pack {argc argv} {

	global env tcl_platform xmllab_bin xmllab lang TMPDIR

	if !$argc {mess "usage : pack file or pack file.xml"; return 1}

	set k 0
	set job ""
	set outputfile ""
	set outputdir "."
	set shell no

	while {$k<$argc} {
		set arg [lindex $argv $k] 
		if {$arg=="-o"} {
    		incr k
			set outputfile [lindex $argv $k] 
		} elseif {$arg=="-d"} {
    		incr k
			set outputdir [lindex $argv $k] 		
		} elseif {$arg=="-shell"} {
			set shell yes 		
		} else {
    		set job $arg	
		}
		incr k
	}

	if {[catch {set xmllab}]} {
		set xmllab $env(xmllab)
	}

	if {[catch {set lang}]} {
		set lang $env(lang)
	}

    set xsltproc xsltproc

	set rootname [file rootname $job]
	set ext [file extension $job]
	set filename [file tail $rootname]
	set dirname [file dirname $rootname]


	if {[file pathtype $dirname]=="relative"} {
		if {$dirname=="."} {
			set dirname ""
		}
		set dirname [file join [pwd] $dirname]
	}

	if ![file isfile $job] {
		if {$ext==""} {set ext .xml}
		set source $rootname$ext
		if ![file isfile $source] {mess "$job, $rootname$ext : file not found"; return 1}
	} else {
		set source $job
	}



    if {$outputfile==""} {
        set outputfile $filename.xmllab
    }

    file copy -force $source content.xml
    set cmd [list exec zip -j [file join $outputdir $outputfile] content.xml]

	foreach href [split [exec $xsltproc $xmllab/xsl/pass1.xsl $source] \&] {
    	lappend cmd [file join $dirname $href]
	}
    
    if {[catch {eval $cmd} mess]} {
        file delete -force content.xml
        mess $mess
        return 1
    }
    file delete -force content.xml
    
	return 0
}

proc publish {argc argv} {

global env tcl_platform xmllab_bin xmllab lang TMPDIR

if !$argc {mess "usage : publish file or publish file.xml"; return 1}


if {[catch {set xmllab}]} {
	set xmllab $env(xmllab)
}

if {[catch {set lang}]} {
	set lang $env(lang)
}

set errorCode "NONE"

set k 0
set job ""
set amp &
set htmlonly 0
set nohtml 0
set application_name publish
set shell no
set keep no

package require Tk
wm withdraw .
tk appname $application_name

while {$k<$argc} {
	set arg [lindex $argv $k] 
	if {$arg=="-list"} {
        incr k
        set job [lindex $argv $k]    
        publishlist $job $htmlonly
        return 0
    } elseif {$arg=="-htmlonly"} {
    	set htmlonly 1
    } elseif {$arg=="-nohtml"} {
    	set nohtml 1
		} elseif {$arg=="-shell"} {
			set shell yes 		
		} elseif {$arg=="-keep"} {
			set keep yes
	} else {
      set job $arg	
    }
    incr k
}

if {$job=="" && $listfile==""} {
	mess "publish : missing argument"; 
	return 1
}

set rootname [file rootname $job]
set filename [file tail $rootname]

set dirname [file dirname $rootname]

if {[file pathtype $dirname]=="relative"} {
	if {$dirname=="."} {
		set dirname ""
	}
	set dirname [file join [pwd] $dirname]
}

set ext [file extension $job]

if ![file isfile $job] {
	if {$ext==""} {set ext .xml}
	set source $rootname$ext
	if ![file isfile $source] {mess "$job, $rootname$ext : file not found"; return 1}
} else {
	set source $job
}

set source [file join $dirname $filename$ext]

if {[file writable [pwd]]} {
set outputname [file join [pwd] $filename]
    set outputdir [pwd]
} else {
	mess "You don't have write access in current directory"
    return 1
}

set xsltproc xsltproc

file mkdir $filename

set imagesdir [file join $xmllab noarch images]

if {[lsearch [winfo interps] scilab]<0} {
            exec $env(SCILAB)/../../bin/scilab -e "TCL_EvalStr(\"tk appname scilab; send $application_name set ::wait no\"),xmllab_loop(%f)" > /dev/null &
            set ::wait yes 
            set after_id [after 20000 set ::wait TIMEOUT] 
            tkwait variable ::wait
            if {$::wait=="TIMEOUT"} {
              mess "Unable to start Scilab"
              return 1
            }
            after cancel $after_id
}

foreach language [exec $xsltproc $xmllab/xsl/languages.xsl $source] {

	if !$htmlonly {
      puts $source
	    if {[send scilab "xmllab 9 {-c -mastername $application_name -appname scilab -screenshots -lang $language $source}"]} {
        exit 1;
      }
      set k 1
      set ::wait yes 
      set after_id [after 60000 set ::wait TIMEOUT] 
      tkwait variable ::wait
      if {$::wait=="TIMEOUT"} {
           mess "Scilab is not responding"
           return 1
       }
      after cancel $after_id

	    foreach windowname [exec $xsltproc --stringparam lang $language $xmllab/xsl/scilab_windows.xsl $source] {
    	      set image $filename\_$language\_$k
            file copy -force $TMPDIR/$image.png $filename 
            exec convert -resize 75%  $TMPDIR/$image.png $filename/$image\_small.png 
    	    incr k
        }
        
        set image $filename\_$language\_0   
        file copy -force $TMPDIR/$image.png $filename/
        exec convert -resize 75%  $filename/$image.png $filename/$image\_small.png
        file delete -force $TMPDIR/$image.png

        file copy -force [file join $imagesdir $language\_flag.png] $filename

	}	
}

# Making the .xmllab bundled files

set archive $filename/$filename.xmllab

if !$htmlonly { 

    # Regular xmllab bundle (needs XMLlab on the client side)

	xmllab_pack 3 "$source -d $filename"

}    

# Generating the html pages

if !$nohtml {
	exec $xsltproc --stringparam file $filename\
                  $xmllab/xsl/publish_html2.xsl $source
}

# Copying the source HTML view of the XML file

exec $xsltproc --novalid\
              --stringparam "css-stylesheet" "../xmlverbatim.css"\
			  -o $filename/$filename.html\
               $xmllab/xsl/xmlverbatimwrapper.xsl $source

# Copying the source XML file

file copy -force $source $filename/

if {$keep=="no"} {
  send -async scilab {set $::keep no}
}

return 0
}

proc publishlist {job htmlonly} {

    global env tcl_platform xmllab_bin xmllab lang TMPDIR

    set xsltproc xsltproc

    set rootname [file rootname $job]
    set filename [file tail $rootname]
    set dirname [file dirname $rootname]

    if {[file pathtype $dirname]=="relative"} {
	    if {$dirname=="."} {
		    set dirname ""
	    }
	    set dirname [file join [pwd] $dirname]
    }

    set job [file join $dirname $filename]

    set ext [file extension $job]

    if ![file isfile $job] {
	    if {$ext==""} {set ext .xml}
	    set source $job$ext
	    if ![file isfile $source] {mess "$job, $job$ext : file not found"; return 1}
    } else {
	    set source $job
    }

    file mkdir $filename
    
    set oldpwd [pwd]
    cd $filename
    set imagesdir [file join $xmllab noarch images]
        
    foreach simulation [exec $xsltproc [file join $xmllab xsl publishlist.xsl] $source] {
    	set item  [file join $dirname $simulation]
        set root [file rootname [file tail $simulation]]
        set todo 0
        if ![file exists $root] {
        	set todo 1
        } elseif ![file exists $root/$root.xmllab] {
            set todo 1    
        } elseif {[file mtime $item] > [file mtime $root/$root.xmllab]} {
            set todo 1
        }
        puts "Processing file : $item"
		if $todo {
		    if $htmlonly {
	    	    publish 4 "-keep -htmlonly -nohtml $item"        	
            } else {
	    	    publish 3 "-keep -nohtml $item"        	
            }
        }
    }
    
    foreach language [exec $xsltproc $xmllab/xsl/list_languages.xsl $source] {
		file copy -force [file join $imagesdir $language\_flag.png] .
    }
    
    file copy -force $xmllab/xsl/xmlverbatim.css .

	cd $oldpwd

	# Generating the html pages
	
    exec $xsltproc --stringparam file $filename\
                   --stringparam dir [pwd]\
                   $xmllab/xsl/publishlist_html2.xsl $source

  send -async scilab {set ::keep no}        
}



