# XMLlabServer.tcl
#
# Main CGI tcl script for the XMLlab Web Server
#
# Copyright Stphane Mottelet, Universit de Technologie de Compigne
#
# Jeu jui 17 17:24:43 CEST 2008

#proc send_file {filename} {
#    set fp [open $filename]
#    fconfigure $fp -translation binary
#    puts -nonewline [read $fp]
#    close $fp
#}

proc send_file {filename} {
    after 100
    set fp [open $filename]
    fconfigure $fp -translation binary
    puts -nonewline [read $fp]
# Avec fcopy il y a parfois des problmes de caractres pas
# trs nets...  creuser 
#    fcopy $fp stdout
    close $fp
}

proc report_error {errortext ref} {
	puts "Content-type: text/html\n"
	puts "<html><body>"
	puts "<h3>The XMLlab webserver encountered an error</h3>"
	puts "<b>Error text</b> : $errortext"
	puts "<br/>"
	puts "<b>Referering page : </b><a href=\"$ref\">$ref</a>"
	puts "</body></html>"
}

# URI decoding procedure. We call xsltproc and feed the program with the string
# to be decoded:

proc uri_decode {uri} {
    return [exec echo " <form-data>$uri</form-data>" | xsltproc [file join $::env(XMLLAB) xsl uridecode.xsl] -]
}

set etime [clock clicks -milliseconds]

# The env variable HTTP_REFERER should be set, but if it is not,
# we put there an empty string:

if [catch {set env(HTTP_REFERER)}] {
    set env(HTTP_REFERER) ""
}

set auto_path "\"[file join $env(XMLLAB) noarch lib]\" $auto_path" 

if [catch {package require ncgi} mess] {
    report_error $mess $env(HTTP_REFERER)
    exit
}

if {$env(REALUSER)!=$env(USER) && $env(REALUSER)!=""} {
    report_error "The CGI script doesn't seem to run as yourself ($env(REALUSER) instead of $env(USER)).
    Ask you system administrator to enable the suexec feature in the Apache server." $env(HTTP_REFERER)
    exit
} 

# On Darwin/Tiger, Apache (1.3.33) does strange things with encoding (brute force
# "on the fly" conversion to UTF-8). So we have to change the HTTP header
# according to this behavior.

switch $tcl_platform(os) {
    Darwin {set HEADER_CHARSET "UTF-8"}
    Linux  {set HEADER_CHARSET "ISO-8859-1"}
    SunOS  {set HEADER_CHARSET "ISO-8859-1"}
}

# Default language is chosen at install time

set timing no
set lang $env(lang)


set logdir [file join $env(SERVERROOT) XMLlabServer sessions logs]

set address $env(REMOTE_ADDR)

if {![catch {set env(HTTP_X_FORWARDED_FOR)} mess]} {
    set address $env(HTTP_X_FORWARDED_FOR)
    set logfile [file join $logdir $env(REMOTE_ADDR).$address.log]
} else {
    set logfile [file join $logdir $address.log]
}

# Parse the form data

::ncgi::parse

if [::ncgi::exists referer] {
    ::ncgi::import referer
    set env(HTTP_REFERER) $referer
}


if [::ncgi::exists lang] {::ncgi::import lang}

if [::ncgi::exists list] {

    ::ncgi::import list listurl

    # Generate main page listing simulations. Such lists are stored in the
    # XMLlabServer/lists directory.
    
    set env(XML_CATALOG_FILES) $env(XMLLAB)/dtd/catalog.xml
    catch {exec xsltproc --stringparam lang $lang\
                    --stringparam timing $timing\
                    --stringparam servername $env(SERVER_NAME)\
                    --stringparam scriptname $env(SCRIPT_NAME)\
                    --stringparam file $listurl\
					--stringparam css "XMLlabServer/css/$env(CSS)"\
					--stringparam images "XMLlabServer/images"\
                      $env(XMLLAB)/xsl/weblist.xsl [file join $env(SERVERROOT) XMLlabServer lists $listurl]} mess
    if {$errorCode=="NONE"} {
        puts "Content-type: text/html; charset=$HEADER_CHARSET\n"
        puts $mess
    } else {
        report_error $mess $env(HTTP_REFERER)
    }
    exit
}

if [::ncgi::exists action] {

    ::ncgi::import action
  
    set address $env(REMOTE_ADDR)
	if {![catch {set env(HTTP_X_FORWARDED_FOR)} mess]} {
		set address $env(HTTP_X_FORWARDED_FOR)
    }


    # We test if there are not more than 8 users already connected

	if {[llength [glob -nocomplain -directory $logdir *.log]] >= 8} {
        report_error "There are too many users, please\
        try again later..." $env(HTTP_REFERER)
	    exit
    }
    if [::ncgi::exists timing] {::ncgi::import timing}

    if [::ncgi::exists simulationFile] {
        set simulation [::ncgi::importFile -server simulationFile]
	# set simulation  [::ncgi::importFile -client simulationFile]
    } else {
        ::ncgi::import simulation
    }
    
    if {$action=="run"} {
       
		# We test if the vnc server is alive and launch a new
		# server if needed.

		if [catch {exec xhost} mess] {
         if {$errorCode != "NONE"} {                        
			if {[catch {exec vncserver :$env(DISPLAYNUMBER) -auth $env(XAUTHORITY)} mess]} {
        		if {$errorCode != "NONE"} {
    	    		report_error $mess $env(HTTP_REFERER)
	        	}
        	}
          }
		}

		# We load the Tk library right now:

		if {[catch {package require Tk} mess]} {
			report_error $mess $env(HTTP_REFERER)
		}

        # Disconnect an eventual previous session which has not already 
        # expired.

        if [file exists $logfile] {
            set fp [open $logfile r]
            set session [read $fp]
            close $fp
            if {![catch {send $session {after cancel $::countdown_id}} mess]} {
	    	    tk appname cgi$session
            	# report_error "Scilab already started" $env(HTTP_REFERER)
            } else {
	            report_error "Scilab abnormally crashed during previous session" $env(HTTP_REFERER)	
		        file delete $logfile
                exit
            }
        } else {
	        set session "session[clock seconds]"
            tk appname cgi$session
            exec xterm -e vglrun scilab -e "TCL_EvalStr(\"tk appname $session; send cgi$session set ::wait no\")" < /dev/null > /dev/null &
            set ::wait yes 
            after 20000 set ::wait TIMEOUT         
            tkwait variable ::wait
            if {$wait!="TIMEOUT"} {
                set fp [open $logfile w]
                puts -nonewline $fp $session
                close $fp            
                # report_error "Scilab started" $env(HTTP_REFERER)
            } else {
                report_error "Unable to start Scilab" $env(HTTP_REFERER)
                exit
            }
	    }
        set html_file 0
        puts "Content-type: text/html; charset=$HEADER_CHARSET;\n"

        send $session "xmllab 25 {-c -logfile $logfile\
               -lang $lang\
               -timing $timing\
               -web -appname $session\
               -scriptname $env(SCRIPT_NAME)\
               -referer \"$env(HTTP_REFERER)\"\
               -css XMLlabServer/css/$env(CSS)\
               -webimages XMLlabServer/sessions/images\
               -images XMLlabServer/images\
               -serverroot $env(SERVERROOT)\
               -mastername cgi$session $simulation}"

        after 20000 set html_file "TIMEOUT"
        tkwait variable html_file

        if {$html_file!="TIMEOUT"} {
            send_file $html_file

            close $fp
            if {$timing=="yes"} {
                set seconds [format %5.2f [expr ([clock clicks -milliseconds] - $etime)/1000.]]
                puts "<small>Generated in $seconds seconds</small>"
            }
        } else {
            report_error "session $session TIMEOUT" $env(HTTP_REFERER)
        }

    } elseif {$action=="launch"}  {

        puts "Content-type: text/html; charset=$HEADER_CHARSET;\n"
	    set uri [string map {action=launch action=run} $env(REQUEST_URI)]
	    puts "<html><head><meta http-equiv=Refresh content=0;url=http://$env(HTTP_HOST)$uri>"
	    puts " </head><body><p/><p align=center>XMLlabWebServer : Please wait while loading<p><p align=center>$simulation</p>"
		puts "<p align=center><img src=http://xmllab.org/ajax-loader.gif></p>"
        if [file exists $logfile] {
            puts " <p align=center>(you are logged in since [exec date] with IP address $address)</p>"	    
        } else {
            puts " <p align=center>(creating a new session for host address $address : launching Scilab)</p>"	        
        }
        puts "<p align=center><img src=http://xmllab.org/xmllab_scilab_powered.gif></img></p>"
	    puts "</body></html>"

    } else {
        report_error "$action : action not supported" $env(HTTP_REFERER)
    }
    exit
}


if [::ncgi::exists session] {
	set command ""
    
	foreach var [::ncgi::names] {
        if {$var=="session"} {        
            ::ncgi::import session
		} elseif {$var=="type"} {
            ::ncgi::import type
        } elseif {$var=="referer"} {
            ::ncgi::import referer
        } elseif {$var=="timing"} {
            ::ncgi::import timing
        } elseif {$var=="window"} {
            ::ncgi::import window
        } elseif {$var=="click.x"} {
            ::ncgi::import click.x xclicked
        } elseif {$var=="click.y"} {
            ::ncgi::import click.y yclicked
        } elseif {$var=="source"} {
            ::ncgi::import source
        } elseif {$var != "sessionFile"} {
            # other parameter/value pairs append in command string
            append command "set $var [::ncgi::import $var]; "
        }
    }

    # Depending on what is requested (parameter change, parameters section change or
    # action triggering, we set the command used by Scilab to output the html code
    # of the simulation page:
    
    set refresh "runScilab {}"
    set imagesrefresh "?[clock seconds]" 
    
    if {$type=="section"} {
        set refresh "output_html"
        set imagesrefresh ""
    } elseif {$type=="action"} {
        set refresh ""
        set imagesrefresh "?[clock seconds]"
    } elseif {$type=="click"} {
        set refresh ""
    }

    append command "set imagesrefresh $imagesrefresh; "
    
    if {$type=="download"} {
        puts "Content-type: text/xml"
        puts "Content-Disposition: inline; filename=[file tail $source]\n"
        send_file $source
        exit
    }


    if {[catch {package require Tk} mess]} {
         report_error $mess $referer
    }

    # We set our own name
    
	tk appname cgi$session
    
    # If the simulation has to be killed, we just send the magic command:

    if {$type=="kill"} {
        puts "Location: $referer\n"
        exit   
    }

    # We verify is the simulation is still alive:
    
	
	if {[lsearch [winfo interps] $session]<0} {
                if {[file exists $logfile]} {
                    file delete $logfile
                    report_error "session $session has abnormally expired because of a Scilab crash" $referer
                } else {
		    report_error "session $session has expired because of no user interaction during 120 seconds" $referer
                }
                exit 
	}
    
    if {$type=="click"} {
        send -async $session "ScilabEval xmllab_handler($window,$xclicked,$yclicked,$env(SCICLICKEVENT),%f)"
    }
    
    if {$type=="getpdf"} {
        set pdffile $env(SERVERROOT)/XMLlabServer/sessions/images/$session\_$window
        set pdf_file 0        
        send -async $session "ScilabEval xmllab_send_pdf($window,\"$pdffile\",\"cgi$session\")"
	    
        after 20000 set pdf_file TIMEOUT

        tkwait variable pdf_file
	    if {$pdf_file!="TIMEOUT"} {

        #
        #   With this method the pdf file is corrupted by Apache under Darwin !
        #
		puts "Content-type: application/pdf"
		puts "Content-Disposition: attachment; filename=$session\_$window.pdf\n"
		send_file $pdffile.pdf

#	        puts "Location: ../XMLlabServer/sessions/images/$session\_$window.pdf?[clock seconds]\n"
    	    exit
        }
    }
    
     
    if {$type=="saveSession"} {
        set session_file 0
        send -async $session "save_session $env(HOME)/.XMLlab/cache/$session.sess; send cgi$session {set session_file 1}"
	    after 15000 set session_file TIMEOUT
        if {$session_file=="0"} {
            tkwait variable session_file
            if {$session_file=="TIMEOUT"} {puts "\n";exit}
        }
        puts "Content-type: text/ascii"
        puts "Content-Disposition: attachment; filename=[file rootname [file tail $source]].sess\n"
        send_file $env(HOME)/.XMLlab/cache/$session.sess
        exit
    }  

    if {$type=="loadSession"} {
	append command "load_session [::ncgi::importFile -server sessionFile];"  
    }  

    # Otherwise, we send to Scilab the remaining commands

	set html_file 0
	send -async $session  "$command $refresh"
	after 10000 set html_file TIMEOUT

    # The "html_file" variable is set by the distant Scilab using a "send". 
    # This variable is supposed to contain the name of the html file to be
    # flushed.

    tkwait variable html_file
	if {$html_file!="TIMEOUT"} {
        puts "Content-type: text/html; charset=$HEADER_CHARSET\n"
		send_file $html_file
        if {$timing=="yes"} {
            set seconds [format %5.2f [expr ([clock clicks -milliseconds] - $etime)/1000.]]
            puts "<small>Generated in $seconds seconds</small>"
        }
		exit
	} else {
        if {[lsearch [winfo interps] $session]<0} {
            if {[file exists $logfile]} {
                file delete $logfile
                report_error "session $session has abnormally expired because of a Scilab crash" $referer
            } 
        } else {
            send $session "ScilabEval abort"
	        report_error "Scilab did not answer after 10 seconds (maybe some parameters are out of range, or the maximum computation time is too low)" $referer
        }
        exit
    }
		
	report_error "$session TIMEOUT" $referer
	exit
}


puts "\nXMLlab WebServer version 1.74\n"
puts "[exec uname -a]\n"
puts [exec env]
exit


