<?xml version="1.0"?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:output method="xml" indent="no" encoding="ISO-8859-1"/>

<xsl:strip-space elements="*"/>

<!-- Generation of code for curves and surfaces -->

<xsl:template match="polyline" mode="script">
	<assign>
		<lhs>
			<xsl:value-of select="@label"/><xsl:text>X</xsl:text>
		</lhs>
		<rhs>
			<list sep=",">
				<xsl:for-each select="vertex">
					<parm>
						<xsl:value-of select="@x1"/>
					</parm>
				</xsl:for-each>
			</list>
		</rhs>
	</assign>
	<assign>
		<lhs>
			<xsl:value-of select="@label"/><xsl:text>Y</xsl:text>
		</lhs>
		<rhs>
			<list sep=",">
				<xsl:for-each select="vertex">
					<parm>
						<xsl:value-of select="@x2"/>
					</parm>
				</xsl:for-each>
			</list>
		</rhs>
	</assign>
</xsl:template>

<xsl:template match="polyline" mode="plot">
	<parm>
		<xsl:value-of select="@label"/><xsl:text>X</xsl:text>
	</parm>
	<parm>
		<xsl:value-of select="@label"/><xsl:text>Y</xsl:text>
	</parm>
</xsl:template>

<xsl:template match="polyline" mode="text">
	<xsl:for-each select="vertex[name]">
		<macro-call name="xstring">
			<inputs>
				<parm>
					<xsl:value-of select="@x1"/>
				</parm>
				<parm>
					<xsl:value-of select="@x2"/>
				</parm>
				<parm>
					<list sep=";">
					    <parm>
                    	    <string>
							    <xsl:value-of select="concat('  ',normalize-space(name[(@lang=$lang) or (not(@lang))]))"/>
						    </string>
					    </parm>
					    <parm>
                    	    <string/>
                        </parm>
					</list>
				</parm>
			</inputs>
		</macro-call>
	</xsl:for-each>
</xsl:template>

<xsl:template match="polyline" mode="functions"/>

<xsl:template match="x1|x2|x3" mode="plot">
	<parm>
		<xsl:choose>
			<xsl:when test="@label">
				<xsl:value-of select="@label"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="generate-id()"/>
			</xsl:otherwise>
		</xsl:choose>
	</parm>
</xsl:template>

<xsl:template match="x1|x2|x3" mode="script">
	<xsl:call-template name="variables"/>
	<assign>
		<lhs>
			<xsl:choose>
				<xsl:when test="@label">
					<xsl:value-of select="@label"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="generate-id()"/>
				</xsl:otherwise>
			</xsl:choose>
		</lhs>
		<rhs>
			<xsl:value-of select="normalize-space(value)"/>
		</rhs>
	</assign>
</xsl:template>


<xsl:template match="nonparametriccurve2d" mode="plot">
	<xsl:apply-templates mode="plot"/>
</xsl:template>

<xsl:template match="nonparametriccurve2d" mode="script">
	<xsl:apply-templates mode="script"/>
</xsl:template>

<xsl:template match="nonparametriccurve2d" mode="labels">
	<macro-call name="_xlabel">
		<inputs>
			<xsl:apply-templates select="refdomain1d|defdomain1d" mode="labels"/>
		</inputs>
	</macro-call>

	<macro-call name="_ylabel">
		<inputs>
		<parm>
			<string>
				<xsl:value-of select="normalize-space(x2/name[(@lang=$lang) or (not(@lang))])"/>
			</string>
		</parm>
		</inputs>
	</macro-call>
</xsl:template>

<xsl:template match="nonparametriccurve2d" mode="functions"/>
<xsl:template match="nonparametriccurve2d" mode="text"/>

<xsl:template match="parametriccurve2d" mode="plot">
	<xsl:apply-templates mode="plot" select="x1|x2"/>
</xsl:template>

<xsl:template match="parametriccurve2d" mode="labels">
	<macro-call name="_xlabel">
		<inputs>
		<parm>
			<string>
				<xsl:value-of select="normalize-space(x1/name[(@lang=$lang) or (not(@lang))])"/>
			</string>
		</parm>
		</inputs>
	</macro-call>
	<macro-call name="_ylabel">
		<inputs>
		<parm>
			<string>
				<xsl:value-of select="normalize-space(x2/name[(@lang=$lang) or (not(@lang))])"/>
			</string>
		</parm>
		</inputs>
	</macro-call>
</xsl:template>



<xsl:template match="parametriccurve2d[not(parent::boundary)]" mode="script">
	<xsl:apply-templates mode="script"/>
</xsl:template>

<xsl:template match="parametriccurve2d" mode="script"/>
<xsl:template match="parametriccurve2d" mode="functions"/>
<xsl:template match="parametriccurve2d" mode="text"/>

<xsl:template match="parametriccurve3d" mode="script">
	<xsl:apply-templates mode="script"/>
</xsl:template>


<xsl:template match="parametriccurve3d" mode="plot">
	<xsl:apply-templates mode="plot" select="x1|x2|x3"/>
</xsl:template>

<xsl:template match="parametriccurve3d" mode="labels">
	<parm><string>xlabel</string></parm>
	<parm><string>
		<xsl:value-of select='normalize-space(x1/name[(@lang=$lang) or (not(@lang))])'/>
	</string></parm>
	<parm><string>ylabel</string></parm>
	<parm><string>
		<xsl:value-of select='normalize-space(x2/name[(@lang=$lang) or (not(@lang))])'/>
	</string></parm>
	<parm><string>zlabel</string></parm>
	<parm><string>
		<xsl:value-of select='normalize-space(x3/name[(@lang=$lang) or (not(@lang))])'/>
	</string></parm>
</xsl:template>


<xsl:template match="parametriccurve3d" mode="functions"/>
<xsl:template match="parametriccurve3d" mode="text"/>


</xsl:stylesheet>
