<?xml version="1.0"?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:str="http://exslt.org/strings"
  xmlns:math="http://exslt.org/math" 
  xmlns:exslt="http://exslt.org/common" 
exclude-result-prefixes="str math">

<xsl:output method="xml" indent="no" encoding="ISO-8859-1"/>

<xsl:key name="point" match="point" use="@label"/>


<xsl:strip-space elements="*"/>

<!-- Generation of code for display -->

<xsl:template match="display" mode="functions">
	<macro-call name="graphinit">
		<inputs>
			<parm>
				<xsl:choose>
					<xsl:when test="@background='white'">
						<xsl:text>[1,1,1]</xsl:text>
					</xsl:when>
					<xsl:when test="@background='black'">
						<xsl:text>[0,0,0]</xsl:text>
					</xsl:when>
					<xsl:when test="@background='blue'">
						<xsl:text>[0,0,0.5]</xsl:text>
					</xsl:when>
				</xsl:choose>
			</parm>
		</inputs>
	</macro-call>

	<xsl:apply-templates mode="functions"/>	

    <function-definition name="xmllab_display">
        <body>
	        <xsl:apply-templates/>	
            <xsl:if test="$web='yes'">
                <macro-call name="TCL_EvalStr">
					<inputs>
						<parm>
							<string>output_html</string>
						</parm>
					</inputs>
				</macro-call>
            </xsl:if>
        </body>
    </function-definition>

</xsl:template>

<xsl:template match="display" mode="script">
    <macro-call name="xmllab_display"/>
</xsl:template>

<xsl:template match="window" mode="functions">

    <xsl:variable name="splitx">
        <xsl:choose>
            <xsl:when test="@splitx">
                <xsl:value-of select="@splitx"/>
            </xsl:when>
            <xsl:when test="@splity">
                <xsl:value-of select="ceiling(count(axis2d|axis3d) div @splity)"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="ceiling(math:sqrt(count(axis2d|axis3d)))"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>

    <xsl:variable name="splity">
        <xsl:choose>
            <xsl:when test="@splity">
                <xsl:value-of select="@splity"/>
            </xsl:when>
            <xsl:when test="@splitx">
                <xsl:value-of select="ceiling(count(axis2d|axis3d) div @splitx)"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="floor(math:sqrt(count(axis2d|axis3d)))"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>


	<assign>
		<lhs>
			<xsl:value-of select="concat('win',position()-1)"/>
		</lhs>
		<rhs>
			<function-call name="_fig">
				<inputs>
					<parm>
						<xsl:value-of select="position()-1"/>
					</parm>
					<parm><string>reset</string></parm>
				</inputs>
			</function-call>
		</rhs>
	</assign>

	<macro-call name="_clf"/>
	
    <xsl:if test="@colormap">
		<macro-call name="_colormap">
			<inputs>
				<parm>
					<function-call name="{@colormap}colormap">
						<inputs>
							<parm><xsl:value-of select="$colormapsize"/></parm>
						</inputs>
					</function-call>
				</parm>
			</inputs>
		</macro-call>
	</xsl:if>
 
   <assign>
	    <lhs>
		    <xsl:value-of select="concat('win',position()-1,'.axes_size')"/>
	    </lhs>
	    <rhs>
            <list>
    		    <parm><xsl:value-of select="concat('450+256*(',$splitx,'-1)')"/></parm>
	    	    <parm><xsl:value-of select="concat('360+224*(',$splity,'-1)')"/></parm>
		    </list>
        </rhs>
    </assign>

    <xsl:if test="not(title[(@lang=$lang) and @variables])">
	    <assign>
		    <lhs>
			    <xsl:value-of select="concat('win',position()-1,'.figure_name')"/>
		    </lhs>
		    <rhs>
                <xsl:choose>
                    <xsl:when test="title[@lang=$lang]">
                        <xsl:apply-templates select="title[@lang=$lang]" mode="format"/>
		            </xsl:when>
                    <xsl:otherwise>
                        <string>
                            <xsl:value-of select="concat('ScilabGraphic',position()-1)"/>
                        </string>
                    </xsl:otherwise>
                </xsl:choose>
		    </rhs>
	    </assign>
    </xsl:if>
    
	<assign>
		<lhs>
			<xsl:value-of select="concat('win',position()-1,'.auto_resize')"/>
		</lhs>
		<rhs>
			<string>off</string>
		</rhs>
	</assign>
    
    	<assign>
		<lhs>
			<xsl:value-of select="concat('win',position()-1,'.auto_resize')"/>
		</lhs>
		<rhs>
			<string>on</string>
		</rhs>
	</assign>
    
<!--	<assign>
		<lhs>
			<xsl:value-of select="concat('win',position()-1,'.pixmap')"/>
		</lhs>
		<rhs>
			<string>on</string>
		</rhs>
	</assign> 
    
    <assign>
		<lhs>
			<xsl:value-of select="concat('win',position()-1,'.immediate_drawing')"/>
		</lhs>
		<rhs>
			<string>on</string>
		</rhs>
	</assign>-->

    <xsl:if test="descendant::drawpoints">
        <macro-call name="_add_event_handler">
            <inputs>
                <parm>
                    <xsl:value-of select="position()-1"/>
                </parm>
                <parm>
                    <string>xmllab_handler</string>
                </parm>
            </inputs>
        </macro-call>
    </xsl:if>
        
</xsl:template>

<xsl:template match="title" mode="format" name="format">
    <xsl:choose>
        <xsl:when test="@variables">
            <function-call name="sprintf">
                <inputs>
                    <parm>
                        <string>
                            <xsl:value-of select="normalize-space(.)"/>
                        </string>
                    </parm>
                    <xsl:for-each select="str:split(@variables,' ')">
                        <parm>
                             <xsl:value-of select="normalize-space(.)"/>
                        </parm>
                    </xsl:for-each>
                </inputs>
            </function-call>
        </xsl:when>
        <xsl:otherwise>
            <string>
                <xsl:value-of select="normalize-space(.)"/>
            </string>
        </xsl:otherwise>
    </xsl:choose>
</xsl:template>


<xsl:template match="window">
	<xsl:choose>
		<xsl:when test="@dependencies">
			<ifthenelse>
				<condition>
					<function-call name="or">
						<inputs>
							<parm>
								<list sep=",">
									<xsl:for-each select="str:split(@dependencies,' ')">
										<parm>
											<xsl:value-of select="concat(.,'_changed')"/>
										</parm>
									</xsl:for-each>
								</list>
							</parm>
						</inputs>
					</function-call>
				</condition>
				<if>
					<xsl:call-template name="window"/>
				</if>
			</ifthenelse>
		</xsl:when>
		<xsl:otherwise>
			<xsl:call-template name="window"/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template name="window">
	
	<xsl:if test="count(../window)>1">
		<macro-call name="_fig">
			<inputs>
				<parm>
					<xsl:value-of select="position()-1"/>
				</parm>
			</inputs>
		</macro-call>
	</xsl:if>

    <xsl:if test="title[(@lang=$lang) and @variables]">
		<macro-call name="xname">
			<inputs>
				<parm>
                    <xsl:apply-templates select="title[@lang=$lang]" mode="format"/>
				</parm>
			</inputs>
		</macro-call>        
    </xsl:if>

<!--	<macro-call name="_cla"/>-->

    <xsl:variable name="splitx">
        <xsl:choose>
            <xsl:when test="@splitx">
                <xsl:value-of select="@splitx"/>
            </xsl:when>
            <xsl:when test="@splity">
                <xsl:value-of select="ceiling(count(axis2d|axis3d) div @splity)"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="ceiling(math:sqrt(count(axis2d|axis3d)))"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>

    <xsl:variable name="splity">
        <xsl:choose>
            <xsl:when test="@splity">
                <xsl:value-of select="@splity"/>
            </xsl:when>
            <xsl:when test="@splitx">
                <xsl:value-of select="ceiling(count(axis2d|axis3d) div @splitx)"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="ceiling(count(axis2d|axis3d) div $splitx)"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>

    <macro-call name="drawlater"/>

	<xsl:for-each select="axis2d|axis3d">

		<xsl:if test="last()>1">
			<macro-call name="_subplot">
				<inputs>
					<parm>
						<xsl:value-of select="$splity"/>
					</parm>
					<parm>
						<xsl:value-of select="$splitx"/>
					</parm>
					<parm>
						<xsl:value-of select="position()"/>
					</parm>
				</inputs>
			</macro-call>
		</xsl:if>

		<xsl:if test="@cmin and @cmax">
			<macro-call name="_caxis">
				<inputs>
					<parm>
						<list>
							<parm>
								<xsl:value-of select="@cmin"/>
							</parm>
							<parm>
								<xsl:value-of select="@cmax"/>
							</parm>
						</list>
					</parm>
				</inputs>
			</macro-call>
		</xsl:if>
				
		<xsl:apply-templates mode="plot" select="*[not(self::drawpoints)]"/>

		<xsl:apply-templates mode="text"/>

		<xsl:apply-templates mode="plot" select="drawpoints"/>

		<xsl:apply-templates mode="labels" select="*[(self::drawcurve2d) or (self::drawsurface)][1]"/>
		<xsl:apply-templates mode="labels" select="title[@lang=$lang]"/>

        
		<xsl:if test="@legend!='off'">
          <xsl:choose>
                <xsl:when test="self::axis2d">
                    <assign>
                        <lhs>legend_list</lhs>
                        <rhs>[]</rhs>
                    </assign>
					<xsl:apply-templates mode="legend"/>                    
			        <macro-call name="_legend">
				        <inputs>
                            <parm>legend_list</parm>
					        <xsl:choose>
						        <xsl:when test="@legend='ne'">
							        <parm>1</parm>
						        </xsl:when>
						        <xsl:when test="@legend='se'">
							        <parm>4</parm>
						        </xsl:when>
						        <xsl:when test="@legend='sw'">
							        <parm>3</parm>
						        </xsl:when>
						        <xsl:when test="@legend='nw'">
							        <parm>2</parm>
						        </xsl:when>
					        </xsl:choose>
				        </inputs>
			        </macro-call>
                </xsl:when>
            </xsl:choose>
		</xsl:if>

		<xsl:if test="@cmin and @cmax">
			<macro-call name="_caxis">
				<inputs>
					<parm>
						<string>auto</string>
					</parm>
				</inputs>
			</macro-call>
		</xsl:if>
		
	</xsl:for-each>	

  <macro-call name="drawnow"/>

	<xsl:if test="($screenshots='yes')">
	   
     <macro-call name="_shg">
		    <inputs>
				<parm>
					<xsl:value-of select="position()-1"/>                	
                </parm>
		    </inputs>
	    </macro-call>    
     <macro-call name="xs2png">
		    <inputs>
				<parm>
					<xsl:value-of select="position()-1"/>                	
                </parm>
			    <parm>
				    <string>
                    	<xsl:value-of select="concat($file,'_',$lang,'_',position(),'.png')"/>
                    </string>
			    </parm>
		    </inputs>
	    </macro-call>    
    </xsl:if>
	
	<xsl:if test="$web='yes'">
	    <macro-call name="xs2png">
		    <inputs>
				<parm>
					<xsl:value-of select="position()-1"/>                	
                </parm>
			    <parm>
				    <string>
                    	<xsl:value-of select="concat($serverroot,'/',$webimages,'/',$file,'_',$lang,'_',position(),'.png')"/>
                    </string>
			    </parm>
		    </inputs>
	    </macro-call>    
        <macro-call name="unix_g">
		    <inputs>
			    <parm>
				    <string>
                    	<xsl:value-of select="concat('chmod a+r ',$serverroot,'/',$webimages,'/',$file,'_',$lang,'_',position(),'.png')"/>
                    </string>
			    </parm>
		    </inputs>
	    </macro-call>
  	</xsl:if>
				
</xsl:template>

<xsl:template name="add-axis-options-2d">
	<xsl:if test="(../@xmin) and (../@xmax) and  (../@ymin) and (../@ymax)">
		<parm>
			<string><verb>axis</verb></string>
		</parm>
		<parm>
			<list>
				<parm>
					<xsl:value-of select="../@xmin"/>
				</parm>
				<parm>
					<xsl:value-of select="../@xmax"/>
				</parm>
				<parm>
					<xsl:value-of select="../@ymin"/>
				</parm>
				<parm>
					<xsl:value-of select="../@ymax"/>
				</parm>
			</list>
		</parm>
	</xsl:if>
	<xsl:if test="../@iso='yes'">
		<parm>
			<string><verb>axis</verb></string>
		</parm>
		<parm>
			<string><verb>equal</verb></string>
		</parm>
	</xsl:if>
	<parm>
		<string><verb>axis</verb></string>
	</parm>
	<parm>
		<string><xsl:value-of select="../@position"/></string>
	</parm>
</xsl:template>

<xsl:template name="add-specific-options-3d">	
	<xsl:if test="@mode != 'wireframe'">
	       <parm>
		       <string>colorbar</string>
	       </parm>
	       <parm>
		       <string><xsl:value-of select="../@colorbar"/></string>
	       </parm>
		   <xsl:choose>
			<xsl:when test="@shading!='mixed'">
	    	   <parm>
		    	   <string>shading</string>
	    	   </parm>
	    	   <parm>
		    	   <string><xsl:value-of select="@shading"/></string>
	    	   </parm>
			</xsl:when>
			<xsl:otherwise> <!-- shading='mixed' -->
	    	   <parm>
		    	   <string>facecolor</string>
	    	   </parm>
	    	   <parm>
		    	   <string>interp</string>
	    	   </parm>
			</xsl:otherwise>
		   </xsl:choose>
	   </xsl:if>
    
        <xsl:choose>
            <xsl:when test="ancestor::axis2d">
                <xsl:apply-templates select="//*[current()/@ref=@label]" mode="labels2d"/>
            </xsl:when>
            <xsl:when test="ancestor::axis3d">
                <xsl:apply-templates select="//*[current()/@ref=@label]" mode="labels3d"/>
            </xsl:when>
        </xsl:choose>

</xsl:template>


<xsl:template name="add-axis-options-3d">
	<xsl:if test="../@iso='yes'">
		<parm>
			<string><verb>axis</verb></string>
		</parm>
		<parm>
			<string><verb>equal</verb></string>
		</parm>
	</xsl:if>
	<xsl:if test="(../@xmin) and (../@xmax) and  (../@ymin) and (../@ymax) and  (../@zmin) and (../@zmax)">
		<parm>
			<string><verb>axis</verb></string>
		</parm>
		<parm>
			<list>
				<parm>
					<xsl:value-of select="../@xmin"/>
				</parm>
				<parm>
					<xsl:value-of select="../@xmax"/>
				</parm>
				<parm>
					<xsl:value-of select="../@ymin"/>
				</parm>
				<parm>
					<xsl:value-of select="../@ymax"/>
				</parm>
				<parm>
					<xsl:value-of select="../@zmin"/>
				</parm>
				<parm>
					<xsl:value-of select="../@zmax"/>
				</parm>
				
			</list>
		</parm>
	</xsl:if>
	<xsl:if test="@hidden">
		<parm>
			<string><verb>hidden</verb></string>
		</parm>
		<parm>
			<string><xsl:value-of select="@hidden"/></string>
		</parm>
	</xsl:if>

	<parm>
		<string>view</string>
	</parm>
	<parm>
		<list>
			<parm><xsl:value-of select="../@azimuth"/></parm>
			<parm><xsl:value-of select="../@elevation"/></parm>
		</list>
	</parm>

</xsl:template>

<xsl:template name="hold-if-necessary">

		<macro-call name="_hold">
			<inputs>
				<parm>
					<string>on</string>
				</parm>
			</inputs>
		</macro-call>
	
	<xsl:if test="position()=last()">
		<macro-call name="_hold">
			<inputs>
				<parm>
					<string>off</string>
				</parm>
			</inputs>
		</macro-call>
	</xsl:if>

</xsl:template>


<xsl:template name="triangulation-test">
		<xsl:for-each select="//*[current()/@ref=@label]">
			<xsl:for-each select="defdomain2d | //defdomain2d[@label=current()/refdomain2d/@ref] | //defdomain2d[@label=current()/../refdomain2d/@ref]">
				<xsl:if test="border">
					<xsl:value-of select="'tri'"/>
				</xsl:if>
			</xsl:for-each>		
		</xsl:for-each>
</xsl:template>

<xsl:template match="drawpoints" mode="plot">
	<macro-call name="init_target">
		<inputs>
			<parm>
				<list sep=";">
                	<xsl:call-template name="split-point-ref">
                    	<xsl:with-param name="content" select="@ref"/>
                    </xsl:call-template>
				</list>
			</parm>
		</inputs>
	</macro-call>
</xsl:template>

<xsl:template name="init_target_parms">
   <parm>
	  <string>
		  <xsl:value-of select="x1/@label"/>
	  </string>
	  <xsl:text>,</xsl:text>
	  <string>
		  <xsl:value-of select="x2/@label"/>
	  </string>
	  <xsl:text>,</xsl:text>
	  <string>
		  <xsl:value-of select="@label"/>
	  </string>							
   </parm>
</xsl:template>

<xsl:template name="split-point-ref">
  <xsl:param name="content"/>
  <xsl:choose>
    <xsl:when test="contains($content, ' ')">
      <xsl:for-each select="key('point',substring-before($content, ' '))">
        <xsl:call-template name="init_target_parms"/>
      </xsl:for-each>
      <xsl:call-template name="split-point-ref">
        <xsl:with-param name="content" select="substring-after($content, ' ')"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:for-each select="key('point',$content)">
        <xsl:call-template name="init_target_parms"/>
      </xsl:for-each>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<xsl:template match="drawcurve2d" mode="plot">
    <xsl:variable name="colors">
        <xsl:choose>
            <xsl:when test="../../../@background='white'">
                <color>b</color>
                <color>g</color>
                <color>r</color>
                <color>c</color>
                <color>m</color>
                <color>y</color>
                <color>k</color>
            </xsl:when>
            <xsl:otherwise>
                <color>y</color>
                <color>m</color>
                <color>c</color>
                <color>b</color>
                <color>g</color>
                <color>r</color>
                <color>w</color>                
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    <xsl:variable name="linetypes">
        <linetype>-</linetype>
        <linetype>--</linetype>
        <linetype>-.</linetype>
        <linetype>.</linetype>
    </xsl:variable>
    <xsl:variable name="colorindex" select="count((. | preceding-sibling::drawcurve2d)[@color='auto'])"/>
    <xsl:variable name="linetypeindex" select="count((. | preceding-sibling::drawcurve2d)[@linetype='auto'])"/>

	<ifthenelse>
      <condition>
        <equal>
            <op>
             <function-call name="TCL_GetVar">
                <inputs>
                    <parm>
                        <string><xsl:value-of select="concat('display_',generate-id())"/></string>
                    </parm>
                </inputs>
             </function-call>   
            </op>
            <op>
                <string>on</string>
            </op>
        </equal>
      </condition>
      <if>
	        <macro-call name="_plot">
		        <inputs>
			        <xsl:apply-templates select="//*[current()/@ref=@label]" mode="plot"/>
			        <parm>
				        <string>
					        <xsl:choose>
                                <xsl:when test="@color='auto'">
                                    <verb>
                                        <xsl:value-of select="exslt:node-set($colors)/color[1+(($colorindex - 1) mod last())]"/>
                                    </verb>
                                </xsl:when>
						        <xsl:when test="@color='blue'">
							        <verb>b</verb>
						        </xsl:when>
						        <xsl:when test="@color='yellow'">
							        <verb>y</verb>
						        </xsl:when>
						        <xsl:when test="@color='green'">
							        <verb>g</verb>
						        </xsl:when>
						        <xsl:when test="@color='red'">
							        <verb>r</verb>
						        </xsl:when>
						        <xsl:when test="@color='cyan'">
							        <verb>c</verb>
						        </xsl:when>
						        <xsl:when test="@color='magenta'">
							        <verb>m</verb>
						        </xsl:when>
						        <xsl:when test="@color='black'">
							        <verb>k</verb>
						        </xsl:when>
					        </xsl:choose>
					        <xsl:choose>
						        <xsl:when test="@marker='point'">
							        <verb>.</verb>
						        </xsl:when>
						        <xsl:when test="@marker='circle'">
							        <verb>o</verb>
						        </xsl:when>
						        <xsl:when test="@marker='trefoil'">
							        <verb>t</verb>
						        </xsl:when>
						        <xsl:when test="@marker='plus'">
							        <verb>+</verb>
						        </xsl:when>
						        <xsl:when test="@marker='star'">
							        <verb>*</verb>
						        </xsl:when>
						        <xsl:when test="@marker='triangle-up'">
							        <verb>^</verb>
						        </xsl:when>
						        <xsl:when test="@marker='triangle-down'">
							        <verb>v</verb>
						        </xsl:when>
						        <xsl:when test="@marker='diamond'">
							        <verb>d</verb>
						        </xsl:when>
						        <xsl:when test="@marker='filled-diamond'">
							        <verb>f</verb>
						        </xsl:when>						
					        </xsl:choose>
					        <xsl:choose>
						        <xsl:when test="@linetype='auto'">
                                        <xsl:value-of select="exslt:node-set($linetypes)/linetype[1+(($linetypeindex - 1) mod last())]"/>
						        </xsl:when>
						        <xsl:when test="@linetype='dashed'">
							        <verb>--</verb>
						        </xsl:when>
						        <xsl:when test="@linetype='dashdot'">
							        <verb>-.</verb>
						        </xsl:when>
						        <xsl:when test="@linetype='dot'">
							        <verb>.</verb>
						        </xsl:when>
						        <xsl:when test="@linetype='solid'">
							        <verb>-</verb>
						        </xsl:when>
					        </xsl:choose>
					        <verb>
						        <xsl:value-of select="@thickness"/>
					        </verb>
				        </string>
			        </parm>
			        <xsl:call-template name="add-axis-options-2d"/>
		        </inputs>
	        </macro-call>
  	        <xsl:call-template name="hold-if-necessary"/>
        </if>
    </ifthenelse>
</xsl:template>


<xsl:template match="drawcurve2d|drawcurve3d" mode="legend">
    <xsl:variable name="id" select="generate-id()"/>
    <xsl:for-each select="//*[current()/@ref=@label]">
	    <ifthenelse>
          <condition>
            <equal>
                <op>
                 <function-call name="TCL_GetVar">
                    <inputs>
                        <parm>
                            <string><xsl:value-of select="concat('display_',$id)"/></string>
                        </parm>
                    </inputs>
                 </function-call>   
                </op>
                <op>
                    <string>on</string>
                </op>
            </equal>
          </condition>
          <if>
            <assign>
                <lhs>legend_list</lhs>
                <rhs>
                    <list sep=";">
                        <parm>legend_list</parm>
                        <parm>
                            <string>
				                <xsl:value-of select="normalize-space(name[(@lang=$lang) or (not(@lang))])"/>
			                </string>
                        </parm>
                    </list>
                </rhs>
            </assign>
          </if>
        </ifthenelse>
	</xsl:for-each>
</xsl:template>

<xsl:template match="title" mode="labels">
      <macro-call name="title">
        <inputs>
            <parm>
                <xsl:call-template name="format"/>
            </parm>
        </inputs>
      </macro-call>
</xsl:template>


<xsl:template match="drawcurve2d" mode="labels">
	<xsl:apply-templates select="//*[current()/@ref=@label]" mode="labels"/>
</xsl:template>

<xsl:template match="drawcurve2d|drawcurve3d" mode="text">
	<xsl:apply-templates select="//*[current()/@ref=@label]" mode="text"/>
</xsl:template>

<xsl:template match="drawsurface" mode="text">
	<xsl:apply-templates select="//*[current()/@ref=@label]" mode="text"/>
</xsl:template>

<xsl:template match="drawsurface" mode="labels">
	<xsl:apply-templates select="//*[current()/@ref=@label]" mode="labels"/>
</xsl:template>



<xsl:template match="axis3d/drawsurface" mode="legend">
	<xsl:for-each select="//*[current()/@ref=@label]">
       <assign>
            <lhs>legend_list($+1)</lhs>
            <rhs>
                <string>
				    <xsl:value-of select="normalize-space(name[(@lang=$lang) or (not(@lang))])"/>
			    </string>
            </rhs>
        </assign>
	</xsl:for-each>
</xsl:template>

<xsl:template match="axis2d/drawsurface" mode="plot">

	<xsl:variable name="triangulation">
		<xsl:call-template name="triangulation-test"/>
	</xsl:variable>
	
	<macro-call name="_{$triangulation}pcolor">
		<inputs>
			<xsl:apply-templates select="//*[current()/@ref=@label]" mode="plot"/>
			<xsl:call-template name="add-axis-options-2d"/>
			<xsl:call-template name="add-specific-options-3d"/>			
		</inputs>
	</macro-call>

	<xsl:call-template name="hold-if-necessary"/>

</xsl:template>

<xsl:template match="axis3d/drawsurface" mode="plot">

	<xsl:variable name="triangulation">
		<xsl:call-template name="triangulation-test"/>
	</xsl:variable>

	<xsl:variable name="macroname">
		<xsl:choose>
			<xsl:when test="@mode='level'">
				<xsl:value-of select="concat('_',$triangulation,'surf')"/>
			</xsl:when>
			<xsl:when test="@mode='light'">
				<xsl:value-of select="concat('_',$triangulation,'surfl')"/>
			</xsl:when>
			<xsl:when test="@mode='wireframe'">
				<xsl:value-of select="concat('_',$triangulation,'mesh')"/>
			</xsl:when>
		</xsl:choose>
	</xsl:variable>

	<macro-call name="{$macroname}">
		<inputs>
			<xsl:apply-templates select="//*[current()/@ref=@label]" mode="plot"/>
			<xsl:call-template name="add-axis-options-3d"/>
			<xsl:call-template name="add-specific-options-3d"/>
		</inputs>
	</macro-call>

	<xsl:call-template name="hold-if-necessary"/>

</xsl:template>

<xsl:template match="drawcurve3d" mode="plot">
	<macro-call name="_plot3">
		<inputs>
			<xsl:apply-templates select="//*[current()/@ref=@label]" mode="plot"/>
			<parm>
				<string>
					<xsl:choose>
						<xsl:when test="@color='blue'">
							<verb>b</verb>
						</xsl:when>
						<xsl:when test="@color='yellow'">
							<verb>y</verb>
						</xsl:when>
						<xsl:when test="@color='green'">
							<verb>g</verb>
						</xsl:when>
						<xsl:when test="@color='red'">
							<verb>r</verb>
						</xsl:when>
						<xsl:when test="@color='cyan'">
							<verb>c</verb>
						</xsl:when>
						<xsl:when test="@color='magenta'">
							<verb>m</verb>
						</xsl:when>
						<xsl:when test="@color='black'">
							<verb>k</verb>
						</xsl:when>
					</xsl:choose>
					<xsl:choose>
						<xsl:when test="@marker='point'">
							<verb>.</verb>
						</xsl:when>
						<xsl:when test="@marker='circle'">
							<verb>o</verb>
						</xsl:when>
						<xsl:when test="@marker='trefoil'">
							<verb>t</verb>
						</xsl:when>
						<xsl:when test="@marker='plus'">
							<verb>+</verb>
						</xsl:when>
						<xsl:when test="@marker='star'">
							<verb>*</verb>
						</xsl:when>
						<xsl:when test="@marker='triangle-up'">
							<verb>^</verb>
						</xsl:when>
						<xsl:when test="@marker='triangle-down'">
							<verb>v</verb>
						</xsl:when>
						<xsl:when test="@marker='diamond'">
							<verb>d</verb>
						</xsl:when>
						<xsl:when test="@marker='filled-diamond'">
							<verb>f</verb>
						</xsl:when>						
					</xsl:choose>
					<xsl:choose>
						<xsl:when test="@linetype='dashed'">
							<verb>--</verb>
						</xsl:when>
						<xsl:when test="@linetype='dashdot'">
							<verb>-.</verb>
						</xsl:when>
						<xsl:when test="@linetype='dot'">
							<verb>.</verb>
						</xsl:when>
						<xsl:when test="@linetype='solid'">
							<verb>-</verb>
						</xsl:when>
					</xsl:choose>
					<verb>
						<xsl:value-of select="@thickness"/>
					</verb>
				</string>
			</parm>
			<xsl:call-template name="add-axis-options-3d"/>
			<xsl:call-template name="add-specific-options-3d"/>
		</inputs>
	</macro-call>

	<xsl:call-template name="hold-if-necessary"/>
	
</xsl:template>

<xsl:template match="*" mode="text"/>
<xsl:template match="*" mode="plot"/>
<xsl:template match="*" mode="labels"/>
<xsl:template match="*" mode="legend"/>


</xsl:stylesheet>
