<?xml version="1.0"?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
										xmlns:str="http://exslt.org/strings"
										extension-element-prefixes="str">

<xsl:output method="xml" indent="yes" encoding="ISO-8859-1"/>

<!-- Generation of code for the domain type elements -->

<!-- 1D domains -->

<xsl:template match="rectangle/defdomain1d" mode="script">
	<parm>
		<xsl:apply-templates select="interval|points1d"/>
	</parm>
</xsl:template>

<xsl:template match="defdomain1d[not(parent::rectangle)]" mode="script">
	<comment>
		<xsl:value-of select="name"/>
	</comment>
	<assign>
		<lhs>
			<xsl:value-of select="@label"/>
		</lhs>
		<rhs>
			<xsl:apply-templates select="interval|points1d"/>
		</rhs>
	</assign>
</xsl:template>

<xsl:template match="defdomain1d" mode="plot">
	<parm>
		<xsl:value-of select="@label"/>
	</parm>
</xsl:template>

<xsl:template match="defdomain1d" mode="labels">
	<parm>
		<string>
			<xsl:value-of select="normalize-space(name[(@lang=$lang) or (not(@lang))])"/>
		</string>
	</parm>
</xsl:template>

<xsl:template match="refdomain1d" mode="labels">
	<xsl:for-each select="//defdomain1d[@label=current()/@ref]">
		<parm>
			<string>
				<xsl:value-of select="normalize-space(name[(@lang=$lang) or (not(@lang))])"/>
			</string>
		</parm>
	</xsl:for-each>
</xsl:template>

<xsl:template match="refdomain2d" mode="labels">
    <xsl:apply-templates select="//defdomain2d[@label=current()/@ref]" mode="labels"/>
</xsl:template>

<xsl:template match="defdomain2d" mode="labels">
    <xsl:apply-templates mode="labels" select="rectangle"/>
</xsl:template>

<xsl:template match="rectangle" mode="labels">
    <parm><string>xlabel</string></parm>
    <xsl:apply-templates select="defdomain1d[1]" mode="labels"/>
    <parm><string>ylabel</string></parm>
    <xsl:apply-templates select="defdomain1d[2]" mode="labels"/>
</xsl:template>





<xsl:template match="defdomain1d" mode="functions"/>

<xsl:template match="refdomain1d" mode="script"/>

<xsl:template match="refdomain1d" mode="plot">
	<parm>
		<xsl:value-of select="@ref"/>
	</parm>
</xsl:template>

<xsl:template match="interval[@discretization='linear']">
	<function-call name="linspace">
		<inputs>
			<parm>
				<xsl:value-of select="normalize-space(initialvalue)"/>
			</parm>
			<parm>
				<xsl:value-of select="normalize-space(finalvalue)"/>
			</parm>
			<parm>
				<xsl:value-of select="@steps"/>
			</parm>
		</inputs>
	</function-call>
	<verb>'</verb>
</xsl:template>

<xsl:template match="interval[@discretization='log']">
	<function-call name="logspace">
		<inputs>
			<parm>
				<xsl:value-of select="normalize-space(initialvalue)"/>
			</parm>
			<parm>
				<xsl:value-of select="normalize-space(finalvalue)"/>
			</parm>
			<parm>
				<xsl:value-of select="@steps"/>
			</parm>
		</inputs>
	</function-call>
	<verb>'</verb>
</xsl:template>

<!-- 2D domains -->

<xsl:template match="defdomain2d" mode="script">
	<xsl:apply-templates mode="script"/>
</xsl:template>

<xsl:template match="defdomain2d">
	<xsl:apply-templates mode="script"/>
</xsl:template>

<xsl:template match="border" mode="script">
	<macro-call name="fem_initdomain"/>
	<xsl:apply-templates/>
	<assign>
		<lhs><xsl:value-of select="../@label"/></lhs>
		<rhs>
			<function-call name="fem_meshdomain"/>
		</rhs>
	</assign>		
</xsl:template>

<xsl:template match="border/parametriccurve2d">
	<assign>
		<lhs><xsl:value-of select="@label"/></lhs>
		<rhs>
			<function-call name="fem_border">
			<inputs>
				<parm>	
					<string><xsl:value-of select="x1/value"/></string>
				</parm>
				<parm>	
					<string><xsl:value-of select="x2/value"/></string>
				</parm>
				<xsl:for-each select="defdomain1d | //defdomain1d[@label=current()/refdomain1d/@ref]">
					<parm>
						<string>
                        	<xsl:value-of select="concat(@label,'=',interval/initialvalue,',',interval/finalvalue)"/>
						</string>
					</parm>
					<parm>
						<xsl:value-of select="interval/@steps"/>
					</parm>
				</xsl:for-each>
			</inputs>
			</function-call>
		</rhs>
	</assign>
</xsl:template>



<xsl:template match="rectangle" mode="script">
	<assign>
		<lhs>
			<list>
				<xsl:for-each select="defdomain1d">
					<parm>
						<xsl:value-of select="@label"/>
					</parm>
				</xsl:for-each>
			</list>		
		</lhs>
		<rhs>
			<function-call name="meshgrid">
				<inputs>
					<xsl:apply-templates select="defdomain1d" mode="script"/>
				</inputs>
			</function-call>
		</rhs>
	</assign>
</xsl:template>

<xsl:template match="rectangle" mode="plot">
	<parm>
		<xsl:value-of select="defdomain1d[1]/@label"/>
	</parm>
	<parm>
		<xsl:value-of select="defdomain1d[2]/@label"/>
	</parm>
</xsl:template>


<xsl:template match="refdomain2d" mode="plot">
	<xsl:for-each select="//defdomain2d[@label=current()/@ref]">
		<xsl:choose>
			<xsl:when test="rectangle">
				<xsl:apply-templates select="rectangle" mode="plot"/>
			</xsl:when>
			<xsl:when test="border">
				<parm>
					<xsl:value-of select="concat(@label,'.tri')"/>
				</parm>
				<parm>
					<xsl:value-of select="concat(@label,'.x')"/>
				</parm>
				<parm>
					<xsl:value-of select="concat(@label,'.y')"/>
				</parm>			
			</xsl:when>
			<xsl:otherwise/>
		</xsl:choose>
	</xsl:for-each>
</xsl:template>

<xsl:template match="defdomain2d" mode="functions"/>

</xsl:stylesheet>
