<?xml version="1.0"?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:output method="xml" indent="yes" encoding="ISO-8859-1"/>

<xsl:strip-space elements="*"/>

<xsl:template match="implicitfunction" mode="script">
  <xsl:call-template name="implicitfunction"/>
</xsl:template>

<xsl:template match="implicitfunction">
  <xsl:call-template name="implicitfunction"/>
</xsl:template>


<xsl:template name="implicitfunction">
	
	<comment>Script code for the <xsl:value-of select="@label"/> implicitfunction</comment>

	<xsl:apply-templates select="unknowns" mode="script"/>

	<assign>
		<lhs>
			<parm>_X</parm>
		</lhs>
		<xsl:choose>
			<xsl:when test="@method='newton'">
				<rhs>
					<function-call name="fsolve">
						<inputs>
							<parm>_X0</parm>
							<parm>f_<xsl:value-of select="@label"/></parm>
						</inputs>
					</function-call>
				</rhs>
			</xsl:when>
			<xsl:when test="@method='dichotomic'">
				<rhs>
					<function-call name="fdichot">
						<inputs>
							<parm>_X0</parm>
							<parm>_XLOW</parm>
							<parm>_XUP</parm>
							<parm>f_<xsl:value-of select="@label"/></parm>
						</inputs>
					</function-call>
				</rhs>				
			</xsl:when>
		</xsl:choose>
	</assign>
	
	<!-- Creation of outputs (states) -->

	<xsl:for-each select="unknowns/unknown">
		<assign>
			<lhs>
				<xsl:value-of select="@label"/>
			</lhs>
			<rhs>
				<select	matrix="_X" row=":" col='{position()}'/>
			</rhs>
		</assign>
	</xsl:for-each>


	<!-- Re-excution of auxilliary code fragment (for eventual use in outputs) -->

	<xsl:for-each select="equations/variable">	
		<assign>
			<lhs>
				<xsl:value-of select="@label"/>
			</lhs>
			<rhs>
				<xsl:value-of select="normalize-space(.)"/>
			</rhs>
		</assign>
	</xsl:for-each>


	<!-- Creation of outputs (others) -->

	<xsl:apply-templates select="outputs"/>

</xsl:template>

<xsl:template match="implicitfunction/unknowns" mode="script">
	<assign>
		<lhs>
			<verb>_X0</verb>
		</lhs>
		<rhs>
			<list sep=",">
				<xsl:for-each select="unknown">
					<parm>
						<xsl:value-of select="guess"/>
						<verb>*</verb>
						<function-call name="ones">
							<inputs>
								<parm>
									<xsl:value-of select="ancestor::implicitfunction/refdomain1d/@ref"/>
								</parm>
							</inputs>
						</function-call>
					</parm>
				</xsl:for-each>
			</list>
		</rhs>
	</assign>
	<xsl:if test="../@method='dichotomic'">
		<assign>
			<lhs>
				<verb>_XLOW</verb>
			</lhs>
			<rhs>
				<list sep=",">
					<xsl:for-each select="unknown">
						<parm>
							<xsl:value-of select="lowerbound"/>
							<verb>*</verb>
							<function-call name="ones">
								<inputs>
									<parm>
										<xsl:value-of select="ancestor::implicitfunction/refdomain1d/@ref"/>
									</parm>
								</inputs>
							</function-call>
						</parm>
					</xsl:for-each>
				</list>
			</rhs>
		</assign>	
		<assign>
			<lhs>
				<verb>_XUP</verb>
			</lhs>
			<rhs>
				<list sep=",">
					<xsl:for-each select="unknown">
						<parm>
							<xsl:value-of select="upperbound"/>
							<verb>*</verb>
							<function-call name="ones">
								<inputs>
									<parm>
										<xsl:value-of select="ancestor::implicitfunction/refdomain1d/@ref"/>
									</parm>
								</inputs>
							</function-call>
						</parm>
					</xsl:for-each>
				</list>
			</rhs>
		</assign>	
	</xsl:if>
</xsl:template>

<xsl:template match="implicitfunction" mode="functions">

        <comment>
                <xsl:value-of select="concat('Function for the rhs of the ',@label,' implicitfunction')"/>
        </comment>

        <function-definition name="f_{@label}">
                <inputs>
                        <parm><verb>_X</verb></parm>
                </inputs>
                <outputs>
                        <parm><verb>lhs</verb></parm>
                </outputs>
                <body>
                        <xsl:apply-templates select="unknowns" mode="functions"/>
						<xsl:apply-templates select="equations" mode="functions"/>
                </body>
        </function-definition>

</xsl:template>


<xsl:template match="unknowns" mode="functions">

	<xsl:for-each select="unknown">
		<assign>
			<lhs>
				<xsl:value-of select="@label"/>
			</lhs>
			<rhs>
				<select matrix="_X" row=":" col="{position()}"/>
			</rhs>
		</assign>
	</xsl:for-each>
	
</xsl:template>


<xsl:template match="equations" mode="functions">
	
	<xsl:for-each select="variable">	
		<assign>
			<lhs>
				<xsl:value-of select="@label"/>
			</lhs>
			<rhs>
				<xsl:value-of select="normalize-space(.)"/>
			</rhs>
		</assign>
	</xsl:for-each>
	
	<assign>
			<lhs>lhs</lhs>
			<rhs>
				<list sep=",">
					<xsl:for-each select="equation">
						<parm>
							<xsl:value-of select="normalize-space(.)"/>
						</parm>
					</xsl:for-each>
				</list>
			</rhs>
	</assign>
	
</xsl:template>

<xsl:template match="implicitfunction/unknowns/unknown | implicitfunction/outputs/output" mode="plot">
	<parm>
		<xsl:value-of select="../../refdomain1d/@ref"/>
	</parm>
	<parm>
		<xsl:value-of select="@label"/>
	</parm>
</xsl:template>


<xsl:template match="implicitfunction/unknowns/unknown | implicitfunction/outputs/output" mode="labels">
	<xsl:variable name="ref" select="../../refdomain1d/@ref"/>
	<macro-call name="_xlabel">
		<inputs>
			<xsl:for-each select="//defdomain1d[@label=$ref]">
				<parm>
					<string><xsl:value-of select="normalize-space(name[(@lang=$lang) or not(@lang)])"/></string>
					<xsl:if test="@unit">
						<xsl:text>+</xsl:text>
						<string> (<xsl:value-of select="@unit"/>)</string>
					</xsl:if> 
				</parm>
			</xsl:for-each>
		</inputs>
	</macro-call>
	<macro-call name="_ylabel">
		<inputs>
		<parm>
			<string>
				<xsl:value-of select="@unit"/>
			</string>
		</parm>
		</inputs>
	</macro-call>
</xsl:template>

</xsl:stylesheet>
