<?xml version="1.0"?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:param name="file"/>

<xsl:output method="xml" indent="yes" encoding="ISO-8859-1"/>

<!-- Generation of code for the "ode" elements -->

<xsl:template match="ode" mode="functions">

	<comment>
		<xsl:value-of select="concat('Function for the rhs of the ',@label,' ode')"/>
	</comment>
	
	<function-definition name="f_{@label}">
		<inputs>
			<parm>
            	<verb>_t</verb>
            </parm>
			<parm>
            	<verb>_X</verb>
            </parm>
		</inputs>
		<outputs>
			<parm>
            	<verb>lhs</verb>
            </parm>
		</outputs>
		<body>
			<assign>
				<lhs>
					<xsl:value-of select="refdomain1d/@ref"/>
					<xsl:value-of select="defdomain1d/@label"/>
				</lhs>
				<rhs>_t</rhs>
			</assign>
			<xsl:apply-templates select="states" mode="functions"/>
		</body>
	</function-definition>
	  
	<macro-call name="execstr"> <!-- size matters... -->
	    <inputs>
	      <parm><string>clear _X0</string></parm>
	      <parm><string>errcatch</string></parm>
	    </inputs>
	</macro-call>
</xsl:template>


<xsl:template match="ode" mode="script">
	<xsl:call-template name="ode"/>
</xsl:template>

<xsl:template match="ode">
	<xsl:call-template name="ode"/>
</xsl:template>


<xsl:template name="ode">
	
	<comment>
		<xsl:value-of select="concat('Script code for the ',@label,' ode')"/>
	</comment>

	<xsl:apply-templates select="states" mode="script"/>

	<assign>
		<lhs>
			<parm>_X</parm>
		</lhs>
		<rhs>
			<function-call name="ode">
				<inputs>
					<parm>_X0</parm>
					<parm>0</parm>
					<parm>
						<xsl:value-of select="refdomain1d/@ref"/>
					</parm>
					<parm>
						<xsl:value-of select="concat('f_',@label)"/>
					</parm>
				</inputs>
			</function-call>
		</rhs>
	</assign>

	<!-- Creation of outputs (states) -->

	<xsl:for-each select="states/state">
		<assign>
			<lhs>
				<xsl:value-of select="@label"/>
			</lhs>
			<rhs>
				<select	matrix="_X" row="{1+sum(preceding-sibling::state/@size)}:{@size+sum(preceding-sibling::state/@size)}" col=':'/>
				<verb>'</verb>
			</rhs>
		</assign>
	</xsl:for-each>

	<!-- Creation of outputs (others) -->

	<xsl:apply-templates select="outputs"/>

</xsl:template>

<xsl:template match="ode/states" mode="script">
	<xsl:for-each select="state">
		<assign>
			<lhs>
				<select matrix="_X0" row="{1+sum(preceding-sibling::state/@size)}:{@size+sum(preceding-sibling::state/@size)}" col="1"/>
			</lhs>
			<rhs>
				<xsl:value-of select="initialcondition"/>
			</rhs>
		</assign>
	</xsl:for-each>
</xsl:template>

<xsl:template match="ode/states" mode="functions">

	<xsl:for-each select="state">
		<assign>
			<lhs>
				<xsl:value-of select="@label"/>
			</lhs>
			<rhs>
				<select matrix="_X" row="{1+sum(preceding-sibling::state/@size)}:{@size+sum(preceding-sibling::state/@size)}" col="1"/>
			</rhs>
		</assign>
	</xsl:for-each>

	<xsl:for-each select="variable">	
		<assign>
			<lhs>
				<xsl:value-of select="@label"/>
			</lhs>
			<rhs>
				<xsl:value-of select="normalize-space(.)"/>
			</rhs>
		</assign>
	</xsl:for-each>

	<assign>
			<lhs>lhs</lhs>
			<rhs>
				<list sep=";">
					<xsl:for-each select="state">
						<parm>
							<xsl:value-of select="concat('(',normalize-space(derivative),')')"/>
						</parm>
					</xsl:for-each>
				</list>
			</rhs>
	</assign>
	
</xsl:template>

<xsl:template match="ode/states/state | ode/outputs/output" mode="plot">
	<parm>
		<xsl:value-of select="../../refdomain1d/@ref"/>
	</parm>
	<parm>
		<xsl:value-of select="@label"/>
	</parm>
</xsl:template>

<xsl:template match="ode/states/state | ode/outputs/output" mode="labels">
	<xsl:variable name="ref" select="../../refdomain1d/@ref"/>
	<macro-call name="_xlabel">
		<inputs>
			<xsl:for-each select="//defdomain1d[@label=$ref]">
				<parm>
					<concat>
						<arg>
							<string>
								<xsl:value-of select="normalize-space(name[(@lang=$lang) or not(@lang)])"/>
							</string>
						</arg>
						<arg>
							<string>
								<xsl:if test="@unit">
									<xsl:value-of select="concat('(',@unit,')')"/>
								</xsl:if>
							</string>
						</arg>
					</concat> 				
				</parm>
			</xsl:for-each>
		</inputs>
	</macro-call>
	<macro-call name="_ylabel">
		<inputs>
		<parm>
			<string>
				<xsl:value-of select="@unit"/>
			</string>
		</parm>
		</inputs>
	</macro-call>
</xsl:template>

</xsl:stylesheet>
