<?xml version="1.0"?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
										xmlns:str="http://exslt.org/strings"
										extension-element-prefixes="str">
<xsl:output method="xml" indent="yes" encoding="ISO-8859-1"/>

<xsl:strip-space elements="*"/>


<xsl:template match="stationary-pde" mode="functions">
	<xsl:apply-templates mode="functions"/>
</xsl:template>
	
<xsl:template name="assemblydom">
	<xsl:for-each select="*">
		<assign>
			<lhs>
				<xsl:value-of select="concat(../../../@label,'.',name(.))"/>
			</lhs>
			<rhs>
				<xsl:choose>
					<xsl:when test="contains(.,'x') or contains(.,'y')">
						<xsl:value-of select="generate-id(.)"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="."/>
					</xsl:otherwise>
				</xsl:choose>			
			</rhs>
		</assign>
	</xsl:for-each>
	<assign>
		<lhs>
			<list>
				<parm>_A</parm>
				<parm>_b</parm>
				<parm>_M</parm>
			</list>
		</lhs>
		<rhs>
			<function-call name="assemblyDom">
				<inputs>
					<parm><xsl:value-of select="../../@label"/></parm>
				</inputs>
			</function-call>
		</rhs>
	</assign>
</xsl:template>

<xsl:template name="solve">
	<assign>
		<lhs><parm><xsl:value-of select="@label"/></parm></lhs>
		<rhs>(_A+_Ab)\(_b+_bb)</rhs>
	</assign>
</xsl:template>

<xsl:template match="stationary-pde" mode="script">
	<xsl:apply-templates mode="script"/>
</xsl:template>


<xsl:template match="stationary-pde/refdomain2d" mode="script">
	<assign>
		<lhs>
			<xsl:value-of select="../@label"/>
		</lhs>
		<rhs>
			<function-call name="fem_newproblem">
				<inputs>
					<parm>
						<xsl:value-of select="@ref"/>
					</parm>
				</inputs>
			</function-call>
		</rhs>
	</assign>
</xsl:template>

<xsl:template match="stationary-pde/pdestate" mode="script">
	<xsl:apply-templates mode="script"/>
	<xsl:call-template name="solve"/>
</xsl:template>


<xsl:template match="stationary-pde/pdestate" mode="functions">
	<xsl:apply-templates select="inside" mode="functions"/>
	<xsl:apply-templates select="boundary" mode="functions"/>
</xsl:template>

<xsl:template match="inside" mode="script">
	<xsl:call-template name="assemblydom"/>
</xsl:template>

<xsl:template match="inside">
	<xsl:call-template name="assemblydom"/>
</xsl:template>

<xsl:template match="stationary-pde/pdestate/inside" mode="functions">
	<xsl:for-each select="*[contains(.,'x') or contains(.,'y')]">
		<function-definition name="{generate-id(.)}">
			<outputs>
            	<parm>z</parm>
			</outputs>
			<inputs>
            	<parm>x</parm>
                <parm>y</parm>
			</inputs>
			<body>
				<assign>
					<lhs>z</lhs>
					<rhs>
						<xsl:value-of select="."/>
					</rhs>
				</assign>
			</body>
		</function-definition>
	</xsl:for-each>
</xsl:template>

<xsl:template match="stationary-pde/pdestate/boundary" mode="functions">
	<xsl:apply-templates mode="functions"/>
</xsl:template>

<xsl:template match="stationary-pde/pdestate/boundary" mode="script">
	<xsl:apply-templates mode="script"/>
	<assign>
		<lhs>
			<list>
				<parm>_Ab</parm>
				<parm>_bb</parm>
			</list>
		</lhs>
		<rhs>
			<function-call name="assemblyBdy">
				<inputs>
					<parm><xsl:value-of select="../../@label"/></parm>
				</inputs>
			</function-call>
		</rhs>
	</assign>
</xsl:template>


<xsl:template match="stationary-pde/pdestate/boundary/condition" mode="script">
	<xsl:variable name="bdy">
		<xsl:value-of select="concat(../../../@label,'.bdycond(',@bdy,')')"/>
	</xsl:variable>

	<assign>
		<lhs>
			<xsl:value-of select="concat($bdy,'.type')"/>
		</lhs>
		<rhs>
			<string>
				<xsl:value-of select="name(*)"/>
			</string>
		</rhs>
	</assign>
	<xsl:apply-templates mode="script">
		<xsl:with-param name="bdy" select="$bdy"/>
	</xsl:apply-templates>
</xsl:template>

<xsl:template match="stationary-pde/pdestate/boundary/condition" mode="functions">
	<xsl:apply-templates mode="functions"/>
</xsl:template>


<xsl:template name="fbdy">
	<function-definition name="{generate-id(.)}">
		<outputs>	
        	<parm>z</parm>
		</outputs>
		<inputs>
        	<parm>x</parm>
            <parm>y</parm>
		</inputs>
		<body>
			<assign>
				<lhs>z</lhs>
				<rhs>
					<xsl:value-of select="."/><xsl:text>+0*x</xsl:text>
				</rhs>
			</assign>
		</body>
	</function-definition>
</xsl:template>

<xsl:template match="dirichlet[contains(.,'x') or contains(.,'y')]" mode="functions">
	<xsl:call-template name="fbdy"/>	
</xsl:template>

<xsl:template match="dirichlet" mode="functions"/>


<xsl:template match="dirichlet" mode="script">
	<xsl:param name="bdy"/>		
	<assign>
		<lhs>
			<xsl:value-of select="concat($bdy,'.g')"/>
		</lhs>
		<rhs>
				<xsl:choose>
					<xsl:when test="contains(.,'x') or contains(.,'y')">
						<xsl:value-of select="generate-id(.)"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="."/>
					</xsl:otherwise>
				</xsl:choose>			
		</rhs>
	</assign>
</xsl:template>


<xsl:template match="robin[contains(*,'x') or contains(*,'y')]" mode="functions">
	<xsl:for-each select="*">
		<xsl:call-template name="fbdy"/>
	</xsl:for-each>
</xsl:template>

<xsl:template match="robin" mode="functions"/>


<xsl:template match="robin" mode="script">
	<xsl:param name="bdy"/>		
	<xsl:for-each select="*">
		<assign>
			<lhs>
				<xsl:value-of select="concat($bdy,'.',name(.))"/>
			</lhs>
			<rhs>
				<xsl:choose>
					<xsl:when test="contains(.,'x') or contains(.,'y')">
						<xsl:value-of select="generate-id(.)"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="."/>
					</xsl:otherwise>
				</xsl:choose>			
			</rhs>
		</assign>
	</xsl:for-each>
</xsl:template>


<xsl:template match="pdestate" mode="plot">
	<xsl:apply-templates select="../refdomain2d" mode="plot"/>
	<parm>
    	<xsl:value-of select="@label"/>
    </parm>
</xsl:template>

<xsl:template match="pdestate" mode="text">
	<macro-call name="title">
		<inputs>
			<parm>
				<string>
					<xsl:value-of select="name[(@lang=$lang) or not(@lang)]"/>
				</string>
			</parm>
		</inputs>
	</macro-call>	
</xsl:template>



<xsl:template match="*" mode="functions"/>
<xsl:template match="*" mode="script"/>


</xsl:stylesheet>
