<?xml version="1.0"?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:str="http://exslt.org/strings"
xmlns:exslt="http://exslt.org/common"
exclude-result-prefixes="str exslt">

<xsl:output method="xml" indent="yes" encoding="ISO-8859-1"/>

<xsl:param name="file"/>
<xsl:param name="dir"/>
<xsl:param name="date"/>


<xsl:include href="localization.xsl"/>


<xsl:strip-space elements="*"/>

<xsl:template match="/">
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="simulation">
	<xsl:for-each select="header/title">
    	<xsl:apply-templates select="../.." mode="single-lang">
        	<xsl:with-param name="lang" select="@lang"/>
        </xsl:apply-templates >
    </xsl:for-each>
</xsl:template>

<xsl:template match="simulation" mode="single-lang">
	<xsl:param name="lang"/>
    <xsl:param name="file" select="$file"/>
	<xsl:param name="refererUrl"/>
	<xsl:param name="refererText"/>
    
    <xsl:document href="{concat($file,'/',$file,'_',$lang,'.html')}" method="html" indent="yes" encoding="ISO-8859-1">
	    <html>
		    <head>
        	    <title>
            	    <xsl:value-of select="normalize-space(header/title[@lang=$lang])"/>
                </title>
                <link href="../../xmllab.css" rel="stylesheet" type="text/css"/>
            </head>
   			<body marginwidth="10" marginheight="10">
                <table border="0" style="width: 100%;" cellpadding="0" cellspacing="0">
                  <tbody>
                    <tr>
                      <td style="align: left; width: 48%;">
                        <b><font size="+3"><xsl:value-of select="normalize-space(header/title[@lang=$lang])"/> </font></b>
                      </td>
                      <td style="align: left; width: 2%;"/>
                      <td style="align: center; width: 25%;">
                          <xsl:if test="$refererUrl">
                          	    <b><a href="{$refererUrl}"><xsl:value-of select="$refererText"/></a></b>
                          </xsl:if>
                      </td>
                      <td style="align: right; width: 25%;">
                      <table border="0" style="width: 100%;" cellpadding="0" cellspacing="0">
                      <tbody>
                      <tr>
					  <xsl:for-each select="header/title[@lang!=$lang]">
                          <td style="vertical-align: top; text-align: right;">
                              <div style="text-align: center;">
                          	    <img align="middle" hspace="5"
                     					    src="{concat(@lang,'_flag.png')}"/><br/>
                              </div>
                              <div style="text-align: center;">
                                <a href="{concat($file,'_',@lang,'.html')}">
                                  	<xsl:value-of select="exslt:node-set($strings)/strings[@lang=current()/@lang]/@versiontext"/>
                                </a>
                                <br/>
                              </div>
                          </td>
                       </xsl:for-each>
                       </tr>
                       </tbody>
                       </table>
                       </td>
	                 </tr>
                  </tbody>
                </table>
			    <h2>
            	    <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@author"/> : <xsl:value-of select="header/author"/>
               </h2>
               <h3>
                	<a href="{concat($file,'.html')}">
                    	<xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@viewsource"/>
                    </a>
                    &#xA0;
                	<a href="{concat($file,'.xml')}">
                    	<xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@downloadsource"/>
                    </a>
                </h3>
                <h2>Description</h2>
        	    <xsl:for-each select="notes[@lang=$lang]/p">
            	    <xsl:copy-of select="."/>
                </xsl:for-each>
                <h3>
                    <a type="application/x-xmllab" href="{concat($file,'.xmllab')}">
                        <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@rundesc"/>
                    </a>
                </h3>
				<h2><xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@screenshots"/></h2>
                <p align="left">
                    <a href="{concat($file,'_',$lang,'_0.png')}">
                        <img hspace="12" border="1" src="{concat($file,'_',$lang,'_0_small.png')}" align="middle"/>
                    </a>
                    <xsl:for-each select="display/window">
						<a href="{concat($file,'_',$lang,'_',position(),'.png')}">
	                        <img hspace="12"  border="1" src="{concat($file,'_',$lang,'_',position(),'_small.png')}" align="middle"/>
						</a>
                    </xsl:for-each>
                </p	>
                <p><hr/><small><i>
                <xsl:for-each select="exslt:node-set($strings)/credits[@lang=$lang]">
                    <xsl:for-each select="comment">
                        <xsl:value-of select="."/><br/>
                    </xsl:for-each>
                </xsl:for-each>
                </i></small></p>
				<p><xsl:value-of select="$date"/></p>
                <p><b><a href="http://xmllab.org">http://xmllab.org</a>
                </b></p>
            </body>
	    </html>
	</xsl:document>
</xsl:template>


</xsl:stylesheet>
