<?xml version="1.0" encoding="ISO-8859-1"?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:exslt="http://exslt.org/common">

<xsl:param name="file"/>
<xsl:param name="dir"/>
<xsl:param name="date"/>

<xsl:include href="publish_html2.xsl"/>

<xsl:output method="html" indent="yes" encoding="ISO-8859-1"/>

<xsl:strip-space elements="*"/>

<xsl:template match="/">
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="publish">
	<xsl:for-each select="title">
    	<xsl:apply-templates select=".." mode="single-lang">
        	<xsl:with-param name="lang" select="@lang"/>
        </xsl:apply-templates>
    </xsl:for-each>
</xsl:template>

<xsl:template match="publish" mode="single-lang">
	<xsl:param name="lang"/>
    <xsl:document href="{concat($file,'/index_',$lang,'.html')}" method="html" indent="yes" encoding="ISO-8859-1">
    	<html>
	        <head>
                <title>
                    <xsl:value-of select="normalize-space(title[@lang=$lang])"/>
                </title>
		       <link href="../xmllab.css" rel="stylesheet" type="text/css"/>
            </head>
            <body marginwidth="10" marginheight="10">
                    <table border="0" style="width: 100%;" cellpadding="0" cellspacing="0">
                      <tbody>
                        <tr>
                          <td style="width: 90%;">
                            <b><font size="+3" ><xsl:value-of select="normalize-space(title[@lang=$lang])"/> </font></b>
                          </td>
                          <div id="Zoneflag">
			              <xsl:for-each select="title[@lang!=$lang]">
                              <td style="vertical-align: top; text-align: right;">
                                  <div style="text-align: center;">
                                    <img align="middle" hspace="5"
                     				            src="{concat(@lang,'_flag.png')}"/><br/>
                                  </div>
                                  <div style="text-align: center;">
                                    <a href="{concat('index_',@lang,'.html')}">
                                        <xsl:value-of select="exslt:node-set($strings)/strings[@lang=current()/@lang]/@versiontext"/>
                                    </a>
                                    <br/>
                                  </div>
                              </td>
                           </xsl:for-each>
                           </div>
	                     </tr>
                      </tbody>
                    </table>
	            <ul>
		            <xsl:apply-templates select="*[not(self::title)]">
						<xsl:with-param name="lang" select="$lang"/>
                    </xsl:apply-templates>
	            </ul>
         	    <xsl:apply-templates select="exslt:node-set($strings)/xmllab-notice[@lang=$lang]"/>
                <xsl:apply-templates select="exslt:node-set($strings)/publishlistcredits[@lang=$lang]"/>
				<p><xsl:value-of select="$date"/></p>
	        </body>
	    </html>
    </xsl:document>
</xsl:template>

<xsl:template match="publish/title"/>

<xsl:template match="xmllab-notice">
	<p/>
   	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="publishlistcredits">
	<hr/>
	<small>
	<i>
    	<xsl:apply-templates/>
    </i>
    </small>
</xsl:template>


<xsl:template match="p">
	<xsl:copy-of select="."/>
</xsl:template>


<xsl:template match="category">
	<xsl:param name="lang"/>
	<li>
		<a name="{generate-id()}"/>
    	<xsl:value-of select="name[@lang=$lang]"/>
		<ul>
        	<xsl:apply-templates>
				<xsl:with-param name="lang" select="$lang"/>
            </xsl:apply-templates>       
        </ul>
	</li>
</xsl:template>


<xsl:template match="simulations">
	<xsl:param name="lang"/>
	<xsl:apply-templates>
		<xsl:with-param name="lang" select="$lang"/>
    </xsl:apply-templates>
</xsl:template>

<xsl:template match="simulations/simulation">
	<xsl:param name="lang"/>
	<xsl:if test="document(concat(ancestor::category/@dir,'/',@href),/)/simulation/header/title[@lang=$lang]">
	    <li>
		    <a href="{concat(substring-before(@href,'.'),'/',substring-before(@href,'.'),'_',$lang,'.html')}">
	    	    <xsl:value-of select="normalize-space(document(concat(ancestor::category/@dir,'/',@href),/)/simulation/header/title[@lang=$lang])"/>
		    </a>
        <xsl:apply-templates select="document(concat(ancestor::category/@dir,'/',@href),/)/simulation" mode="single-lang">
				  <xsl:with-param name="lang" select="$lang"/>
          <xsl:with-param name="file" select="substring-before(@href,'.')"/>
				  <xsl:with-param name="refererUrl" select="concat('../index_',$lang,'.html','#',generate-id(ancestor::category))"/>
				  <xsl:with-param name="refererText" select="concat(exslt:node-set($strings)/strings[@lang=$lang]/@backtocategory,' (',ancestor::category/name[@lang=$lang],')')"/>
        </xsl:apply-templates>
	    </li>       
    </xsl:if>
</xsl:template>

<xsl:template match="text()"/>


</xsl:stylesheet>


