<?xml version="1.0"?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:str="http://exslt.org/strings"
exclude-result-prefixes="str">

<xsl:output method="text" indent="no" encoding="ISO-8859-1"/>
  <xsl:strip-space elements="*"/>

<xsl:template match="/">
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="sciml">
<!--    <xsl:text>lines(0)&#xA;</xsl:text>-->
	<xsl:apply-templates select="function-definition"/>
	<xsl:apply-templates select="node()[not (self::function-definition)]"/>
</xsl:template>


<xsl:template match="assign">
	<xsl:apply-templates select="lhs"/>
	<xsl:text>=</xsl:text>
	<xsl:apply-templates select="rhs"/>
	<xsl:text>;&#xA;</xsl:text>	
</xsl:template>

<xsl:template match="lhs">
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="rhs">
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="select">
	<xsl:value-of select="concat(@matrix,'(',@row,',',@col,')')"/>
</xsl:template>

<xsl:template match="function-definition">
	<xsl:text>function </xsl:text>
	<xsl:if test="outputs">
		<xsl:apply-templates select="outputs"/>
		<xsl:text>=</xsl:text>
	</xsl:if>
	<xsl:value-of select="@name"/>
	<xsl:text>(</xsl:text>
	<xsl:apply-templates select="inputs"/>
	<xsl:text>)&#xA;</xsl:text>	
	<xsl:apply-templates select="body"/>
	<xsl:text>&#xA;endfunction&#xA;&#xA;</xsl:text>
</xsl:template>

<xsl:template match="function-call">
	<xsl:value-of select="@name"/>
	<xsl:text>(</xsl:text>
	<xsl:apply-templates select="inputs"/>
	<xsl:text>)</xsl:text>	
</xsl:template>

<xsl:template match="macro-call">
	<xsl:value-of select="@name"/>
	<xsl:if test="inputs">
	    <xsl:text>(</xsl:text>
	    <xsl:apply-templates select="inputs"/>
	    <xsl:text>);</xsl:text>	
	</xsl:if>
	<xsl:text>&#xA;</xsl:text>	
</xsl:template>

<xsl:template match="inputs">
	<xsl:for-each select="parm">
		<xsl:apply-templates/>
		<xsl:if test="position()!=last()">
			<xsl:text>,</xsl:text>
		</xsl:if>
	</xsl:for-each>
</xsl:template>

<xsl:template match="body">
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="string">
	<xsl:text>"</xsl:text>
	<xsl:apply-templates/>
	<xsl:text>"</xsl:text>
</xsl:template>

<xsl:template match="string/text()">
	<xsl:variable name="apos">&apos;</xsl:variable>
	<xsl:variable name="str">
        <xsl:for-each select="str:split(.,'&quot;')">
	        <xsl:value-of select="."/>
            <xsl:if test="position()&lt;last()">
                <xsl:text>&quot;&quot;</xsl:text>
            </xsl:if>
        </xsl:for-each>
	</xsl:variable>
    <xsl:for-each select="str:split($str,$apos)">
	    <xsl:value-of select="."/>
        <xsl:if test="position()&lt;last()">
            <xsl:text>&apos;&apos;</xsl:text>
        </xsl:if>
    </xsl:for-each>
</xsl:template>


<xsl:template match="list|outputs">
	<xsl:text>[</xsl:text>
	<xsl:for-each select="parm">
		<xsl:apply-templates/>
		<xsl:if test="position()!=last()">
			<xsl:choose>
				<xsl:when test="../@sep">
					<xsl:value-of select="../@sep"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>,</xsl:text>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:if>
	</xsl:for-each>
	<xsl:text>]</xsl:text>
</xsl:template>

<xsl:template match="verb">
	<xsl:value-of select="normalize-space(.)"/>
</xsl:template>

<xsl:template match="comment">
	<xsl:value-of select="concat('// ',normalize-space(.),'&#xA;')"/>
</xsl:template>

<xsl:template match="line">
	<xsl:value-of select="concat(normalize-space(.),'&#xA;')"/>
</xsl:template>

<xsl:template match="ifthenelse">
	<xsl:text>if </xsl:text>
	<xsl:apply-templates/>
	<xsl:text>end&#xA;</xsl:text>
</xsl:template>

<xsl:template match="condition">
  <xsl:text>(</xsl:text>
	<xsl:apply-templates/>
  <xsl:text>)</xsl:text>
</xsl:template>

<xsl:template match="not">
  <xsl:text>~(</xsl:text>
	<xsl:apply-templates/>
  <xsl:text>)</xsl:text>  
</xsl:template>

<xsl:template match="or">
  <xsl:for-each select="arg">
    <xsl:text>(</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>)</xsl:text>
    <xsl:if test="position()&lt;last()">
      <xsl:text>|</xsl:text>
    </xsl:if>
  </xsl:for-each>
</xsl:template>

<xsl:template match="concat">
  <xsl:for-each select="arg">
    <xsl:apply-templates/>
    <xsl:if test="position()&lt;last()">
      <xsl:text>+</xsl:text>
    </xsl:if>
  </xsl:for-each>
</xsl:template>

<xsl:template match="equal">
    <xsl:apply-templates select="op[1]"/> 
    <xsl:text>==</xsl:text>   
    <xsl:apply-templates select="op[2]"/> 
</xsl:template>

<xsl:template match="op">
	<xsl:apply-templates/>
</xsl:template>


<xsl:template match="if">
	<xsl:text>&#xA;</xsl:text>
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="else">
	<xsl:value-of select="'&#xA;else&#xA;'"/>
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="switch">
	<xsl:text>select </xsl:text>
	<xsl:apply-templates select="@expr"/>
	<xsl:apply-templates select="case"/>
	<xsl:text>&#xA;end&#xA;</xsl:text>
</xsl:template>

<xsl:template match="case">
	<xsl:text>&#xA;case </xsl:text>
	<xsl:value-of select="@value"/>
	<xsl:text> then&#xA;</xsl:text>
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="while">
	<xsl:text>while (</xsl:text>
	<xsl:apply-templates select="condition"/>
	<xsl:text>)&#xA;</xsl:text>
	<xsl:apply-templates select="do"/>
	<xsl:text>&#xA;end&#xA;</xsl:text>
</xsl:template>

<xsl:template match="for">
    <xsl:value-of select="concat('for ',@variable,'=',@min,':',@step,':',@max,'&#xA;')"/>
    <xsl:apply-templates select="do"/>
	<xsl:text>&#xA;end&#xA;</xsl:text>
</xsl:template>

<xsl:template match="do">
	<xsl:apply-templates/>
</xsl:template>


<xsl:template match="case">
	<xsl:text>&#xA;case </xsl:text>
	<xsl:value-of select="@value"/>
	<xsl:text> then&#xA;</xsl:text>
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="command">
	<xsl:value-of select="concat(.,'&#xA;')"/>
</xsl:template>

</xsl:stylesheet>


