<?xml version="1.0"?>

<xsl:stylesheet version="1.0" 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:str="http://exslt.org/strings"
   xmlns:tcl="http://xmllab.org/Tcl" >

<xsl:param name="file"/>
<xsl:param name="dirname"/>
<xsl:param name="web"/>
<xsl:param name="screenshots"/>
<xsl:param name="os"/>
<xsl:param name="outputname"/>

<xsl:strip-space elements="*"/>

<xsl:output method="text" encoding="ISO-8859-1"/>

<xsl:variable name="upcl">ABCDEFGHIJKLMNOPQRSTUVWXYZ</xsl:variable>
<xsl:variable name="lcl">abcdefghijklmnopqrstuvwxyz</xsl:variable>

<xsl:template match="/">
 
    <xsl:choose>
	 	<xsl:when test="$web='yes'">
		    	<xsl:text>encoding system iso8859-1;</xsl:text>
                <xsl:apply-templates/>
		</xsl:when>
		<xsl:otherwise>
            	<xsl:text>encoding system iso8859-1;</xsl:text>
	    	    <xsl:apply-templates/>
		</xsl:otherwise>
	</xsl:choose>			
</xsl:template>

<xsl:template match="tkml">

    <xsl:choose>
    <xsl:when test="($os='Linux') or ($os='SunOS') or ($os='Darwin')">
	    <xsl:value-of select="&#xA;concat('tk appname ',@label)"/>
    </xsl:when>
    <xsl:when test="$os='Windows NT'">
	      <xsl:value-of select="concat('&#xA;package require dde;dde servername ',@label)"/>
    </xsl:when>
    </xsl:choose>

	<xsl:text>&#xA;global ok wait but;set ok 0;set wait 0;set but release;set web no;</xsl:text>
	<xsl:text>&#xA;package require -exact Tktable 2.8</xsl:text>

	<xsl:value-of select="concat('&#xA;set sourcedir &quot;',$dirname,'&quot;')"/>

    <xsl:value-of select="concat('&#xA;if {[file exists ',$file,'.mod]} {source ',$file,'.mod}')"/>
		
	<xsl:apply-templates/>
	
	<xsl:text>&#xA;catch {wm state . withdrawn};</xsl:text>
	<xsl:text>&#xA;bind Table &lt;Return&gt; {runScilab {}; ::tk::table::MoveCell %W 1 0};</xsl:text>
	<xsl:text>&#xA;DynamicHelp::configure -bg white -font mainFont -delay 200</xsl:text>
</xsl:template>

<xsl:template match="toplevel">

	<xsl:variable name="top">
		<xsl:value-of select="concat('.',translate(@label,$upcl,$lcl))"/>
	</xsl:variable>

  <xsl:value-of select="concat('&#xA;catch {bind ',$top,' &lt;Destroy&gt; {}}')"/>

	<xsl:value-of select="concat('&#xA;if [catch {toplevel ',$top,'}] {destroy ',$top,';toplevel ',$top,'};')"/>
			
	<xsl:apply-templates>
        <xsl:with-param name="parent-id" select="$top"/>
    </xsl:apply-templates>
	
	<xsl:call-template name="pack-children">
		<xsl:with-param name="window" select="$top"/>
	</xsl:call-template>	

	<xsl:value-of select="concat('&#xA;wm resizable ',$top,' 0 0;')"/>
	<xsl:value-of select="concat('&#xA;wm title ',$top,' &quot;',@title,'&quot;')"/>

	<xsl:choose>
		<xsl:when test="ancestor::procedure[@type='tearoff']">
            <xsl:value-of select="concat('&#xA;bind $::toplevelWindow &lt;Destroy&gt; {+destroy ',$top,'}')"/>
        </xsl:when>
		<xsl:when test="ancestor::procedure[@type='transient']">
			<!--<xsl:value-of select="concat('&#xA;wm transient ',$top,' .',ancestor::procedure/@parent)"/>-->
			<xsl:value-of select="concat('&#xA;set gc [grab current];if {$gc!=&quot;&quot;} {bind ',$top,' &lt;Destroy&gt; &quot;grab $gc&quot;}')"/>
			<xsl:value-of select="concat('&#xA;grab ',$top)"/>
		</xsl:when>
		<xsl:when test="$screenshots='no'">
			<xsl:value-of select="concat('&#xA;bind ',$top,' &lt;Destroy&gt; {+set keep no}')"/>
		</xsl:when>
	</xsl:choose>

</xsl:template>

<xsl:template match="font-definition">
	<xsl:value-of select="concat('&#xA;catch {font create ',@name,' -family ',@family,
	                            ' -slant ',@slant,' -size ',@size,' -weight ',@weight,'}')"/>
</xsl:template>

<xsl:template name="name-or-id">
	<xsl:choose>
		<xsl:when test="@name">
			<xsl:value-of select="translate(@name,$upcl,$lcl)"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="generate-id()"/>
		</xsl:otherwise>
	</xsl:choose>	
</xsl:template>

<xsl:template name="pack-children">
	<xsl:param name="window"/>

	<xsl:choose>
		<xsl:when test="@manager='grid'">
			<xsl:for-each select="*[not(self::procedure or self::command) and not(@pack='no')]">
				<xsl:variable name="id">
					<xsl:value-of select="concat($window,'.')"/>
					<xsl:call-template name="name-or-id"/>
				</xsl:variable>		
				<xsl:value-of select="concat('&#xA;grid ',$id,' -row ',@row,' -column ',@col,' -sticky news')"/>
			</xsl:for-each>
		</xsl:when>
		<xsl:otherwise>
			<xsl:for-each select="*[not(self::procedure or self::command) and not(@pack='no')]">
				<xsl:variable name="id">
					<xsl:value-of select="concat($window,'.')"/>
					<xsl:call-template name="name-or-id"/>
				</xsl:variable>		
				<xsl:value-of select="concat('&#xA;pack ',$id)"/>
				<xsl:if test="../@packside">
					<xsl:value-of select="concat(' -side ',../@packside)"/>
				</xsl:if>
				<xsl:if test="@anchor">
					<xsl:value-of select="concat(' -anchor ',@anchor)"/>
				</xsl:if>
        		<xsl:value-of select="concat(' -padx ','1')"/>
        		<xsl:value-of select="concat(' -pady ','1')"/>
				<xsl:if test="@expand">
					<xsl:value-of select="concat(' -expand ',@expand)"/>
				</xsl:if>
				<xsl:if test="@fill">
					<xsl:value-of select="concat(' -fill ',@fill)"/>
				</xsl:if>
			</xsl:for-each>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="scrollableframe">
    <xsl:param name="parent-id"/>

	<xsl:variable name="window">
		<xsl:value-of select="concat($parent-id,'.')"/>
        <xsl:call-template name="name-or-id"/>
	</xsl:variable>

    <xsl:variable name="sff" select="concat($window,'.f')"/>

	<xsl:value-of select="concat('&#xA;ScrolledWindow ',$window)"/>
	<xsl:if test="@relief">
		<xsl:value-of select="concat(' -relief ',@relief,' -bd 1')"/>
	</xsl:if>
	<xsl:value-of select="concat('&#xA;ScrollableFrame ',$sff, ' -areawidth 0 -constrainedwidth true -height 400')"/>
	<xsl:value-of select="concat('&#xA;',$window,' setwidget ',$sff)"/>
    
	<xsl:apply-templates>
        <xsl:with-param name="parent-id" select="concat($window,'.f.frame')"/>
    </xsl:apply-templates>

	<xsl:call-template name="pack-children">
		<xsl:with-param name="window" select="concat($window,'.f.frame')"/>
	</xsl:call-template>

    <!-- the following is a must otherwise the size of the container (pagemanager or notebook ) 
         is not correctly computed : -->
    
    <xsl:text>&#xA;update idletasks</xsl:text>
    <xsl:value-of select="concat('&#xA;',$sff,' configure -width [winfo reqwidth ',$window,'.f.frame]')"/>

</xsl:template>


<xsl:template match="frame[(not(@text)) or (@text='') and not(@textwidget)]">
    <xsl:param name="parent-id"/>

	<xsl:variable name="window">
		<xsl:value-of select="concat($parent-id,'.')"/>
        <xsl:call-template name="name-or-id"/>
	</xsl:variable>

	<xsl:value-of select="concat('&#xA;frame ',$window)"/>
	<xsl:if test="@font">
		<xsl:value-of select="concat(' -font ',@font)"/>
	</xsl:if>	
	<xsl:if test="@height">
		<xsl:value-of select="concat(' -height ',@height)"/>
	</xsl:if>	
	<xsl:if test="@font">
		<xsl:value-of select="concat(' -font ',@font)"/>
	</xsl:if>	
	<xsl:if test="@relief">
		<xsl:value-of select="concat(' -relief ',@relief,' -bd 1')"/>
	</xsl:if>

	<xsl:apply-templates>
        <xsl:with-param name="parent-id" select="$window"/>
    </xsl:apply-templates>

	<xsl:call-template name="pack-children">
        <xsl:with-param name="window" select="$window"/>
	</xsl:call-template>

</xsl:template>

<xsl:template match="frame[@text and (@text!='')]">
    <xsl:param name="parent-id"/>

	<xsl:variable name="window">
		<xsl:value-of select="concat($parent-id,'.')"/>
        <xsl:call-template name="name-or-id"/>
	</xsl:variable>

	<xsl:value-of select="concat('&#xA;labelframe ',$window,' -text &quot;',@text,'&quot;')"/>
	<xsl:if test="@font">
		<xsl:value-of select="concat(' -font ',@font)"/>
	</xsl:if>	

	<xsl:apply-templates>
        <xsl:with-param name="parent-id" select="$window"/>
    </xsl:apply-templates>

	<xsl:call-template name="pack-children">
		<xsl:with-param name="window" select="$window"/>
	</xsl:call-template>

</xsl:template>


<xsl:template match="frame[@textwidget]">
    <xsl:param name="parent-id"/>

	<xsl:variable name="window">
		<xsl:value-of select="concat($parent-id,'.')"/>
        <xsl:call-template name="name-or-id"/>
	</xsl:variable>

	<xsl:variable name="labelwidget">
        <xsl:value-of select="concat($parent-id,'.',translate(@textwidget,$upcl,$lcl))"/>
    </xsl:variable> 

 	<xsl:value-of select="concat('&#xA;labelframe ',$window,' -labelwidget ',$labelwidget)"/>
    <xsl:value-of select="concat('&#xA;foreach sl [pack slaves ',$labelwidget,'] {raise $sl}')"/>

	<xsl:apply-templates>
        <xsl:with-param name="parent-id" select="$window"/>
    </xsl:apply-templates>
    
	<xsl:call-template name="pack-children">
		<xsl:with-param name="window" select="$window"/>
	</xsl:call-template>

</xsl:template>



<xsl:template match="saveTable">
 	<xsl:value-of select="'set savefile [tk_getSaveFile -initialdir $env(HOME) -defaultextension .dat -filetypes {{{data files} {.dat}}} ];set fp [open $savefile w];puts $fp ['"/>
	<xsl:value-of select="concat(@window,' selection set origin end; selection get];close $fp;')"/>
</xsl:template>

<xsl:template match="loadTable">
	<xsl:variable name="label" select="@name"/>
	<xsl:variable name="rows" select="@rows"/>
	<xsl:variable name="cols" select="@cols"/>
	<xsl:value-of select="'set openfile [tk_getOpenFile -initialdir $env(HOME) -filetypes {{{data files} {.dat}}}]; set fp [open $openfile r];'"/>
	<xsl:value-of select="concat(' for {set i 1} {$i&lt;=',$rows+1,'} {incr i} {')"/>
	<xsl:value-of select="concat(' for {set j 1} {$j&lt;=',$cols+1,'} {incr j} {array unset ',$label,' $i,$j}};')"/>
	<xsl:value-of select="' clipboard clear;while {![eof $fp]} {clipboard append [gets $fp]};close $fp;'"/>
	<xsl:value-of select="concat(@window,' activate origin; event generate ',@window,' &lt;&lt;Paste&gt;&gt;')"/>
	<xsl:text>&#xA;runScilab {}</xsl:text>
</xsl:template>

<xsl:template match="eraseTable">
	<xsl:call-template name="erasetable"/>
	<xsl:text>&#xA;runScilab {}</xsl:text>
</xsl:template>

<xsl:template name="erasetable">
	<xsl:variable name="label" select="@name"/>
	<xsl:variable name="rows" select="@rows"/>
	<xsl:variable name="cols" select="@cols"/>
  	<xsl:value-of select="concat('&#xA;for {set i 1} {$i&lt;=',$rows+1,'} {incr i} {')"/>
	<xsl:value-of select="concat(' for {set j 1} {$j&lt;=',$cols+1,'} {incr j} {')"/>
	<xsl:value-of select="concat('array unset ',$label,' $i,$j}};')"/>
</xsl:template>


<xsl:template match="table">
    <xsl:param name="parent-id"/>
    
    <xsl:variable name="window" select="concat($parent-id,'.',translate(@name,$upcl,$lcl))"/>

	<xsl:variable name="table_window">
		<xsl:choose>
			<xsl:when test="ancestor::procedure">
				<xsl:value-of select="concat(@name,'_proc_window')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="concat(@name,'_window')"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	
    <xsl:value-of select="concat('&#xA;set ::',$table_window,' ',$window)"/>

    <xsl:variable name="cellzero" select="concat($window,'.f')"/>

	<xsl:if test="not(ancestor::procedure)">
		<xsl:value-of select="concat('&#xA;unset -nocomplain ::',@name)"/>
    </xsl:if>
	
	<xsl:value-of select="concat('&#xA;table ',$window)"/>
	<xsl:text> -cache 1 -colwidth -70 </xsl:text>
    <xsl:value-of select="concat(' -maxwidth ',@maxwidth)"/>
    <xsl:value-of select="concat(' -maxheight ',@maxheight)"/>
	<xsl:text> -maxheight 400 </xsl:text>
	<xsl:text> -rowseparator \r </xsl:text>
	<xsl:text> -colseparator \t </xsl:text>
	<xsl:text> -multiline 0 </xsl:text>
	<xsl:text> -selectmode extended </xsl:text>	
	<xsl:text> -selecttitle 1 </xsl:text>	
	<xsl:text> -resizeborders none </xsl:text>	
	<xsl:text> -titlecols 1 </xsl:text>
	<xsl:text> -titlerows 1 </xsl:text>
	<xsl:text> -autoclear 1 </xsl:text>
	<xsl:text> -justify left -wrap 1 </xsl:text>
	<xsl:value-of select="concat(' -state ',@state,' -rows ',@rows+1, ' -cols ',@cols+1,' -variable ',@name,' -background #eeeeee -colstretchmode unset &#xA;')"/>
	
    
	<xsl:value-of select="concat($window,' tag col rowtitle 0&#xA;')"/>
	<xsl:value-of select="concat($window,' tag row coltitle 0&#xA;')"/>
	
	<xsl:choose>

		<xsl:when test="@colheaders &gt; 0">	
		    <xsl:value-of select="$window"/>
		    <xsl:text> height 0 2;&#xA;</xsl:text>

			<xsl:if test="not(ancestor::procedure)">
				<xsl:text>for {set i 1} {$i&lt;=</xsl:text>
				<xsl:value-of select="@rows"/>
				<xsl:text>} {incr i} {</xsl:text>
				<xsl:value-of select="$window"/>
				<xsl:text> set $i,0 $i};</xsl:text>
			</xsl:if>
			
			<xsl:value-of select="concat('&#xA;',$window,' tag configure coltitle -relief groove;&#xA;')"/>	
		    <xsl:text>for {set i 1} {$i&lt;=</xsl:text>
		    <xsl:value-of select="@cols"/>
		    <xsl:text>} {incr i} {</xsl:text>
		    <xsl:value-of select="$window"/>
		    <xsl:text> width $i 20};&#xA;</xsl:text>
		</xsl:when>

		<xsl:when test="@rowheaders &gt; 0">
		    <xsl:value-of select="$window"/>
		    <xsl:text> height 0 2;&#xA;</xsl:text>

			<xsl:if test="not(ancestor::procedure)">
		    	<xsl:text>for {set i 1} {$i&lt;=</xsl:text>
		    	<xsl:value-of select="@cols"/>
		    	<xsl:text>} {incr i} {</xsl:text>
		    	<xsl:value-of select="$window"/>
		    	<xsl:text> set 0,$i $i}</xsl:text>
		    	<xsl:text>;&#xA;</xsl:text>
			</xsl:if>
			
		    <xsl:value-of select="$window"/>
			<xsl:text> tag configure rowtitle -relief groove -anchor w;&#xA;</xsl:text>
		</xsl:when>

		<xsl:otherwise>
		    <xsl:value-of select="$window"/>
			<xsl:text> height 0 0;&#xA;</xsl:text>	
		    <xsl:value-of select="$window"/>
			<xsl:text> width 0 0;&#xA;</xsl:text>
		    <xsl:value-of select="$window"/>
			<xsl:text> tag configure title -background #d9d9d9;&#xA;</xsl:text>
		</xsl:otherwise>		

	</xsl:choose>



	<xsl:value-of select="concat($window,' tag configure title -background #b0bee9 -foreground #000000 -relief sunken;&#xA;')"/>
	<xsl:value-of select="concat($window,' tag configure disabled -state disabled -background #dddddd&#xA;')"/>

	<xsl:choose>
		<xsl:when test="not(ancestor::procedure)">
			<xsl:apply-templates>
        		<xsl:with-param name="parent-id" select="$window"/>
    		</xsl:apply-templates>
		</xsl:when>
		<xsl:otherwise>
			<xsl:apply-templates select="*[not(self::command)]">
        		<xsl:with-param name="parent-id" select="$window"/>
    		</xsl:apply-templates>
		</xsl:otherwise>
	</xsl:choose>

	<xsl:choose>
		<xsl:when test="@rowheaders &gt; 0">
			<xsl:text>&#xA;set w 70;for {set i 1} {$i&lt;=</xsl:text>
		    <xsl:value-of select="@rows"/>
		    <xsl:text>} {incr i} {</xsl:text>
			<xsl:value-of select="concat('set nw [pixelStringLength [',$window,' get $i,0] mainFont];&#xA;')"/>
			<xsl:value-of select="concat('if {$nw&gt;$w} {',$window,' width 0 -$nw; set w $nw;}&#xA;')"/>		
		    <xsl:text>};&#xA;</xsl:text>
		</xsl:when>
	</xsl:choose>


	<xsl:choose>
    	<xsl:when test="buttonbox">
	 		<xsl:value-of select="concat('&#xA;',$window,' window configure 0,0 -window ',$window,'.',generate-id(buttonbox),'&#xA;')"/>
	 		<xsl:value-of select="concat('&#xA;',$window,' tag cell title 0,0 &#xA;')"/>
            <xsl:if test="count(buttonbox/*)&gt;2">
                <xsl:value-of select="concat('&#xA;if {[',$window,' width 0] > -90} {',$window,' width 0 -90}&#xA;')"/>
            </xsl:if>        
        </xsl:when>
        <xsl:otherwise>	
	 		<xsl:value-of select="concat('&#xA;frame ',$cellzero,' -background #ff0000&#xA;')"/>
	 		<xsl:value-of select="concat($window,' window configure 0,0 -window ',$cellzero,'&#xA;')"/>
		</xsl:otherwise>
    </xsl:choose>
	    
</xsl:template>

<xsl:template match="scrollbar[@orientation='vertical']">
    <xsl:param name="parent-id"/>

	<xsl:variable name="table_window">
		<xsl:choose>
			<xsl:when test="ancestor::procedure">
				<xsl:value-of select="concat(@what,'_proc_window')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="concat(@what,'_window')"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>

	<xsl:variable name="window">
		<xsl:value-of select="concat($parent-id,'.')"/>
        <xsl:call-template name="name-or-id"/>
	</xsl:variable>

	<xsl:value-of select="concat('&#xA;if {[winfo reqheight $::',$table_window,'] &gt; ',preceding-sibling::*[@name=current()/@what]/@maxheight,'} {')"/>
	<xsl:value-of select="concat('scrollbar ',$window,' -orient vertical -command {$::',$table_window,' yview}')"/>
	<xsl:value-of select="concat('&#xA;$::',$table_window,' configure -yscrollcommand {',$window,' set}')"/>
	<xsl:value-of select="concat('} else {frame ',$window,'}')"/>
</xsl:template>

<xsl:template match="scrollbar[@orientation='horizontal']">
    <xsl:param name="parent-id"/>

	<xsl:variable name="table_window">
		<xsl:choose>
			<xsl:when test="ancestor::procedure">
				<xsl:value-of select="concat(@what,'_proc_window')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="concat(@what,'_window')"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>

	<xsl:variable name="window">
		<xsl:value-of select="concat($parent-id,'.')"/>
        <xsl:call-template name="name-or-id"/>
	</xsl:variable>

	<xsl:value-of select="concat('&#xA;if {[winfo reqwidth $::',$table_window,'] &gt; ',preceding-sibling::*[@name=current()/@what]/@maxwidth,'} {')"/>
	<xsl:value-of select="concat('scrollbar ',$window,' -orient horizontal -command {$::',$table_window,' xview}')"/>
	<xsl:value-of select="concat('&#xA;$::',$table_window,' configure -xscrollcommand {',$window,' set}')"/>
	<xsl:value-of select="concat('} else {frame ',$window,'}')"/>
</xsl:template>


<xsl:template match="colheader">
    <xsl:param name="parent-id"/>
	<xsl:for-each select="col">
		<xsl:if test="@state='disabled'">
			<xsl:value-of select="concat($parent-id,' tag col disabled ',position(),'&#xA;')"/>
		</xsl:if>
	</xsl:for-each>
</xsl:template>

<xsl:template match="buttonbox">
    <xsl:param name="parent-id"/>

	<xsl:variable name="window" select="concat($parent-id,'.',generate-id())"/>
	
	<xsl:value-of select="concat('&#xA;ButtonBox ',$window,' -spacing ',@spacing,' -padx 1 -pady 1  &#xA;')"/>
    	
	<xsl:apply-templates>
        <xsl:with-param name="parent-id" select="$window"/>
    </xsl:apply-templates>

</xsl:template>


<xsl:template match="button[not(@image)]">
    <xsl:param name="parent-id"/>

	<xsl:variable name="window">
			<xsl:value-of select="concat($parent-id,'.')"/>
            <xsl:call-template name="name-or-id"/>
	</xsl:variable>

	<xsl:value-of select="concat('&#xA;button ',$window,
							' -font mainFont -relief groove -text {',text,
							'} -command {')"/>
	<xsl:apply-templates select="command"/>
	<xsl:text>}&#xA;</xsl:text>
</xsl:template>

<xsl:template match="helptext">
    <xsl:text> -helptext &quot;</xsl:text>
    <xsl:for-each select="p">
        <xsl:value-of select="normalize-space(.)"/>
        <xsl:if test="position()&lt;last()">
	        <xsl:text>\n</xsl:text>
        </xsl:if>
    </xsl:for-each>
    <xsl:text>&quot;</xsl:text>
</xsl:template>

<xsl:template match="button[@image]">
    <xsl:param name="parent-id"/>

	<xsl:variable name="window">
		<xsl:value-of select="concat($parent-id,'.')"/>
            <xsl:call-template name="name-or-id"/>
	</xsl:variable>

	<xsl:value-of select="concat('&#xA;Button ',$window,
							' -font mainFont -relief flat -bd 0 -image ',@image,
							' -command {')"/>
	<xsl:apply-templates select="command"/>
	<xsl:text>}</xsl:text>	
    <xsl:apply-templates select="helptext"/>
	<xsl:text>&#xA;</xsl:text>	
</xsl:template>

<xsl:template match="buttonbox/button[@image]">
    <xsl:param name="parent-id"/>

	<xsl:value-of select="concat($parent-id,' add ',
							' -font mainFont -relief groove -bd 1 -image ',@image,
							' -command {')"/>
	<xsl:apply-templates select="command"/>
	<xsl:text>}</xsl:text>	
    <xsl:apply-templates select="helptext"/>
	<xsl:text>&#xA;</xsl:text>	
</xsl:template>

<xsl:template match="label">

    <xsl:param name="parent-id"/>

	<xsl:variable name="window">
		<xsl:value-of select="concat($parent-id,'.')"/>
            <xsl:call-template name="name-or-id"/>
	</xsl:variable>

	<xsl:value-of select="concat('&#xA;Label ',$window)"/> 
	<xsl:choose>
		<xsl:when test="@font">
			<xsl:value-of select="concat(' -font ',@font)"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:text> -font mainFont </xsl:text>
		</xsl:otherwise>
	</xsl:choose>	
    <xsl:if test="@width">
    	<xsl:value-of select="concat(' -wraplength ',@width)"/>
    </xsl:if>
    <xsl:if test="@justify">
    	<xsl:value-of select="concat(' -justify ',@justify)"/>
    </xsl:if>	
	<xsl:value-of select="' -text {'"/>
	<xsl:apply-templates select="text"/>
	<xsl:value-of select="'}'"/>	
    <xsl:apply-templates select="helptext"/>

</xsl:template>

<xsl:template match="text">
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="p">
	<xsl:apply-templates/>
	<xsl:if test="position()&lt;last()">
		<xsl:text>&#xA;</xsl:text>
	</xsl:if>
</xsl:template>


<xsl:template match="checkbutton">
    <xsl:param name="parent-id"/>

	<xsl:variable name="window">
		<xsl:value-of select="concat($parent-id,'.')"/>
            <xsl:call-template name="name-or-id"/>
	</xsl:variable>

	<xsl:value-of select="concat('&#xA;checkbutton ',$window,' -onvalue on -offvalue off -font mainFont',
							' -variable ',@variable)"/> 
	<xsl:if test="@text"><xsl:value-of select="concat(' -text ','&quot;',@text,'&quot;')"/></xsl:if>
	<xsl:if test="command">	
		<xsl:value-of select="concat(' -command {',command,'}')"/>
	</xsl:if>    

</xsl:template>


<xsl:template match="entry">

    <xsl:param name="parent-id"/>

	<xsl:variable name="window">
		<xsl:value-of select="concat($parent-id,'.')"/>
            <xsl:call-template name="name-or-id"/>
	</xsl:variable>
    
	<xsl:value-of select="concat('&#xA;LabelEntry ',$window,' -font mainFont -entrybg white -disabledforeground black',
							' -textvariable ',@variable,' -width ',@width,' -state ',@state)"/> 
	<xsl:if test="@text"><xsl:value-of select="concat(' -labelfont mainFont -label ','&quot;',@text,'&quot;')"/></xsl:if>
	<xsl:if test="command">	
		<xsl:value-of select="concat(' -command {',command,'}')"/>
	</xsl:if>    
<!--	<xsl:value-of select="concat('&#xA;global ',@variable)"/>
    <xsl:if test="value">
        <xsl:value-of select="concat('&#xA;set ',@variable, ' ',value)"/>
    </xsl:if>-->

</xsl:template>

<xsl:template match="scale">
    <xsl:param name="parent-id"/>

	<xsl:variable name="window">
		<xsl:value-of select="concat($parent-id,'.')"/>
            <xsl:call-template name="name-or-id"/>
	</xsl:variable>

	<xsl:value-of select="concat('&#xA;scale ',$window,' -font mainFont -variable ',@variable,
								' -width ',@width,' -state ',@state,' -from ',@from,' -to ',@to,
								' -resolution ',@resolution,' -bigincrement ',@resolution*10,
								' -orient horizontal -width ',@width)"/>
	<xsl:if test="command">
		<xsl:value-of select="concat(' -command {',command,'}')"/>
	</xsl:if>
	<xsl:if test="@repeatdelay">
		<xsl:value-of select="concat(' -repeatdelay ',@repeatdelay,' -repeatinterval 10')"/>
	</xsl:if>

	<xsl:value-of select="concat('&#xA;bind ',$window,' &lt;ButtonRelease-1&gt; {set but release;runScilab {}}')"/>
	<xsl:value-of select="concat('&#xA;bind ',$window,' &lt;ButtonPress-1&gt; {set but press}')"/>

</xsl:template>

<xsl:template match="jog">
    <xsl:param name="parent-id"/>

	<xsl:variable name="window">
		<xsl:value-of select="concat($parent-id,'.')"/>
            <xsl:call-template name="name-or-id"/>
	</xsl:variable>

    <xsl:if test="not(ancestor::procedure)">
        <xsl:value-of select="concat('&#xA;unset -nocomplain ',@variable)"/>
    </xsl:if>

	<xsl:value-of select="concat('&#xA;jog {',$window,'} {',@variable,'} {',@from,'} {',@to,'} {',@resolution,'} {',command,'} {',@period,'}')"/>
</xsl:template>



<xsl:template match="litteral">
	<xsl:value-of select="concat('&#xA;',.)"/>
</xsl:template>

<xsl:template match="command">
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="set">
	<xsl:value-of select="concat('&#xA;set ',@variable,' {',@value,'}')"/>
</xsl:template>

<xsl:template match="unset">
	<xsl:value-of select="concat('&#xA;unset -nocomplain ',@variable)"/>
</xsl:template>

<xsl:template match="unsetmatrix">
	<xsl:call-template name="erasetable"/>
</xsl:template>

<xsl:template match="PagesManager[@style='notebook']">
    <xsl:param name="parent-id"/>

	<xsl:variable name="window" select="concat($parent-id,'.',translate(generate-id(),$upcl,$lcl))"/>

	<xsl:value-of select="concat('&#xA;NoteBook ',$window,' -font ',@font,' -tabpady {5 10} ')"/>

	<xsl:apply-templates>
        <xsl:with-param name="parent-id" select="$window"/>
    </xsl:apply-templates>

	<xsl:value-of select="concat('&#xA;',$window,' compute_size')"/>
    <xsl:choose>
    	<xsl:when test="@raise">
        		<xsl:value-of select="concat('&#xA;',$window,' raise ',@raise)"/>
		</xsl:when>
		<xsl:otherwise>
	    	<xsl:value-of select="concat('&#xA;',$window,' raise ',page[1]/@name)"/>
		</xsl:otherwise>
    </xsl:choose>
</xsl:template>


<xsl:template match="PagesManager[@style='plain']">
    <xsl:param name="parent-id"/>

	<xsl:variable name="window" select="concat($parent-id,'.',translate(generate-id(),$upcl,$lcl))"/>

	<xsl:value-of select="concat('&#xA;PagesManager ',$window)"/>

	<xsl:apply-templates>
        <xsl:with-param name="parent-id" select="$window"/>
    </xsl:apply-templates>

	<xsl:value-of select="concat('&#xA;',$window,' compute_size')"/>
    <xsl:choose>
    	<xsl:when test="@raise">
        		<xsl:value-of select="concat('&#xA;',$window,' raise ',@raise)"/>
		</xsl:when>
		<xsl:otherwise>
	    	<xsl:value-of select="concat('&#xA;',$window,' raise ',page[1]/@name)"/>
		</xsl:otherwise>
    </xsl:choose>
</xsl:template>


<xsl:template match="PagesManager[@style='notebook']/page">
    <xsl:param name="parent-id"/>

	<xsl:variable name="window" select="concat($parent-id,'.f',translate(@name,$upcl,$lcl))"/>

	<xsl:value-of select="concat('&#xA;',$parent-id,' insert ',position(),' ',@name,' -text &quot;',@text,'&quot;')"/>

	<xsl:apply-templates>
        <xsl:with-param name="parent-id" select="$window"/>
    </xsl:apply-templates>
    
	<xsl:call-template name="pack-children">
		<xsl:with-param name="window" select="$window"/>
	</xsl:call-template>

</xsl:template>


<xsl:template match="PagesManager[@style='plain']/page">
    <xsl:param name="parent-id"/>

	<xsl:variable name="window" select="concat($parent-id,'.f',translate(@name,$upcl,$lcl))"/>

	<xsl:value-of select="concat('&#xA;',$parent-id,' add ',@name)"/>

	<xsl:apply-templates>
        <xsl:with-param name="parent-id" select="$window"/>
    </xsl:apply-templates>

	<xsl:call-template name="pack-children">
		<xsl:with-param name="window" select="$window"/>
	</xsl:call-template>

</xsl:template>

<xsl:template match="UserFrame">
    <xsl:param name="parent-id"/>

	<xsl:variable name="window" select="concat($parent-id,'.frame')"/>
    
	<xsl:apply-templates>
        <xsl:with-param name="parent-id" select="$window"/>
    </xsl:apply-templates>    

	<xsl:call-template name="pack-children">
		<xsl:with-param name="window" select="$window"/>
	</xsl:call-template>

</xsl:template>

<xsl:template match="MainFrame">
    <xsl:param name="parent-id"/>

	<xsl:variable name="window" select="concat($parent-id,'.',translate(generate-id(),$upcl,$lcl))"/>
    
	<xsl:apply-templates select="MenuBar"/>

	<xsl:value-of select="concat('&#xA;MainFrame ',$window,' -menu $',generate-id(),'menu',' -textvariable status -separator both')"/>
	<xsl:if test="@menubarfont">
		<xsl:value-of select="concat(' -menubarfont ',@menubarfont)"/>
	</xsl:if>
	<xsl:if test="@menuentryfont">
		<xsl:value-of select="concat(' -menuentryfont ',@menuentryfont)"/>
    </xsl:if> 
	<xsl:if test="@statusbarfont">
		<xsl:value-of select="concat(' -statusbarfont ',@statusbarfont)"/>
    </xsl:if>
	
	<xsl:apply-templates select="ToolBar|UserFrame">
        <xsl:with-param name="parent-id" select="$window"/>
    </xsl:apply-templates>

</xsl:template>

<xsl:template match="MenuBar">
	<xsl:value-of select="concat('&#xA;set ',generate-id(..),'menu ','{')"/>

	<xsl:apply-templates/>

	<xsl:text>}</xsl:text>
</xsl:template>

<xsl:template match="MenuBar/Menu">
	<xsl:value-of select="concat('{',name,'} {} {} 0 {')"/>

	<xsl:apply-templates select="item"/>

	<xsl:text>} </xsl:text>
</xsl:template>

<xsl:template match="ToolBar"/>

<xsl:template match="MenuBar/Menu/item">
	<xsl:value-of select="concat('{ ',@type,' {',text,'} {} {',description,'} {} -command ')"/>
	<xsl:apply-templates select="command"/>
	<xsl:text> } </xsl:text>
</xsl:template>

<xsl:template match="MenuBar/Menu/item/command">
	<xsl:text>{</xsl:text>
	<xsl:apply-templates/>
	<xsl:text>}</xsl:text>
</xsl:template>

<xsl:template match="raise">
	<xsl:variable name="manager" select="concat('.',ancestor::toplevel[1]/@label,'.',generate-id(ancestor::MainFrame),'.frame.',generate-id(ancestor::MainFrame//PagesManager))"/>
	<xsl:value-of select="concat(translate($manager,$upcl,$lcl),' raise ',@name)"/>
</xsl:template>

<xsl:template match="image">
    <xsl:param name="parent-id"/>
	<xsl:variable name="image" select="concat($parent-id,'.image')"/>
	<xsl:variable name="label" select="concat($parent-id,'.',translate(generate-id(),$upcl,$lcl))"/>
    <xsl:value-of select="concat('&#xA;if {[catch {image create photo ',$image,' -file ',@href,'}]} {')"/>
	<xsl:value-of select="concat('image create photo ',$image,' -file [file join $xmllab noarch images notfound.gif]}')"/>
	<xsl:value-of select="concat('&#xA;label ',$label,' -image ',$image)"/>
	<xsl:choose>
		<xsl:when test="@relief">
			<xsl:value-of select="concat(' -relief ',@relief)"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="' -relief groove'"/>
		</xsl:otherwise>
	</xsl:choose>

</xsl:template>

<xsl:template match="menubutton">
    <xsl:param name="parent-id"/>

	<xsl:variable name="window" select="concat($parent-id,'.',translate(@name,$upcl,$lcl))"/>

    <xsl:value-of select="concat('&#xA;set ',@name,'_window ',$window)"/>
	<xsl:value-of select="concat('&#xA;ComboBox ',$window,' -font mainFont -editable no ',
							' -insertontime 0 -entrybg white -width 32 ',
							' -textvariable ',@textvariable,' -values {')"/>
	<xsl:apply-templates select="menu">
		<xsl:with-param name="parent-id" select="$window"/>
	</xsl:apply-templates>

	<xsl:if test="not(ancestor::procedure)">
    <xsl:value-of select="concat('&#xA;',$window,' setvalue first')"/>
  </xsl:if>
</xsl:template>

<xsl:template match="menubutton/menu">
    <xsl:param name="parent-id"/>

	<xsl:apply-templates select="item"/>
	<xsl:value-of select="concat('} -modifycmd {switch [',$parent-id,' getvalue] {')"/> 
		<xsl:for-each select="item">
			<xsl:value-of select="concat(' ',position()-1,' { ')"/>
                <xsl:apply-templates select="command"/>
				<xsl:choose>
					<xsl:when test="@editable='yes'">
						<xsl:value-of select="concat('&#xA;',$parent-id,' configure -editable yes -insertontime 500')"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="concat('&#xA;',$parent-id,' configure -editable no -insertontime 0')"/>
					</xsl:otherwise>
				</xsl:choose>
			<xsl:text>}</xsl:text>
		</xsl:for-each>
	<xsl:text>}}</xsl:text>
</xsl:template>

<xsl:template match="menubutton/menu/item">
	<xsl:value-of select="concat('&quot;',text,'&quot; ')"/>
</xsl:template>

<xsl:template match="scilab-script">
	<xsl:value-of select="concat('&#xA;ScilabEval {exec(&quot;',@href,'&quot;,-1)}')"/>
</xsl:template>

<xsl:template match="scilab-eval">
	<xsl:value-of select="concat('&#xA;ScilabEval {',@cmd,'}')"/>
</xsl:template>


<xsl:template match="comment">
	<xsl:value-of select="concat('&#xA;# ',normalize-space(.))"/>
</xsl:template>

<xsl:template match="user-widget">
    <xsl:param name="parent-id"/>
	<xsl:value-of select="concat('&#xA;',@procedure,' ',$parent-id,' ',@name,' create {',@text,'} {',@variables,'} {',@values,'} {',@parameters,'} {',@lang,'}')"/>
</xsl:template>


<xsl:template match="procedure">
	<xsl:value-of select="concat('&#xA;proc ',@name,' {',@parms,'} ')"/>
	<xsl:apply-templates select="body"/>
</xsl:template>

<xsl:template match="body">
	<xsl:text>{</xsl:text>
	<xsl:apply-templates/>
	<xsl:text>}&#xA;</xsl:text>
</xsl:template>

<xsl:template match="xslt">
	<xsl:text>&#xA;exec xsltproc</xsl:text>
       <xsl:if test="@validate='no'">
    	<xsl:text> --novalid</xsl:text>
	</xsl:if>
    <xsl:value-of select="concat(' --stringparam default-system-uri &quot;',@system-uri,'&quot;',' --stringparam default-public-id &quot;',@public-id,'&quot;',' --stringparam workdir &quot;',@workdir,'&quot;',' --stringparam output &quot;',@output,'&quot; ',@stylesheet,' &quot;',@input,'&quot;')"/>
</xsl:template>

<xsl:template match="xslt[@output='Tcl']">
	<xsl:text>&#xA;if {![catch {exec xsltproc </xsl:text>
    <xsl:for-each select="parm">
        <xsl:value-of select="concat(' --stringparam ',@name,' ',@value)"/>
    </xsl:for-each>
    <xsl:value-of select="concat(' ',@stylesheet,' &quot;',@input,'&quot;} info]} {eval $info}')"/>
</xsl:template>



<xsl:template match="rename-file">
	 <xsl:value-of select="concat('&#xA;file rename -force &quot;',@old,'&quot; &quot;',@new,'&quot;')"/>
</xsl:template>


<xsl:template match="delete-file">
	 <xsl:value-of select="concat('&#xA;file delete -force [file join &quot;',@dir,'&quot; &quot;',@name,'&quot;]')"/>
</xsl:template>

<!--<xsl:template match="write-file">
	<xsl:value-of select="concat('&#xA;set fp [open [file join &quot;',@dir,'&quot; &quot;',@name,'&quot;] w];')"/>
	<xsl:text>&#xA;puts $fp "&lt;?xml version=\"1.0\" encoding=\"ISO-8859-1\"?&gt;"</xsl:text>	
    <xsl:apply-templates mode="write-file"/>
    <xsl:text>&#xA;close $fp;</xsl:text>
</xsl:template>-->

<xsl:template match="write-file">
	<xsl:choose>
		<xsl:when test="@choose='yes'">
			<xsl:value-of select="concat('if {[catch {tk_getSaveFile -initialdir ',@dir,' -defaultextension ',@type,'} filestring]} {mess $filestring;return}&#xA;')"/>
			<xsl:value-of select="'if {$filestring==&quot;&quot;} {return}&#xA;'"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="concat('set filestring [file join &quot;',@dir,'&quot; &quot;',@name,'&quot;]&#xA;')"/>
		</xsl:otherwise>
	</xsl:choose>			
	<xsl:value-of select="'&#xA;set fp [open $filestring w];'"/>
	<xsl:text>&#xA;puts $fp "&lt;?xml version=\"1.0\" encoding=\"ISO-8859-1\"?&gt;"</xsl:text>	
    <xsl:apply-templates mode="write-file"/>
    <xsl:text>&#xA;close $fp;</xsl:text>
</xsl:template>

<xsl:template match="write-html">
	<xsl:value-of select="concat('&#xA;set fp [open [file join &quot;',@dir,'&quot; &quot;',@name,'&quot;] w];')"/>
    <xsl:apply-templates mode="write-file"/>
    <xsl:text>&#xA;close $fp;&#xA;</xsl:text>
    <xsl:value-of select="concat('&#xA;send -async ',@master,' set ::html_file ',@name)"/>
</xsl:template>

<xsl:template match="*" mode="write-file">
	<xsl:variable name="start">
		<xsl:value-of select="concat('&lt;',name())"/>
    	<xsl:apply-templates select="@*" mode="write-file"/>
		<xsl:text>&gt;</xsl:text>
		<xsl:if test="not(text())">
			<xsl:text>&#xA;</xsl:text>
		</xsl:if>
	</xsl:variable>
	<xsl:value-of select="concat('&#xA;puts -nonewline $fp &quot;',$start,'&quot;')"/>
	<xsl:apply-templates select="*|text()" mode="write-file"/>
    <xsl:value-of select="concat('&#xA;puts -nonewline $fp &quot;&lt;/',name(),'&gt;&quot;')"/>
	<xsl:choose>
		<xsl:when test="following-sibling::*">
			<xsl:text>&#xA;puts $fp ""</xsl:text>
		</xsl:when>
		<xsl:when test="not(following-sibling::node())">
			<xsl:text>&#xA;puts $fp ""</xsl:text>
		</xsl:when>
	</xsl:choose>
</xsl:template>

<xsl:template match="tcl:switch" mode="write-file">
	<xsl:value-of select="concat('&#xA;switch ',@expr,' {')"/>
	<xsl:apply-templates mode="write-file"/>
	<xsl:value-of select="'}&#xA;'"/>
</xsl:template>

<xsl:template match="tcl:case" mode="write-file">
    <xsl:value-of select="concat('&#xA;',@value,' {')"/>
    <xsl:apply-templates mode="write-file"/>
	<xsl:value-of select="'}'"/>
</xsl:template>

<xsl:template match="tcl:for" mode="write-file">
    <xsl:value-of select="concat('&#xA; for {set ',@var,' ',@start,'} {$',@var,'&lt;=',@end,'} {incr ',@var,'} {')"/>
    <xsl:apply-templates mode="write-file"/>
	<xsl:value-of select="'}'"/>
</xsl:template>


<xsl:template match="@*" mode="write-file">
	<xsl:value-of select="concat(' ',name(),'=\&quot;',.,'\&quot;')"/>
</xsl:template>

<!--<xsl:template match="text()" mode="write-file">
    <xsl:for-each select="str:split(.,'&quot;')">
		<xsl:value-of select="concat('&#xA;puts -nonewline $fp &quot;',.,'&quot;')"/>
        <xsl:if test="position()&lt;last()">
            <xsl:text>&#xA;puts -nonewline $fp "\&quot;"</xsl:text>
        </xsl:if>
    </xsl:for-each>
</xsl:template>-->

<xsl:template match="text()" mode="write-file">
    <xsl:text>&#xA;puts -nonewline $fp &quot;</xsl:text>
    <xsl:for-each select="str:split(.,'&quot;')">
		<xsl:value-of select="."/>
        <xsl:if test="position()&lt;last()">
            <xsl:text>\&quot;</xsl:text>
        </xsl:if>
    </xsl:for-each>
    <xsl:text>&quot;</xsl:text>
</xsl:template>


<xsl:template match="switch">
	<xsl:value-of select="concat('&#xA;switch ',@expr,' {')"/>
	<xsl:apply-templates/>
	<xsl:value-of select="'}&#xA;'"/>
</xsl:template>

<xsl:template match="case">
        <xsl:value-of select="concat('&#xA;',@value,' {')"/>
        <xsl:apply-templates/>
	<xsl:value-of select="'}'"/>
</xsl:template>

<xsl:template match="upvar">
        <xsl:value-of select="concat('&#xA;upvar ',@name,' ',@variable)"/>
</xsl:template>

<xsl:template match="uplevel">
		 <xsl:value-of select="concat('&#xA;uplevel ',@level,' {')"/>
		 <xsl:apply-templates/>
		 <xsl:value-of select="'}&#xA;'"/>
</xsl:template>

<xsl:template match="trace">
		 <xsl:value-of select="concat('&#xA;trace add variable ',@variable,' write ',@procedure)"/>
</xsl:template>


</xsl:stylesheet>













