<?xml version="1.0"?>

<xsl:stylesheet version="1.0" 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:str="http://exslt.org/strings"
   xmlns:tcl="http://xmllab.org/Tcl" 
   xmlns:exslt="http://exslt.org/common">

<xsl:strip-space elements="*"/>

<xsl:variable name="lang">
  <xsl:choose>
      <xsl:when test="$language='DEFAULT'">
          <xsl:value-of select="/simulation/@default-lang"/>
      </xsl:when>
      <xsl:otherwise>
          <xsl:value-of select="$language"/>
      </xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<xsl:include href="localization.xsl"/>
<xsl:include href="xml2tkml.xsl"/>
<xsl:include href="tkml2tk.xsl"/>

<xsl:param name="database"/>
<xsl:param name="action"/>
<xsl:param name="toplevel"/>
<xsl:param name="language">DEFAULT</xsl:param>

<xsl:output method="text"/>

<xsl:template match="/">
    <xsl:variable name="tkml">
        <tkml>
            <xsl:for-each select="//database[@label=$database]">

                <xsl:variable name="parent">
                  <xsl:choose>
                    <xsl:when test="parent::section[@hidden='yes']">
                      <xsl:value-of select="parent::section/@label"/>
                    </xsl:when>
                    <xsl:otherwise>
                      <xsl:value-of select="$toplevel"/>
                    </xsl:otherwise>
                  </xsl:choose>
                </xsl:variable>

                <xsl:apply-templates select="." mode="init">
                    <xsl:with-param name="action" select="$action"/>
                </xsl:apply-templates>

                <xsl:call-template name="database-menubutton">
                    <xsl:with-param name="label" select="$database"/>
                </xsl:call-template>
                
            </xsl:for-each>
        </tkml>
    </xsl:variable>
    <xsl:apply-templates select="exslt:node-set($tkml)/tkml"/>
</xsl:template>

<xsl:template match="tkml">
        <xsl:apply-templates/>
</xsl:template>

<xsl:template match="menubutton">
    <xsl:variable name="menubutton" select="concat('$',@name,'_window')"/>
    <xsl:value-of select="concat('&#xA;',$menubutton,' configure -values {')"/>
	<xsl:apply-templates select="menu"/>
	<xsl:if test="$action='delete'">
        <xsl:value-of select="concat('&#xA;',$menubutton,' setvalue first')"/>
        <xsl:value-of select="'&#xA;runScilab {}'"/>
    </xsl:if>
</xsl:template>

<xsl:template match="menubutton/menu">
    <xsl:variable name="menubutton" select="concat('$',../@name,'_window')"/>
	<xsl:apply-templates select="item"/>
	<xsl:value-of select="concat('} -modifycmd {switch [',$menubutton,' getvalue] {')"/> 
		<xsl:for-each select="item">
			<xsl:value-of select="concat(' ',position()-1,' { ')"/>
                <xsl:apply-templates select="command"/>
				<xsl:choose>
					<xsl:when test="@editable='yes'">
						<xsl:value-of select="concat('&#xA;',$menubutton,' configure -editable yes -insertontime 500')"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="concat('&#xA;',$menubutton,' configure -editable no -insertontime 0')"/>
					</xsl:otherwise>
				</xsl:choose>
			<xsl:text>}</xsl:text>
		</xsl:for-each>
	<xsl:text>}}</xsl:text>
</xsl:template>

    
</xsl:stylesheet>

