<?xml version="1.0" encoding="ISO-8859-1"?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:exslt="http://exslt.org/common"
exclude-result-prefixes="exslt">

<xsl:param name="file"/>
<xsl:param name="date"/>
<xsl:param name="lang"/>
<xsl:param name="scriptname"/>
<xsl:param name="css"/>
<xsl:param name="images"/>
<xsl:param name="timing"/>

<xsl:include href="localization.xsl"/>

<xsl:output method="xml" 
indent="yes" 
encoding="ISO-8859-1" 
doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN"
doctype-system="DTD/xhtml1-strict.dtd"/>

<xsl:strip-space elements="*"/>

<xsl:template match="/">
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="publish">
    <xsl:apply-templates/>
</xsl:template>

<xsl:template match="publish">
    	<html xmlns="http://www.w3.org/1999/xhtml">
	        <head>
                <title>
                    <xsl:value-of select="normalize-space(title[@lang=$lang])"/>
                </title>
		       <link href="../{$css}" rel="stylesheet" type="text/css"/>
            </head>
            <body marginwidth="10" marginheight="10" class="list">
                <div id="ZoneTexte">
                    <table border="0" style="width: 100%;" cellpadding="0" cellspacing="0">
                      <tbody>
                        <tr>
                          <td style="width: 90%;">
                            <b><font size="+3" ><xsl:value-of select="normalize-space(title[@lang=$lang])"/> </font></b>
                          </td>
			              <xsl:for-each select="title[@lang!=$lang]">
                              <td style="vertical-align: top; text-align: right;">
                                  <div style="text-align: center;">
                                    <img align="middle" hspace="5"
                     				            src="../{$images}/{@lang}_flag.png"/><br/>
                                  </div>
                                  <div style="text-align: center;">
                                    <a href="{$scriptname}?lang={@lang}&amp;list={$file}">
                                        <xsl:value-of select="exslt:node-set($strings)/strings[@lang=current()/@lang]/@versiontext"/>
                                    </a>
                                    <br/>
                                  </div>
                              </td>
                           </xsl:for-each>
	                     </tr>
                      </tbody>
                    </table>
	            <ul>
		            <xsl:apply-templates select="*[not(self::title)]"/>
	            </ul>
                <xsl:apply-templates select="exslt:node-set($strings)/publishlistcredits[@lang=$lang]"/>
				<p><xsl:value-of select="$date"/></p>
            </div>
	        </body>
	    </html>
</xsl:template>

<xsl:template match="publish/title"/>

<xsl:template match="polyscilab-notice">
	<p/>
   	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="publishlistcredits">
	<hr/>
	<small>
	<i>
    	<xsl:apply-templates/>
    </i>
    </small>
</xsl:template>


<xsl:template match="p">
	<xsl:copy-of select="."/>
</xsl:template>


<xsl:template match="category">
	<li>
		<a name="{generate-id()}"/>
    	<xsl:value-of select="name[@lang=$lang]"/>
		<ul>
        	<xsl:apply-templates>
				<xsl:with-param name="lang" select="$lang"/>
            </xsl:apply-templates>       
        </ul>
	</li>
</xsl:template>


<xsl:template match="simulations">
	<xsl:apply-templates>
		<xsl:with-param name="lang" select="$lang"/>
    </xsl:apply-templates>
</xsl:template>

<xsl:template match="simulations/simulation">
    <xsl:variable name="simulation" select="concat(ancestor::category/@dir,'/',@href)"/>
	<xsl:if test="document(concat(ancestor::category/@dir,'/',@href),/)/simulation/header/title[@lang=$lang]">
	    <li>
		    <a href="{$scriptname}?timing={$timing}&amp;lang={$lang}&amp;simulation={$simulation}&amp;action=launch" target="_blank">
	    	    <xsl:value-of select="normalize-space(document(concat(ancestor::category/@dir,'/',@href),/)/simulation/header/title[@lang=$lang])"/>
		    </a>
	    </li>       
    </xsl:if>
</xsl:template>

<xsl:template match="text()"/>

</xsl:stylesheet>


