<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet xmlns:exslt="http://exslt.org/common" 
                xmlns:str="http://exslt.org/strings" 
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
                version="1.0" 
                exclude-result-prefixes="exslt str">
  
  <xsl:strip-space elements="*"/>

  <xsl:variable name="colormapsize">128</xsl:variable>

  <xsl:include href="domains.xsl"/>
  <xsl:include href="ode.xsl"/>
  <xsl:include href="implicit.xsl"/>
  <xsl:include href="pde.xsl"/>

  <xsl:template match="simulation" mode="sciml">  
  
  <!-- This main template generates the sciml code for
    generating the main Scilab script that defines
    auxilliary functions (ode's and implicit functions rhs,
    and others) and the principal callback function "run()"
    which proceeds to computations and display. -->
  
    <sciml>
        
      <function-definition name="proceed">
        <body>
        
          <xsl:if test="descendant::drawpoints">
              <macro-call name="seteventhandler">
                  <inputs>
                      <parm>
                          <string/>
                      </parm>
                  </inputs>
              </macro-call>
          </xsl:if>
          
          <xsl:apply-templates mode="script"/>
        
		  <xsl:if test="$web='yes'">
    			<macro-call name="TCL_EvalStr">
    	  			<inputs>
        				<parm>
        	  				<string>output_html</string>
        				</parm>
    	  			</inputs>
    			</macro-call>
    	   </xsl:if>
        
          <xsl:if test="descendant::drawpoints">
              <macro-call name="seteventhandler">
                  <inputs>
                      <parm>
                          <string>plotlib_handler</string>
                      </parm>
                  </inputs>
              </macro-call>
          </xsl:if>
        
        </body>
      </function-definition>

        <assign>
          <lhs>first_run</lhs>
          <rhs>%t</rhs>
        </assign>

        <xsl:copy-of select="exslt:node-set($strings)/credits[@lang=$lang]"/>

  <!-- At last, we launch the TK interface code -->

        <assign>
          <lhs>oldpwd</lhs>
          <rhs>
            <function-call name="pwd"/>
          </rhs>
        </assign>

        <macro-call name="chdir">
          <inputs>
            <parm>
              <function-call name="get_absolute_file_path">
                <inputs>
                  <parm>
                    <string>
                      <xsl:value-of select="concat($file,'.sce')"/>
                    </string>
                  </parm>
                </inputs>
              </function-call>
            </parm>
          </inputs>
        </macro-call>

        <macro-call name="TCL_EvalFile">
          <inputs>
            <parm>
              <string>
                <xsl:value-of select="concat($file,'.tk')"/>
              </string>
            </parm>
          </inputs>
        </macro-call>

        <xsl:apply-templates mode="functions"/>

        <command>exec(xmllab_run,-1);</command>
        <command>TCL_EvalStr('set ok 1');</command>

        <assign>
          <lhs>first_run</lhs>
          <rhs>%f</rhs>
        </assign>

	    <xsl:choose>

      <xsl:when test="$screenshots='yes'">
       	<macro-call name="TCL_EvalStr">
        	  <inputs>
            	<parm>
            	  <string>wm attributes $::toplevelWindow -topmost 1</string>
            	</parm>
        	  </inputs>
        	</macro-call>
        	  <macro-call name="unix_g">
            	  <inputs>
                	  <parm>
                    	  <string>
	                    	  <xsl:value-of select="concat('import -window &quot;&quot;',normalize-space(header/title[@lang=$lang]),' (0)&quot;&quot; ',$file,'_',$lang,'_0.png')"/>
                    	  </string>
                      </parm>
                  </inputs>
              </macro-call>
			        <macro-call name="TCL_EvalStr">
              <inputs>
                	  <parm>
                    	  <string>send -async publish set ::wait no; destroy $::toplevelWindow</string>
                      </parm>
                  </inputs>
              </macro-call>

          </xsl:when>		  
			
		<xsl:when test="$web='yes'"/>

		<xsl:when test="$shell='yes'"/>

    <xsl:otherwise>

          <macro-call name="chdir">
            <inputs>
              <parm>oldpwd</parm>
            </inputs>
          </macro-call>        

          <!--<xsl:if test="//point">
             <command>seteventhandler('');</command>
            </xsl:if>-->


          </xsl:otherwise>
        </xsl:choose>

    </sciml>
  
  </xsl:template>
  
  <xsl:template match="header" mode="script">
    <xsl:apply-templates select="title" mode="script"/>
  </xsl:template>
  
  <xsl:template match="header" mode="functions">
    <xsl:apply-templates select="script" mode="script"/>
  </xsl:template>
  
  <xsl:template match="title|name" mode="script">
     <comment>
    	<xsl:if test="@lang=$lang or not(@lang)">
        	<xsl:value-of select="normalize-space(.)"/>
        </xsl:if>
      </comment>
  </xsl:template>
    
  <xsl:template match="name" mode="functions"/>
  <xsl:template match="name" mode="script"/>
  <xsl:template match="title" mode="functions"/>
  <xsl:template match="title" mode="script"/>
  <xsl:template match="image" mode="functions"/>
  <xsl:template match="image" mode="script"/>
  
  <xsl:template match="section" mode="script">
    <xsl:apply-templates mode="script"/>
  </xsl:template>
  
  <xsl:template match="subsection" mode="script">
    <xsl:apply-templates mode="script"/>
  </xsl:template>
  
  <xsl:template match="section" mode="functions">
    <xsl:apply-templates mode="functions"/>
  </xsl:template>
  
  <xsl:template match="subsection" mode="functions">
    <xsl:apply-templates mode="functions"/>
  </xsl:template>
  
  <xsl:template match="section[@hidden='yes']" mode="script"/>
  <xsl:template match="subsection[@hidden='yes']" mode="script"/>
<!--<xsl:template match="section[@hidden='yes']" mode="functions"/>
  <xsl:template match="subsection[@hidden='yes']" mode="functions"/>-->
  
  <xsl:template match="database" mode="script">
    <xsl:apply-templates mode="script"/>
  </xsl:template>
  

  <xsl:template match="database" mode="functions">
    <xsl:apply-templates mode="functions"/>
  </xsl:template>
  
  <xsl:template match="record" mode="script"/>
  <xsl:template match="record" mode="functions"/>
  
  <xsl:template match="parameters" mode="functions">
    <xsl:apply-templates mode="functions"/>
  </xsl:template>
  
  <xsl:template match="parameters" mode="script">
    <xsl:apply-templates mode="script"/>
  </xsl:template>

  
  <xsl:template match="group" mode="functions">
    <xsl:apply-templates mode="functions"/>
  </xsl:template>

  <xsl:template match="group" mode="script">
    <xsl:apply-templates mode="script"/>
  </xsl:template>


  <xsl:template match="point" mode="functions">

    <xsl:for-each select="x1|x2|x3">
      <assign>
        <lhs>
          <xsl:value-of select="@label"/>
        </lhs>
        <rhs>
          <xsl:value-of select="value"/>
        </rhs>
      </assign>
    </xsl:for-each>

    <function-definition name="{concat('update_',@label)}">
      <inputs>
        <parm>old_<xsl:value-of select="x1/@label"/></parm>
        <parm>old_<xsl:value-of select="x2/@label"/></parm>
        <parm>new_<xsl:value-of select="x1/@label"/></parm>
        <parm>new_<xsl:value-of select="x2/@label"/></parm>
      </inputs>
      <outputs>
        <parm>

          <xsl:value-of select="x1/@label"/>
        </parm>
        <parm>
          <xsl:value-of select="x2/@label"/>
        </parm>
      </outputs>
      <body>
        <assign>
          <lhs>
            <xsl:value-of select="x1/@label"/>
          </lhs>
          <rhs>new_<xsl:value-of select="x1/@label"/></rhs>
        </assign>
        <assign>
          <lhs>
            <xsl:value-of select="x2/@label"/>
          </lhs>
          <rhs>new_<xsl:value-of select="x2/@label"/></rhs>
        </assign>
        <xsl:if test="constraints/curve">
          <assign>
            <lhs>
              <list sep=",">
                <parm>
                  <xsl:value-of select="x1/@label"/>
                </parm>
                <parm>
                  <xsl:value-of select="x2/@label"/>
                </parm>
              </list>
            </lhs>
            <rhs>
              <function-call name="project_on_curve">
                <inputs>
                  <parm>
                    <xsl:value-of select="x1/@label"/>
                  </parm>
                  <parm>
                    <xsl:value-of select="x2/@label"/>
                  </parm>
                  <xsl:apply-templates select="//*[current()/constraints/curve/@ref=@label]" mode="plot"/>
                </inputs>
              </function-call>
            </rhs>
          </assign>
        </xsl:if>
        <ifthenelse>
          <condition>
            <function-call name="or">
              <inputs>
                <parm>
                  <list sep=";">
                    <parm>-1</parm>
                    <xsl:for-each select="constraints/inequality | //constraints[@label=current()/refconstraints/@ref]/inequality">
                      <parm>
                        <xsl:value-of select="normalize-space(.)"/>
                      </parm>
                    </xsl:for-each>
                  </list>
                  <xsl:text>&gt;-1e-7</xsl:text>
                </parm>
              </inputs>
            </function-call>
          </condition>
          <if>
            <assign>
              <lhs>
                <xsl:value-of select="x1/@label"/>
              </lhs>
              <rhs>
			  	<xsl:value-of select="concat('old_',x1/@label)"/>
			  </rhs>
            </assign>
            <assign>
              <lhs>
                <xsl:value-of select="x2/@label"/>
              </lhs>
              <rhs>
			  	<xsl:value-of select="concat('old_',x2/@label)"/>
			  </rhs>
            </assign>
          </if>
        </ifthenelse>
      </body>
    </function-definition>

  </xsl:template>

  <xsl:template match="matrix" mode="functions">
	<xsl:call-template name="matrix">
		<xsl:with-param name="label" select="@label"/>
	</xsl:call-template>
  </xsl:template>

  <xsl:template match="matrix|scalar" mode="script">
    <xsl:call-template name="TCL_GetVar"/>
  </xsl:template>

  <xsl:template match="point" mode="script">
    <xsl:for-each select="x1|x2|x3">
      <xsl:call-template name="TCL_GetVar"/>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="scalar" mode="functions">
    <xsl:variable name="label" select="@label"/>
	<xsl:call-template name="scalar">
		<xsl:with-param name="label" select="@label"/>
	</xsl:call-template>
    <xsl:if test="@min and @max and @increment and @period">
        <function-definition name="{concat('animate_',@label)}">
          <inputs/>
          <body>
                <macro-call name="driver">
                    <inputs>
                        <parm><string>Pos</string></parm>
                    </inputs>
                </macro-call>
                <for variable="{@label}" min="{@min}" max="{@max}" step="{@increment}">
                    <do>
                        <xsl:apply-templates select="//compute" mode="script"/>
                        <xsl:apply-templates select="//graphs" mode="script"/>
                        <xsl:for-each select="//window[not(@dependencies) or (contains(@dependencies,current()/@label))]">
                            <comment><xsl:value-of select="title[@lang=$lang]"/></comment>
                            <macro-call name="xinit">
                                <inputs>
                                    <parm>
                                        <function-call name="sprintf">
                                            <inputs>
                                                <parm>
                                                    <string>
                                                        <xsl:value-of select="concat('win_',position(),'_%03d')"/>
                                                    </string>
                                                </parm>
                                                <parm>
                                                    <xsl:value-of select="$label"/>
                                                </parm>
                                            </inputs>
                                        </function-call>
                                    </parm>
                                </inputs>
                            </macro-call>                
                            <xsl:call-template name="window"/>
                            <macro-call name="xend"/>
                        </xsl:for-each>
                    </do>
                </for>
                <macro-call name="driver">
                    <inputs>
                        <parm><string>Rec</string></parm>
                    </inputs>
                </macro-call>
          </body>
        </function-definition>    
    </xsl:if>
  </xsl:template>

  <xsl:template name="matrix">
    <xsl:param name="label"/>
    <assign>
      <lhs>
        <list>
          <parm>
            <xsl:value-of select="$label"/>
          </parm>
          <parm>ierr</parm>
          <parm>errtext</parm>
        </list>
      </lhs>
      <rhs>
        <function-call name="check_matrix">
          <inputs>
            <parm>
				<string>
					<xsl:value-of select="@label"/>
				</string>
			</parm>
            <parm>
				<string>
					<xsl:value-of select="normalize-space(name[@lang=$lang])"/>
				</string>
			</parm>
			<parm><xsl:value-of select="@rows"/></parm>
			<parm><xsl:value-of select="@cols"/></parm>
			<parm><string><xsl:value-of select="@striprow"/></string></parm>
			<parm><string><xsl:value-of select="@stripcol"/></string></parm>
          </inputs>
        </function-call>
      </rhs>
    </assign>
	
	<xsl:call-template name="error-message"/>
  
  </xsl:template>



  <xsl:template name="scalar">
    <xsl:param name="label"/>
    <assign>
      <lhs>
        <list>
          <parm>
            <xsl:value-of select="$label"/>
          </parm>
          <parm>ierr</parm>
          <parm>errtext</parm>
        </list>
      </lhs>
      <rhs>
        <function-call name="check_scalar">
          <inputs>
            <parm>
				<string>
					<xsl:value-of select="@label"/>
				</string>
			</parm>
            <parm>
				<string>
					<xsl:value-of select="normalize-space(name[@lang=$lang])"/>
				</string>
			</parm>
            <parm>
				<string>
					<xsl:choose>
						<xsl:when test="@min">
							<xsl:value-of select="@min"/>
						</xsl:when>
						<xsl:otherwise>-%inf</xsl:otherwise>
					</xsl:choose>
            	</string>
			</parm>
            <parm>
				<string>
					<xsl:choose>
						<xsl:when test="@max">
							<xsl:value-of select="@max"/>
						</xsl:when>
						<xsl:otherwise>+%inf</xsl:otherwise>
					</xsl:choose>
				</string>
            </parm>
			<parm>
				<string>
					<xsl:choose>
						<xsl:when test="@scale">
							<xsl:value-of select="@scale"/>
						</xsl:when>
						<xsl:otherwise>1</xsl:otherwise>
					</xsl:choose>
				</string>
			</parm>
          </inputs>
        </function-call>
      </rhs>
    </assign>
	
	<xsl:call-template name="error-message"/>
  
  </xsl:template>

  <xsl:template name="error-message">

	<macro-call name="TCL_SetVar">
		<inputs>
			<parm>
				<string>
					<xsl:value-of select="concat(@label,'_errortext')"/>
				</string>
			</parm>
			<parm>errtext</parm>
		</inputs>
	</macro-call>  

	<ifthenelse>
      <condition>ierr</condition>
      <if>
	  	<xsl:choose>
			<xsl:when test="$web='yes'">
				<macro-call name="TCL_EvalStr">
					<inputs>
						<parm>
							<string>output_html</string>
						</parm>
					</inputs>
				</macro-call>
	  			<macro-call name="return"/>
			</xsl:when>
			<xsl:otherwise>
				<macro-call name="TCL_EvalStr">
					<inputs>
						<parm>
							<string>
								<xsl:value-of select="concat('mess $',@label,'_errortext')"/>
							</string>
						</parm>
					</inputs>
				</macro-call>  		
	  			<macro-call name="return"/>
			</xsl:otherwise>
		</xsl:choose>
	  </if>
    </ifthenelse>

  </xsl:template>
  
  <xsl:template name="TCL_GetVar"> <!-- processing of parameters elements in the main script generation
                                        phase : here we generate the Scilab code that allows to convert the
                                        associated values of parameters (strings) in the TK workspace.
                                        example : a=evtsr(TCL_GetVar('a')) -->


    <xsl:if test="not(@state='disabled')">
        <xsl:choose>
		    <xsl:when test="self::matrix">
			    <xsl:call-template name="matrix">
				    <xsl:with-param name="label" select="concat(@label,'_new')"/>
			    </xsl:call-template>
		    </xsl:when>
		    <xsl:otherwise>
			    <xsl:call-template name="scalar">
    			    <xsl:with-param name="label" select="concat(@label,'_new')"/>
			    </xsl:call-template>
		    </xsl:otherwise>
        </xsl:choose>

        <assign>
          <lhs>
            <xsl:value-of select="concat(@label,'_changed')"/>
          </lhs>
          <rhs>first_run</rhs>
        </assign>

        <ifthenelse>
          <condition>
	  	    <xsl:value-of select="concat('or(',@label,'_new','~=',@label,')')"/>
	      </condition>
          <if>
            <assign>
              <lhs>
                <xsl:value-of select="concat(@label,'_changed')"/>
              </lhs>
              <rhs>%t</rhs>
            </assign>
          </if>
        </ifthenelse>

        <assign>
          <lhs>
            <xsl:value-of select="@label"/>
          </lhs>
          <rhs>
            <xsl:value-of select="concat(@label,'_new')"/>
          </rhs>
        </assign>
    </xsl:if>
  
  </xsl:template>
  
  <!--	Processing of the "script" element : -->
  
  <xsl:template match="script" mode="functions"/>
  
  <xsl:template name="sciml-action-update">
    <xsl:param name="label"/>
    <xsl:for-each select="key('scalars',$label)">
  		<macro-call name="TCL_SetVar">
			<inputs>
				<parm>
					<string>
						<xsl:value-of select="@label"/>
					</string>
				</parm>
				<parm>
					<function-call name="string">
						<inputs>
							<parm>
								<xsl:value-of select="@label"/>
							</parm>
						</inputs>
					</function-call>
				</parm>
			</inputs>
		</macro-call>
      </xsl:for-each>
      <xsl:for-each select="key('matrices',$label)">  		
		<macro-call name="TCL_SetMatrix">
			<inputs>
				<parm>
					<string>
						<xsl:value-of select="@label"/>
					</string>
				</parm>
				<parm>
					<xsl:value-of select="@label"/>
				</parm>
			</inputs>
		</macro-call>
      </xsl:for-each>
  </xsl:template>

<xsl:template name="split-sciml-action-update">
  <xsl:param name="content"/>
  <xsl:choose>
    <xsl:when test="contains($content, ' ')">
      <xsl:call-template name="sciml-action-update">
      	<xsl:with-param name="label" select="substring-before($content, ' ')"/>
      </xsl:call-template>
      <xsl:call-template name="split-sciml-action-update">
        <xsl:with-param name="content" select="substring-after($content, ' ')"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
     <xsl:call-template name="sciml-action-update">
      	<xsl:with-param name="label" select="$content"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>
  
  <xsl:template match="script[@href]" mode="script">
    <macro-call name="exec">
      <inputs>
        <parm>
          <string>
            <xsl:value-of select="concat($utf8outputdir,'/',@href)"/>
          </string>
        </parm>
        <parm>
          <verb>-1</verb>
        </parm>
      </inputs>
    </macro-call>

	<xsl:call-template name="split-sciml-action-update">
        <xsl:with-param name="content" select="@update"/>
    </xsl:call-template>  

</xsl:template>
  
  <xsl:template match="script[not(@href)]" mode="script">
    <xsl:for-each select="str:split(text(),'&#xA;')">
      <command>
        <xsl:value-of select="normalize-space(.)"/>
      </command>
    </xsl:for-each>

	<xsl:call-template name="split-sciml-action-update">
        <xsl:with-param name="content" select="@update"/>
    </xsl:call-template>  

  </xsl:template>
  
<!-- Processing of the "compute" element : definition of 
     domains, high level elements such as ode's, implicit functions,
     etc. and curves.-->

  <xsl:template match="compute" mode="functions">
    <xsl:apply-templates mode="functions"/>
  </xsl:template>
  
  <xsl:template match="compute" mode="script">
    <xsl:apply-templates mode="script"/>
  </xsl:template>

  <xsl:template match="graphs" mode="functions">
    <xsl:apply-templates mode="functions"/>
  </xsl:template>
  
  <xsl:template match="graphs" mode="script">
    <xsl:apply-templates mode="script"/>
  </xsl:template>


  <xsl:template match="outputs"> <!-- Creation of outputs (others) -->
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="output">
    <assign>
      <lhs>
        <xsl:value-of select="@label"/>
      </lhs>
      <rhs>
        <xsl:value-of select="normalize-space(value)"/>
      </rhs>
    </assign>
  </xsl:template>

<!-- Templates for curves and surfaces -->

  <xsl:include href="curves.xsl"/>
  <xsl:include href="surfaces.xsl"/>

<!-- Templates for display -->

  <xsl:include href="display.xsl"/>
  <xsl:template name="variables">
    <xsl:for-each select="variable">
      <assign>
        <lhs>
          <xsl:value-of select="@label"/>
        </lhs>
        <rhs>
          <xsl:value-of select="normalize-space(.)"/>
        </rhs>
      </assign>
    </xsl:for-each>
  </xsl:template>

<!-- Dependencies/independencies in script mode -->

  <xsl:template match="*[@dependencies]" mode="script">
    <ifthenelse>
      <condition>
        <function-call name="or">
          <inputs>
            <parm>
              <list sep=",">
                <xsl:for-each select="str:split(@dependencies,' ')">
                  <parm>
                    <xsl:value-of select="concat(.,'_changed')"/>
                  </parm>
                </xsl:for-each>
              </list>
            </parm>
          </inputs>
        </function-call>
      </condition>
      <if>
        <xsl:apply-templates select="."/>
      </if>
    </ifthenelse>
  </xsl:template>

  <xsl:template match="*[@independencies]" mode="script">
    <ifthenelse>
      <condition>
        <or>
          <arg>first_run</arg>
          <arg>
            <not>
              <function-call name="or">
                <inputs>
                  <parm>
                    <list sep=",">
                      <xsl:for-each select="str:split(@independencies,' ')">
                        <parm>
                          <xsl:value-of select="concat(.,'_changed')"/>
                        </parm>
                      </xsl:for-each>
                    </list>
                  </parm>
                </inputs>
              </function-call>
            </not>
          </arg>
        </or>
      </condition>
      <if>
        <xsl:apply-templates select="."/>
      </if>
    </ifthenelse>
  </xsl:template>



<!-- Templates for save -->

  <xsl:template match="save" mode="script"/>

  <xsl:template match="save" mode="functions">
    <function-definition name="save_stuff">
      <body>
        <xsl:apply-templates/>
      </body>
    </function-definition>
  </xsl:template>

  <xsl:template name="get-filename">
    <function-call name="TCL_GetVar">
      <inputs>
        <parm>
          <string>
		  	<xsl:value-of select="concat('file',position())"/>
		  </string>
        </parm>
      </inputs>
    </function-call>
  </xsl:template>

  <xsl:template match="file[@format='scilab']">
    <macro-call name="save">
      <inputs>
        <parm>
          <xsl:call-template name="get-filename"/>
        </parm>
        <parm>
          <xsl:value-of select="translate(@labels,' ',',')"/>
        </parm>
      </inputs>
    </macro-call>
  </xsl:template>

  <xsl:template match="file[@format='scilab']">
    <macro-call name="save">
      <inputs>
        <parm>
          <xsl:call-template name="get-filename"/>
        </parm>
        <parm>
          <xsl:value-of select="translate(@labels,' ',',')"/>
        </parm>
      </inputs>
    </macro-call>
  </xsl:template>

  <xsl:template match="file[@format='csv']">
    <assign>
      <lhs>
        <list>
          <parm>fp</parm>
          <parm>err</parm>
        </list>
      </lhs>
      <rhs>
        <function-call name="mopen">
          <inputs>
            <parm>
              <xsl:call-template name="get-filename"/>
            </parm>
            <parm>
              <string>w</string>
            </parm>
          </inputs>
        </function-call>
      </rhs>
    </assign>
    <ifthenelse>
      <condition>~err</condition>
      <if>
        <xsl:call-template name="split-save">
          <xsl:with-param name="content" select="normalize-space(@labels)"/>
        </xsl:call-template>
        <macro-call name="mclose">
          <inputs>
            <parm>fp</parm>
          </inputs>
        </macro-call>
      </if>
    </ifthenelse>
  </xsl:template>

  <xsl:template name="split-save">
    <xsl:param name="content"/>
    <xsl:choose>
      <xsl:when test="contains($content, ' ')">
        <xsl:for-each select="//*[@label=substring-before($content, ' ')]">
          <xsl:call-template name="write-data"/>
        </xsl:for-each>
        <xsl:call-template name="split-save">
          <xsl:with-param name="content" select="substring-after($content, ' ')"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:for-each select="//*[@label=$content]">
          <xsl:call-template name="write-data"/>
        </xsl:for-each>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="write-data">
    <macro-call name="mfprintf">
      <inputs>
        <parm>fp</parm>
        <parm>
          <string>\r""%s : ""\t</string>
        </parm>
        <parm>
          <string>
            <xsl:value-of select="normalize-space(name[(@lang=$lang) or not(@lang)])"/>
          </string>
        </parm>
      </inputs>
    </macro-call>
    <macro-call name="mfprintf">
      <inputs>
        <parm>fp</parm>
        <parm>
          <string>%s</string>
        </parm>
        <parm>
		 <xsl:value-of select="concat(@label,'(:)')"/>
		</parm>
      </inputs>
    </macro-call>
  </xsl:template>
  
  <xsl:template match="notes" mode="script"/>
  <xsl:template match="notes" mode="functions"/>
  <xsl:template match="actions" mode="script"/>
  <xsl:template match="actions" mode="functions"/>
  <xsl:template match="user-widget" mode="script"/>
  <xsl:template match="user-widget" mode="functions"/>

</xsl:stylesheet>
