<?xml version="1.0" encoding="ISO-8859-1"?>

<xsl:stylesheet 
  xmlns:exslt="http://exslt.org/common" 
  xmlns:math="http://exslt.org/math" 
  xmlns:str="http://exslt.org/strings" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:tcl="http://xmllab.org/Tcl"
  version="1.0" 
  exclude-result-prefixes="exslt str">

  <xsl:include href="webtk.xsl"/>

  <xsl:strip-space elements="*"/>
  
  <xsl:param name="toplevel">xmllab</xsl:param>

  <xsl:template match="simulation" mode="tkml">

    <tkml>

        <xsl:variable name="label">
            <xsl:choose>
              <xsl:when test="$appname!='NONAME'">
                <xsl:value-of select="$appname"/>
              </xsl:when>
              <xsl:when test="@label">
                <xsl:value-of select="@label"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="$file"/>
              </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:attribute name="label">
            <xsl:value-of select="$label"/>
        </xsl:attribute>

        <xsl:copy-of select="exslt:node-set($strings)/credits[@lang=$lang]"/>

        <command>
            <set variable="::toplevelWindow" value="{concat('.',$toplevel)}"/>
        </command>
 
        <xsl:apply-templates select="parameters/section[@hidden='yes']" mode="pagemanager"/>
        <xsl:apply-templates select="parameters/section/subsection[@hidden='yes']"/>

        <toplevel label="{$toplevel}">
      
		    <xsl:attribute name="title">
			    <xsl:if test="not(header/title[@lang=$lang])">
				    <xsl:message>
                	    <xsl:value-of select="concat('Warning, no title for language &quot;',$lang,'&quot;') "/>
                	    <xsl:value-of select="concat(' (default language is &quot;',/simulation/@default-lang, '&quot;)')"/>
                    </xsl:message> 
                </xsl:if>
        	    <xsl:value-of select="normalize-space(header/title[@lang=$lang])"/>
                <xsl:if test="$screenshots='yes'">
	        	    <xsl:value-of select="' (0)'"/>
			    </xsl:if>
            </xsl:attribute>

            <MainFrame menubarfont="menuFont" menuentryfont="menuFont" statusbarfont="menuFont">

                <MenuBar>

                    <Menu>  <!-- This is the "File" menu -->
                      <name>
                        <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@file"/>
                      </name>
                      <item type="command">
                        <text>
                          <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@run"/>
                        </text>
                        <description>
                          <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@rundesc"/>
                        </description>
                        <command>
                          <litteral>runScilab {}</litteral>
                        </command>
                      </item>

                      <item type="command">
                        <text>
                          <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@savesession"/>
                        </text>
                        <description>
                          <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@savesessiondesc"/>
                        </description>
						<command>
	                        <xsl:call-template name="save-xml-style"/>
                      	</command>
					  </item>

                      <item type="command">
                        <text>
                          <xsl:value-of select="concat(exslt:node-set($strings)/strings[@lang=$lang]/@loadsession,' (XMLlab&lt;1.7 files)')"/>
                        </text>
                        <description>
                          <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@loadsessiondesc"/>
                        </description>
                        <command>
                          <xsl:text>set sessionFile [tk_getOpenFile -initialdir $SESSIONDIR];</xsl:text>
                          <xsl:text>source $sessionFile;</xsl:text>
                          <xsl:text>runScilab {}</xsl:text>
                        </command>
                      </item>

                      <item type="command">
                       <text>
                          <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@loadsession"/>
                        </text>
                        <description>
                          <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@loadsessiondesc"/>
                        </description>
						<command>
	                        <xsl:call-template name="load-xml-style"/>
	                        <xsl:text>;runScilab {}</xsl:text>
    					</command>
	                  </item>
					  
					  
                      <xsl:if test="$widget='plain'">
                        <xsl:apply-templates select="save" mode="menu"/>
                      </xsl:if>
                      <item type="command">
                        <text>
                          <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@quit"/>
                        </text>
                        <description>
                          <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@quitdesc"/>
                        </description>
                        <command>destroy $::toplevelWindow;set ::keep no</command>
                      </item>
                    </Menu>

                    <xsl:apply-templates select="parameters/actions" mode="menu"/>  <!-- This is the "Actions" menu -->

                    <xsl:if test="$widget='plain'">

                      <xsl:if test="(parameters//point[@state!='hidden']) or (parameters//scalar[@state!='hidden']) or (parameters//matrix[@state!='hidden']) or (parameters//database)">
	                      <Menu>  <!-- This is the "Parameters" menu -->
                            <name>
                              <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@parameters"/>
                            </name>
	 	                       <xsl:apply-templates select="parameters/section" mode="menu"/>                
                          </Menu>
	                  </xsl:if>

                      <xsl:if test="display/window">
                        <Menu>
                            <name>
                                <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@windows"/>
                            </name>
	 	                    <xsl:apply-templates select="display/window" mode="menu"/>                
                        </Menu>
                      </xsl:if>

                      <Menu>  <!-- This is the "XMLlab" menu -->
                        <name>XMLlab</name>
                        <item type="command">
                          <text>
                            <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@about"/>
                          </text>
                          <description>
                            <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@aboutdesc"/>
                          </description>
                          <command>
                            <raise name="credits"/>
                          </command>
                        </item>
                        <item type="command">
                          <text>
                            <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@abouthis"/>
                          </text>
                          <description>
                            <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@aboutthisdesc"/>
                          </description>
                          <command>
                            <raise name="notes"/>
                          </command>
                        </item>
                      </Menu>

                    </xsl:if>

                    <xsl:if test="$havesource='yes' and count(header/title)&gt;1">
	                 <Menu> <!-- This is the "Languages" menu -->
                        <name>
                          <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@languages"/>
                        </name>
                        <xsl:for-each select="header/title">
					    	<item type="command">
                                <text>
                                    <xsl:value-of select="exslt:node-set($strings)/strings[@lang=current()/@lang]/@langlang"/>
                                </text>
                                <command>
						            <xsl:value-of select="concat('xmllab 6 &quot;-w ',$widget,' -lang ',@lang,' -run \&quot;',$utf8sourcename,'\&quot;&quot;')"/>
                                </command>
                            </item>
                        </xsl:for-each> 
                      </Menu>
	                </xsl:if>
                </MenuBar>

                <ToolBar/>

                <UserFrame padx="5" pady="5">

	                <PagesManager style="{$widget}" font="mainFont" >
                    
                        <xsl:if test="not ((parameters//point[@state!='hidden']) or (parameters//scalar[@state!='hidden']) or (parameters//matrix[@state!='hidden']) or (parameters//database))">
			                <xsl:attribute name="raise">notes</xsl:attribute>
                        </xsl:if>

                        <xsl:apply-templates select="parameters/section[@hidden='no']" mode="pagemanager"/>
                        <xsl:apply-templates select="display" mode="pagemanager"/>
                        <xsl:apply-templates select="save" mode="pagemanager"/>

                        <page name="credits" text="XMLlab">
                            <xsl:copy-of select="exslt:node-set($strings)/menucredits[@lang=$lang]/frame"/>
							<image href="{$xmllab}/noarch/images/puffin.gif" relief="flat"/>
                        </page>

                        <page name="notes" text="Notes">
                            <frame packside="top" anchor="n" pady="5">
                              <label font="boldFont">
                                <text>
                                  <xsl:value-of select="normalize-space(/simulation/header/title[(@lang=$lang) or (not(@lang))])"/>
                                </text>
                              </label>
                              <label font="mainFont">
                                <text>
                                  <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@author"/>
                                  <xsl:text> : </xsl:text>
                                  <xsl:value-of select="/simulation/header/author"/>
                                  <xsl:text>, </xsl:text>
                                  <xsl:value-of select="/simulation/header/date"/>
                                </text>
                              </label>
                              <xsl:for-each select="notes[@lang=$lang or (not(@lang))]/p">
                                <label font="mainFont" width="300" justify="left">
                                  <text>
                                    <xsl:value-of select="normalize-space(.)"/>
                                  </text>
                                </label>
                              </xsl:for-each>
                            </frame>
                        </page>

	                </PagesManager>

	                <xsl:if test="header/image">
                      <xsl:copy-of select="header/image"/>
                    </xsl:if>
          
	            </UserFrame>
	  
            </MainFrame>

        </toplevel>

        <xsl:apply-templates select="parameters" mode="init"/>
        <xsl:apply-templates select="display" mode="init"/>


<!--        <procedure name="save_session" parms="savefile">
            <body>
                <command>set fp [open $savefile w];</command>            
                <xsl:call-template name="generate-save-cmd-new"/>
                <command>close $fp</command>
            </body>
        </procedure>-->

    </tkml>
    
  </xsl:template>

  <xsl:template match="parameters|group|section|subsection" mode="init">
  	<xsl:apply-templates mode="init"/>
  </xsl:template>

  <xsl:template match="title" mode="init"/>



  <xsl:template match="point" mode="init">
  	<xsl:apply-templates select="x1|x2" mode="init"/>
  </xsl:template>

  <xsl:template match="scalar|x1|x2" mode="init">
    <command>
      <unset variable="{@label}"/>
      <set variable="{@label}" value="{translate(normalize-space(value),' ','')}"/>
    </command>
  </xsl:template>

  <xsl:template match="matrix" mode="init">
    <command>
		<xsl:call-template name="init-matrix"/>
		<xsl:call-template name="setmatrix"/>
    </command>
  </xsl:template>

  <xsl:template match="user-widget[@values]" mode="init">
    <command>
      <xsl:variable name="values" select="@values"/>
      <xsl:for-each select="str:split(@variables)">
        <xsl:variable name="pos" select="position()"/>
        <unset variable="{.}"/>
        <set variable="{.}" value="{str:split($values)[$pos]}"/>
      </xsl:for-each>
    </command>
  </xsl:template>

  <xsl:template match="user-widget[not(@values)]" mode="init"/>

  <xsl:template match="database" mode="init">
    <xsl:param name="action" select="'create'"/>
	<procedure name="{@label}_select" parms="name index op">
		 <body>
    		 <xsl:if test="record">
				 <uplevel level="1">
					 <switch expr="${@label}_index">
						 <xsl:for-each select="record">
							 <case value="{position()}">
								 <xsl:call-template name="database-item-command"/>
							 </case>
						 </xsl:for-each>
					 </switch>
				 </uplevel>
    		</xsl:if>
		 </body>
	</procedure>

    <trace variable="{@label}_index" procedure="{@label}_select"/>

    <xsl:apply-templates select="matrix" mode="init"/>

    <xsl:if test="($action='create') or ($action='delete') or ($action='makedefault')">
	     <set variable="{@label}_index" value="1"/>
    </xsl:if>

  </xsl:template>

  <xsl:template match="actions" mode="init">
    
	<xsl:variable name="label">
      <xsl:choose>
        <xsl:when test="@label">
          <xsl:value-of select="@label"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="concat('actions',generate-id())"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
	  	
	<procedure name="{$label}" parms="name indefx op">
		 <body>
			 <uplevel level="1">
				 <switch expr="${$label}_index">
					 <xsl:for-each select="action">
						 <case value="{position()}">
							 <xsl:call-template name="action-item-command"/>
						 </case>
					 </xsl:for-each>
				 </switch>
			 </uplevel>
		 </body>
	 </procedure>
	 <trace variable="{$label}_index" procedure="{$label}"/>
  </xsl:template>

  <xsl:template name="save-xml-style">
    <write-file dir="$::SESSIONDIR" choose="yes" content="xml" type=".session">
    	<xsl:call-template name="save-xml-style-session"/>
    </write-file>
  </xsl:template>
  
  <xsl:template name="save-xml-style-session">
      <session date="[clock format [clock seconds]]" version="{$xmllab-version}">
	      <header>
		      <xsl:copy-of select="//header/title"/>
	      </header>
	      <data version="{$xmllab-version}">
		      <xsl:for-each select="//scalar|//point/x1 | //point/x2">
    		      <xsl:if test="not(ancestor::database)">
				      <scalar label="{@label}" value="$::{@label}"/>
    		      </xsl:if>
    	      </xsl:for-each>
    	      <xsl:for-each select="//database">
			      <database label="{@label}" index="$::{@label}_index"/>
    	      </xsl:for-each>   
		      <xsl:for-each select="//matrix">
			      <xsl:if test="not(ancestor::database)">
				      <matrix label="{@label}" rows="{@rows}" cols="{@cols}">
					      <tcl:for var="i" start="1" end="{@rows}">
						      <row>
							      <xsl:value-of select="concat('[::$',@label,'_window get $i,1 $i,',@cols,']')"/>
						      </row>
					      </tcl:for>
				      </matrix>
			      </xsl:if>
		      </xsl:for-each> 
	      </data>
      </session>
  </xsl:template>

  <xsl:template name="load-xml-style">
	<xslt workdir="{$utf8outputdir}" input="[tk_getOpenFile -filetypes {{ {{ {{XMLlab sessions}} {{.session}} }} }} -initialdir $SESSIONDIR]"
		stylesheet="[file join $xmllab xsl loadsession.xsl]" output="Tcl">
    </xslt>    
  </xsl:template>

  <xsl:template match="parameters/section[@hidden='no']" mode="menu">

    <item type="command">
      <text>
        <xsl:value-of select="normalize-space(title[(@lang=$lang) or (not(@lang))])"/>
      </text>
      <description>
        <xsl:value-of select="normalize-space(title[(@lang=$lang) or (not(@lang))])"/>
      </description>
      <command>
        <raise name="{translate(generate-id(),'.','_')}"/>
      </command>
    </item>

  </xsl:template>
  
  <xsl:template match="parameters/section[@hidden='yes']" mode="menu"/>
  
  <xsl:template match="parameters/section[@hidden='no']" mode="pagemanager">

	<xsl:variable name="label" select="translate(generate-id(),'.','_')"/>

    <page name="{$label}" text="{normalize-space(title[(@lang=$lang) or (not(@lang))])}" pady="4" type="{$widget}">
           <button anchor="ne" image="tearoffImage">
             <command>
               <xsl:value-of select="concat($label,' {',$toplevel,'}&#10;')"/>
             </command>
           </button>
		<xsl:call-template name="page"/>
    </page>
	
	<procedure name="{$label}" parms="top" parent="{$toplevel}" type="tearoff">
        <body>
          <xsl:for-each select="descendant::matrix">
            <command>
              <litteral>
                <xsl:value-of select="concat('$::',@label,'_window activate 0,0&#10;')"/>
              </litteral>
            </command>
          </xsl:for-each>
          <toplevel label="{$label}" title="{normalize-space(title[(@lang=$lang) or (not(@lang))])}">
			<xsl:call-template name="page"/>
          </toplevel>
        </body>
     </procedure>

  </xsl:template>
  
  <xsl:template name="page">
      <xsl:if test="$widget='plain'">
        <frame packside="top" anchor="n" pady="5" >
          <label font="boldFont">
            <text>
              <xsl:value-of select="normalize-space(title[(@lang=$lang) or (not(@lang))])"/>
            </text>
            <xsl:call-template name="helptext"/>
          </label>
        </frame>
      </xsl:if>
      <xsl:choose>
        <xsl:when test="frame">
          <xsl:apply-templates select="frame"/>
        </xsl:when>
        <xsl:otherwise>
			<xsl:variable name="height">
				<xsl:for-each select="descendant::*[not(ancestor-or-self::subsection[@hidden='yes'])]">
					<xsl:choose>
						<xsl:when test="self::scalar">
							<token>1</token>
						</xsl:when>
						<xsl:when test="self::matrix[@rows&lt;23]">
							<token>
								<xsl:value-of select="(@rows)+1"/>
							</token>
						</xsl:when>
						<xsl:when test="self::matrix[@rows&gt;=23]">
							<token>24</token>
						</xsl:when>
					</xsl:choose>
				</xsl:for-each>
			</xsl:variable>
<!--			<xsl:message>
				<xsl:value-of select="concat('tokens : ',count(exslt:node-set($height)/token))"/>
			</xsl:message>-->
            <xsl:choose>
		 		<xsl:when test="sum(exslt:node-set($height)/token)&lt;30">
                    <frame packside="top" anchor="n" pady="0" padx="0" fill="x" expand="yes">
                        <xsl:apply-templates select="*[not(self::subsection[@hidden='yes'])]"/>
                    </frame>
                </xsl:when>
                <xsl:otherwise>
                    <scrollableframe packside="top" anchor="n" pady="0" padx="0" fill="both" expand="yes" relief="sunken">
                        <xsl:apply-templates select="*[not(self::subsection[@hidden='yes'])]"/>
                    </scrollableframe>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
  </xsl:template>
  
  <xsl:template match="parameters/section[@hidden='yes']" mode="pagemanager">

    <procedure name="{@label}" parms="top" parent="{$toplevel}" type="transient">
      <body>
        <toplevel label="{@label}" title="{normalize-space(title[(@lang=$lang) or (not(@lang))])}">
          <xsl:choose>
            <xsl:when test="frame">
              <xsl:apply-templates select="frame"/>
            </xsl:when>
            <xsl:otherwise>
              <frame packside="top" fill="both" expand="true" padx="5" pady="5">
                <xsl:apply-templates select="*[not(self::subsection[@hidden='yes'])]"/>
              </frame>
            </xsl:otherwise>
          </xsl:choose>
        </toplevel>
      </body>
    </procedure>

  </xsl:template>
  
  <xsl:template match="subsection[@hidden='no']">

    <xsl:variable name="label">
      <xsl:choose>
        <xsl:when test="@label">
          <xsl:value-of select="@label"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="concat('database',generate-id())"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="parent">
      <xsl:choose>
        <xsl:when test="parent::section[@hidden='yes']">
          <xsl:value-of select="parent::section/@label"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$toplevel"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
     <frame name="{$label}tearoff" packside="left" pack="no">
       <label font="boldFont">
         <text>
             <xsl:value-of select="normalize-space(title[(@lang=$lang) or (not(@lang))])"/>
         </text>
       </label>
       <xsl:if test="@tearoff='yes'">
           <button anchor="ne" image="tearoffImage">
             <command>
               <xsl:value-of select="concat($label,' {',$parent,'}&#10;')"/>
             </command>
           </button>
       </xsl:if>
     </frame>


    <frame packside="top" pady="3" fill="both" expand="true">
      <xsl:if test="title or (@tearoff='yes')">
        <xsl:attribute name="textwidget">
            <xsl:value-of select="concat($label,'tearoff')"/>
        </xsl:attribute>
      </xsl:if>
      <xsl:apply-templates/>
    </frame>

    <xsl:if test="@tearoff='yes'">
      <procedure name="{$label}" parms="top" parent="{$parent}" type="tearoff">
        <body>
          <xsl:for-each select="descendant::matrix">
            <command>
              <litteral>
                <xsl:value-of select="concat('$::',@label,'_window activate 0,0&#10;')"/>
              </litteral>
            </command>
          </xsl:for-each>
          <toplevel label="{$label}" title="{normalize-space(title[(@lang=$lang) or (not(@lang))])}">
            <frame packside="top" fill="both" expand="true" padx="5" pady="5">
              <xsl:apply-templates/>
            </frame>
          </toplevel>
        </body>
      </procedure>
    </xsl:if>

  </xsl:template>
  
  <xsl:template match="subsection[@hidden='yes']">

    <xsl:variable name="label">
      <xsl:choose>
        <xsl:when test="@label">
          <xsl:value-of select="@label"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="concat('database',generate-id())"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <procedure name="{$label}" parms="top" type="transient">
      <xsl:attribute name="parent">
        <xsl:choose>
          <xsl:when test="parent::section[@hidden='yes']">
            <xsl:value-of select="parent::section/@label"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$toplevel"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <body>
        <toplevel label="{$label}" title="{normalize-space(title[(@lang=$lang) or (not(@lang))])}">
          <frame packside="top" fill="both" expand="true" padx="5" pady="5">
            <xsl:apply-templates/>
          </frame>
        </toplevel>
      </body>
    </procedure>

  </xsl:template>
  
  <xsl:template match="frame">
    <frame packside="{@packside}" fill="{@fill}" expand="{@expand}" padx="{@padx}" pady="{@pady}">
      <xsl:apply-templates/>
    </frame>
  </xsl:template>
  
  <xsl:template match="pack" name="pack">
    <xsl:param name="content" select="@what"/>
      <xsl:choose>
        <xsl:when test="contains($content, ' ')">
            <xsl:apply-templates select="key('subsections',substring-before($content, ' '))"/>
            <xsl:call-template name="pack">
                <xsl:with-param name="content" select="substring-after($content, ' ')"/>
            </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
            <xsl:apply-templates select="key('subsections',$content)"/>
        </xsl:otherwise>
      </xsl:choose>
  </xsl:template>

  <xsl:template match="actions" mode="menu">
    <Menu>
      <name>
        <xsl:value-of select="normalize-space(title[(@lang=$lang) or (not(@lang))])"/>
      </name>
      <xsl:apply-templates/>
    </Menu>
  </xsl:template>
  
  <xsl:template match="action">
 
    <item type="command">
      <text>
        <xsl:value-of select="normalize-space(title[(@lang=$lang) or (not(@lang))])"/>
      </text>
      <description>
        <xsl:value-of select="normalize-space(title[(@lang=$lang) or (not(@lang))])"/>
      </description>
	  <xsl:call-template name="action-item-command"/>
    </item>
  
  </xsl:template>

<xsl:template name="action-item-command">
  <command>
    <xsl:choose>
      <xsl:when test="script/text()">
        <xsl:element name="scilab-eval">
          <xsl:attribute name="cmd">
            <xsl:for-each select="str:split(script/text(),'&#10;')">
              <xsl:value-of select="normalize-space(.)"/>
              <xsl:if test="position()&lt;last()">
                <xsl:text>,</xsl:text>
              </xsl:if>
            </xsl:for-each>
			<xsl:call-template name="split-action-update">
                <xsl:with-param name="content" select="@update"/>
            </xsl:call-template>
            <xsl:text>,exec(xmllab_run,-1)</xsl:text>
          </xsl:attribute>
        </xsl:element>
      </xsl:when>
      <xsl:otherwise>
        <xsl:element name="scilab-eval">
          <xsl:attribute name="cmd">
            <xsl:value-of select="concat('exec(&quot;',./script/@href,'&quot;,-1)')"/>
				<xsl:call-template name="split-action-update">
                	<xsl:with-param name="content" select="@update"/>
                </xsl:call-template>
            <xsl:text>,exec(xmllab_run,-1)</xsl:text>
          </xsl:attribute>
        </xsl:element>
      </xsl:otherwise>
    </xsl:choose>
  </command>
</xsl:template>

<xsl:template name="action-update">
    <xsl:param name="label"/>
    <xsl:for-each select="key('scalars',$label)">
        <xsl:value-of select="concat(',xmllab_TCL_SetVar(&quot;',@label,'&quot;,string(',@label,'))')"/>
      </xsl:for-each>
      <xsl:for-each select="key('matrices',$label)">
        <xsl:value-of select="concat(',TCL_SetMatrix(&quot;',@label,'&quot;,',@label,')')"/>
      </xsl:for-each>
</xsl:template>

<xsl:template name="split-action-update">
  <xsl:param name="content"/>
  <xsl:choose>
    <xsl:when test="contains($content, ' ')">
      <xsl:call-template name="action-update">
      	<xsl:with-param name="label" select="substring-before($content, ' ')"/>
      </xsl:call-template>
      <xsl:call-template name="split-action-update">
        <xsl:with-param name="content" select="substring-after($content, ' ')"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
     <xsl:call-template name="action-update">
      	<xsl:with-param name="label" select="$content"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="database-item-command">
    <command>
        <set variable="{../@label}" value="{name[@lang=$lang]}"/>
      <xsl:for-each select="../scalar|../group/scalar">
        <set variable="{@label}" value="{value}"/>
      </xsl:for-each>
      <xsl:for-each select="scalar-value">
        <set variable="{@ref}" value="{.}"/>
      </xsl:for-each>
      <xsl:for-each select="../matrix|../group/matrix">
        <unsetmatrix name="{@label}" cols="{@cols}" rows="{@rows}"/>
        <xsl:call-template name="setmatrix"/>
      </xsl:for-each>
      <xsl:for-each select="matrix-value">
        <xsl:call-template name="setmatrix">
          <xsl:with-param name="label" select="@ref"/>
        </xsl:call-template>
      </xsl:for-each>
    </command>
</xsl:template>  

<xsl:template name="database-menubutton">
    <xsl:param name="label"/>
      <menubutton textvariable="{$label}" variable="{concat($label,'index')}" name="{$label}">
        <text>truc</text>
        <menu>
          <xsl:for-each select="record">
            <item editable="no">
              <text>
                <xsl:value-of select="normalize-space(name[(@lang=$lang) or (not(@lang))])"/>
              </text>
			  <command>
		      	<set variable="{$label}_index" value="{position()}"/>
			  </command>
              <command>
                <litteral>runScilab {}</litteral>
              </command>
            </item>
          </xsl:for-each>
          <xsl:if test="@editable='yes' and descendant::*[@state!='hidden']">
            <item editable="yes">
              <text>
                <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@userdefined"/>
              </text>
			  <command>
		      	<set variable="{$label}_index" value="{count(record)+1}"/>
			  </command>            
            </item>
          </xsl:if>
        </menu>
      </menubutton>
</xsl:template>
  
<xsl:template name="database-menu">
   <xsl:param name="parent" select="'normal'"/>

    <frame packside="left" padx="50" anchor="w" fill="x">
  
      <xsl:variable name="label">
        <xsl:choose>
          <xsl:when test="@label">
            <xsl:value-of select="@label"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="concat('database',generate-id())"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
  
      <xsl:call-template name="database-menubutton">
        <xsl:with-param name="label" select="$label"/>
      </xsl:call-template>
        
      <xsl:if test="@editable='yes' and $havesource='yes' and $parent='normal' and descendant::*[@state!='hidden']">

        <button image="saveImage">
          <helptext>
            <p>
            <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@store"/>
            </p>
          </helptext>
          <command>
            <write-file dir="{$utf8dirname}" name="modify.xml" content="xml">
              <modify action="add" database="{$label}" record="${$label}_index" name="${$label}" lang="{$lang}">
                <xsl:for-each select="scalar|group/scalar">
                  <scalar-value ref="{@label}">
                    <xsl:value-of select="concat('$',@label)"/>
                  </scalar-value>
                </xsl:for-each>
                <xsl:for-each select="matrix|group/matrix">
                  <matrix-value ref="{@label}">
                    <xsl:variable name="padd" select="str:padding(2*@rows,' ,')"/>
                    <xsl:variable name="reflabel" select="@label"/>
                    <xsl:variable name="cols" select="@cols"/>
                    <xsl:for-each select="str:split($padd,',')">
                      <row>
                        <value>
                          <xsl:value-of select="concat('[string map {\{\} &quot;&quot;} [$',$reflabel,'_window  get ',position(),',1 ',position(),',',$cols,']]')"/>
                        </value>
                      </row>
                    </xsl:for-each>
                  </matrix-value>
                </xsl:for-each>
                <xsl:for-each select="user-widget|group/user-widget">
                  <xsl:for-each select="str:split(@variables)">
                    <scalar-value ref="{.}">
                      <xsl:value-of select="concat('$',.)"/>
                    </scalar-value>
                  </xsl:for-each>
                </xsl:for-each>
              </modify>
            </write-file>
           <xsl:call-template name="database-xslt-process">
                <xsl:with-param name="label" select="$label"/>
                <xsl:with-param name="action" select="'add'"/>
            </xsl:call-template>        
         </command>
        </button>
	
        <button image="deleteImage">
          <helptext>
            <p>
            <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@erase"/>
            </p>
          </helptext>
          <command>
            <write-file dir="{$utf8dirname}" name="modify.xml" content="xml">
              <modify action="delete" database="{$label}" record="${$label}_index" />
            </write-file>
           <xsl:call-template name="database-xslt-process">
                <xsl:with-param name="label" select="$label"/>
                <xsl:with-param name="action" select="'delete'"/>
            </xsl:call-template>
          </command>
        </button>
	
        <button image="homeImage">
          <helptext>
            <p>
            <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@default"/>
            </p>
          </helptext>
          <command>
            <write-file dir="{$utf8dirname}" name="modify.xml" content="xml">
              <modify action="makedefault" database="{$label}" record="${$label}_index"/>
            </write-file>
            <xsl:call-template name="database-xslt-process">
                <xsl:with-param name="label" select="$label"/>
                <xsl:with-param name="action" select="'makedefault'"/>
            </xsl:call-template>
          </command>
        </button>
	
      </xsl:if>
      
    </frame>
    
</xsl:template>
  
<xsl:template name="database-xslt-process">
    <xsl:param name="label"/>
    <xsl:param name="action"/>
    <xslt workdir="{$utf8outputdir}" validate="no" input="{$utf8sourcename}" 
          output="{concat($utf8sourcename,'.new')}" stylesheet="[file join $xmllab xsl modify.xsl]"
          public-id="{/simulation/@default-public-id}" system-uri="{/simulation/@default-system-uri}"/>
    <rename-file old="{concat($utf8sourcename,'.new')}" new="{$utf8sourcename}"/>
    <xslt workdir="{$utf8outputdir}" input="{$utf8sourcename}"
         stylesheet="[file join $xmllab xsl update.xsl]" output="Tcl">
        <parm name="database" value="{$label}"/>
        <parm name="action" value="{$action}"/>
        <parm name="language" value="{$lang}"/>
    </xslt>    
</xsl:template>
  
<xsl:template match="database">
  
    <xsl:variable name="parent">
      <xsl:choose>
        <xsl:when test="parent::section[@hidden='yes']">
          <xsl:value-of select="parent::section/@label"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$toplevel"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
  
     <frame name="{@label}tearoff" packside="left" pack="no">
       <label font="boldFont">
         <text>
             <xsl:value-of select="normalize-space(title[(@lang=$lang) or (not(@lang))])"/>
         </text>
       </label>
       <xsl:if test="@tearoff='yes'">
           <button anchor="ne" image="tearoffImage">
             <command>
               <xsl:value-of select="concat(@label,' {',$parent,'}&#10;')"/>
             </command>
           </button>
       </xsl:if>
     </frame>
  
  	<xsl:choose>
		<xsl:when test="descendant::*[@state!='hidden']">  
    		<frame packside="top" anchor="n" pady="5" fill="both" expand="true">

    		  <xsl:if test="title or @tearoff='yes'">
        		<xsl:attribute name="textwidget">
            		<xsl:value-of select="concat(@label,'tearoff')"/>
        		</xsl:attribute>
    		  </xsl:if>

    		  <xsl:call-template name="database-menu"/>
    		  <xsl:apply-templates select="scalar|group|matrix|user-widget"/>

    		</frame>
		</xsl:when>
		<xsl:otherwise>
				<frame packside="left" padx="5" expand="true" fill="x">
					<label font="boldFont">
						<text>
							 <xsl:value-of select="concat(normalize-space(title[(@lang=$lang) or (not(@lang))]),'    ')"/>
						</text>
					</label>
	    		  	<xsl:call-template name="database-menu"/>
				</frame>
		</xsl:otherwise>
	</xsl:choose>

    <xsl:if test="@tearoff='yes'">

      <procedure name="{@label}" parms="top" parent="{$parent}" type="tearoff">
        <body>
          <toplevel label="{@label}" title="{normalize-space(title[(@lang=$lang) or (not(@lang))])}">
            <frame packside="top" fill="both" expand="true" pady="5" padx="5">
              <xsl:call-template name="database-menu">
                <xsl:with-param name="parent" select="'procedure'"/>
              </xsl:call-template>
              <xsl:apply-templates select="scalar|group|matrix|user-widget"/>
            </frame>
          </toplevel>
        </body>
      </procedure>

    </xsl:if>

</xsl:template>
  
<xsl:template match="point[@state='hidden']"/>
  
<xsl:template match="point[@state!='hidden']">

    <xsl:variable name="state">
      <xsl:choose>
        <xsl:when test="constraints|refconstraints|@state='disabled'">disabled</xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="@state"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <frame packside="left" padx="5" expand="true" fill="x">
      <label anchor="w" expand="true">
        <text>
          <xsl:value-of select="normalize-space(name[(@lang=$lang) or (not(@lang))])"/>
        </text>
        <xsl:call-template name="helptext"/>
      </label>
      <entry anchor="e" variable="{x1/@label}" width="5" state="{$state}" text="{normalize-space(x1/name[(@lang=$lang) or (not(@lang))])}">
        <value>
          <xsl:value-of select="x1/value"/>
        </value>
        <command>runScilab {}</command>
      </entry>
      <entry anchor="e" variable="{x2/@label}" width="5" state="{$state}" text="{normalize-space(x2/name[(@lang=$lang) or (not(@lang))])}">
        <value>
          <xsl:value-of select="x2/value"/>
        </value>
        <command>runScilab {}</command>
      </entry>
    </frame>

</xsl:template>
  
  
<xsl:template name="helptext">
    <xsl:if test="notes">
      <helptext>
        <xsl:for-each select="notes[(@lang=$lang) or (not(@lang))]/p">
            <xsl:copy-of select="."/>
        </xsl:for-each>
      </helptext>
    </xsl:if>  
</xsl:template>
  
<xsl:template match="group">
  
    <xsl:if test="title">
      <frame packside="left" padx="5" expand="true" fill="x">
        <label anchor="w" expand="true" font="boldFont">
          <text>
            <xsl:value-of select="normalize-space(title[(@lang=$lang) or (not(@lang))])"/>
          </text>
        </label>
      </frame>
    </xsl:if>
    <frame packside="left" padx="2" anchor="e">
      <xsl:apply-templates/>
    </frame>
  
</xsl:template>
  
<xsl:template match="user-widget">

    <xsl:variable name="name">
      <xsl:choose>
        <xsl:when test="@label">
          <xsl:value-of select="@label"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="concat('u',generate-id())"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <frame packside="left" pady="5">
      <user-widget procedure="{@procedure}" name="{$name}" variables="{@variables}" values="{@values}" parameters="{@parameters}" lang="{$lang}">
        <xsl:attribute name="text">
          <xsl:value-of select="strings[(@lang=$lang) or (not(@lang))]"/>
        </xsl:attribute>
      </user-widget>
    </frame>

</xsl:template>

<xsl:template match="scalar[@state='hidden']"/>
<xsl:template match="matrix[@state='hidden']"/>
  
<xsl:template match="scalar">
    <xsl:variable name="state">
        <xsl:choose>
          <xsl:when test="@state='disabled'">disabled</xsl:when>
          <xsl:when test="ancestor::database/@editable='no'">disabled</xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="@state"/>
          </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    <xsl:choose>
        <xsl:when test="@min and @max and @increment and @period"> <!-- animation widget -->
          <frame packside="left" padx="5" expand="true" fill="x">
            <label anchor="w" expand="true">
              <text>
                <xsl:value-of select="normalize-space(name[(@lang=$lang) or (not(@lang))])"/>
              </text>
              <xsl:call-template name="helptext"/>
            </label>
            <frame packside="left" padx="5" relief="groove">
              <scale anchor="e" variable="{@label}" state="{$state}" width="8" from="{@min}" to="{@max}" resolution="{@increment}">
                <value>
                  <xsl:value-of select="value"/>
                </value>
                <command>runScilab</command>
              </scale>
              <jog variable="{@label}" state="{$state}" from="{@min}" to="{@max}" resolution="{@increment}" period="{@period}">
                <command>runScilab {}</command>
              </jog>
            </frame>
          </frame>
        </xsl:when>
        <xsl:when test="@min and @max and @increment"> <!--slider widget -->
          <frame packside="left" padx="5" expand="true" fill="x">
            <label anchor="w" expand="true">
              <text>
                <xsl:value-of select="normalize-space(name[(@lang=$lang) or (not(@lang))])"/>
              </text>
              <xsl:call-template name="helptext"/>
            </label>
            <scale anchor="e" variable="{@label}" state="{$state}" width="8" from="{@min}" to="{@max}" resolution="{@increment}">
              <value>
                <xsl:value-of select="value"/>
              </value>
              <command>runScilab</command>
            </scale>
          </frame>
        </xsl:when>        
        <xsl:otherwise>
          <frame packside="left" padx="5" expand="true" fill="x">
            <label anchor="w" expand="true">
              <text>
                <xsl:value-of select="normalize-space(name[(@lang=$lang) or (not(@lang))])"/>
              </text>
              <xsl:call-template name="helptext"/>
            </label>
            <entry anchor="e" variable="{@label}" state="{$state}" width="12">
              <value>
                <xsl:choose>
                  <xsl:when test="string-length(normalize-space(value))&gt;0">
                    <xsl:value-of select="value"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of select="'0'"/>
                  </xsl:otherwise>
                </xsl:choose>
              </value>
              <command>runScilab {}</command>
            </entry>
          </frame>
        </xsl:otherwise>
    </xsl:choose>              

</xsl:template>
  
<xsl:template name="table-buttonbox">

    <buttonbox spacing="0">
      <xsl:if test="@save='yes'">
        <button image="saveImage">
          <command>
		   <write-file dir="$env(HOME)" choose="yes" content="xml" type=".dat">
				<data date="[clock format [clock seconds]]" version="{$xmllab-version}">
					<matrix label="{@label}" rows="{@rows}" cols="{@cols}">
						<tcl:for var="i" start="1" end="{@rows}">
							<row>
								<xsl:value-of select="concat('[::$',@label,'_window get $i,1 $i,',@cols,']')"/>
							</row>
						</tcl:for>
					</matrix>
				</data>
			</write-file>
          </command>
          <helptext>
		  	<p>
	            <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@savearray"/>
			</p>
          </helptext>
        </button>
      </xsl:if>
      <xsl:if test="@load='yes'">
        <button image="loadImage">
          <command>
			<xslt workdir="{$utf8outputdir}" input="[tk_getOpenFile -filetypes {{ {{ {{XMLlab data}} {{.dat}} }} }} -initialdir $env(HOME)]"
			      stylesheet="[file join $xmllab xsl loadsession.xsl]" output="Tcl">
			</xslt>
			<xsl:text>;runScilab {}</xsl:text> 
          </command>
          <helptext>
    		<p>
	        	<xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@loadarray"/>
          	</p>
		  </helptext>
        </button>
      </xsl:if>
      <xsl:if test="@clear='yes'">
        <button image="deleteImage">
          <command>
            <eraseTable name="{@label}" rows="{@rows}" cols="{@cols}"/>
          </command>
          <helptext>
		  	<p>
            	<xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@cleararray"/>
          	</p>
		  </helptext>
        </button>
      </xsl:if>
    </buttonbox>

  </xsl:template>
  
  <xsl:template match="matrix">

    <frame packside="top" padx="5" pady="5" expand="true" fill="y">

      <frame packside="left" padx="5">
        <label anchor="c" expand="true" font="boldFont">
          <text>
            <xsl:value-of select="normalize-space(name[(@lang=$lang) or (not(@lang))])"/>
          </text>
          <xsl:call-template name="helptext"/>
        </label>
        <xsl:if test="(@save='yes' or @load='yes' or @clear='yes') and not (col/name or row/name)">
          <xsl:call-template name="table-buttonbox"/>
        </xsl:if>
      </frame>

	<frame manager="grid" relief="flat">
      <table row="0" col="0" name="{@label}" rows="{@rows}" cols="{@cols}" 
	         colheaders="{count(col[name])+count(colnames)}" 
			 rowheaders="{count(row[name])+count(rownames)}" 
			 maxwidth="800"
			 maxheight="400"
			 state="{@state}">
        <xsl:if test="(@save='yes' or @load='yes' or @clear='yes') and (col/name or row/name)">
          <xsl:call-template name="table-buttonbox"/>
        </xsl:if>
		<xsl:copy-of select="colheader|rowheader"/>
    	<command>
    	  <xsl:call-template name="setmatrix"/>
    	</command>
      </table>
	  <scrollbar what="{@label}" row="0" col="1" orientation="vertical"/>
	  <scrollbar what="{@label}" row="1" col="0" orientation="horizontal"/>
	</frame>

    </frame>


  </xsl:template>
  
  <xsl:template name="setmatrix">
     <xsl:param name="label" select="@label"/>
	 <xsl:for-each select="row">
		 <xsl:variable name="i" select="position()"/>
		 <set variable="{$label}({$i},0)" value="  {name[@lang=$lang]} "/>
		 <xsl:for-each select="str:split(translate(value,' ',','),',')">
			 <set variable="{$label}({$i},{position()})" value="{.}"/>
		 </xsl:for-each>
	 </xsl:for-each>
	 <xsl:for-each select="col">
		 <xsl:variable name="j" select="position()"/>
		 <set variable="{$label}(0,{$j})" value="{name[@lang=$lang]}"/>
		 <xsl:for-each select="str:split(translate(value,' ',','),',')">
			 <set variable="{$label}({position()},{$j})" value="{.}"/>
		 </xsl:for-each>
	 </xsl:for-each>
	 <xsl:for-each select="rowheader">
		<xsl:for-each select="row">
			<set variable="{$label}({position()},0)" value="  {name[@lang=$lang]} "/>
		</xsl:for-each>
	 </xsl:for-each>
	 <xsl:for-each select="colheader">
		<xsl:for-each select="col">
			<set variable="{$label}(0,{position()})" value="{name[@lang=$lang]}"/>
		</xsl:for-each>
	 </xsl:for-each>
  </xsl:template>
  
  <xsl:template name="init-matrix">
	<!-- initialization with empty strings -->
    <unset variable="{@label}"/>
	<xsl:call-template name="matrix-rows">
		<xsl:with-param name="i" select="number(1)"/>
	</xsl:call-template>
</xsl:template>

<xsl:template name="matrix-rows">
	<xsl:param name="i"/>
	<xsl:call-template name="matrix-cols">
		<xsl:with-param name="j" select="number(1)"/>
		<xsl:with-param name="i" select="$i"/>
	</xsl:call-template>

	<xsl:if test="$i&lt;@rows">
		<xsl:call-template name="matrix-rows">
			<xsl:with-param name="i" select="$i+1"/>
		</xsl:call-template>
	</xsl:if>
</xsl:template>

<xsl:template name="matrix-cols">
	<xsl:param name="i"/>
	<xsl:param name="j"/>
	<set variable="{@label}({$i},{$j})" value=""/>
	<xsl:if test="$j&lt;@cols">
		<xsl:call-template name="matrix-cols">
			<xsl:with-param name="j" select="$j+1"/>
			<xsl:with-param name="i" select="$i"/>
		</xsl:call-template>
	</xsl:if>

</xsl:template>

  
  <xsl:template match="save" mode="menu">
    <item type="command">
      <text>
        <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@saveresults"/>
      </text>
      <description>
        <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@saveresultsdesc"/>
      </description>
      <command>
        <raise name="{translate(generate-id(),'.','_')}"/>
      </command>
    </item>
  </xsl:template>

<!-- Display and window stuff that appear in the gui -->

  <xsl:template match="window" mode="menu">
    <item type="command">
      <text>
        <xsl:choose>
            <xsl:when test="title[@lang=$lang]">
                <xsl:value-of select="normalize-space(title[@lang=$lang])"/>
		    </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="concat('ScilabGraphic',position()-1)"/>
            </xsl:otherwise>
        </xsl:choose>
    </text>
      <description>
       <xsl:choose>
            <xsl:when test="title[@lang=$lang]">
                <xsl:value-of select="normalize-space(title[@lang=$lang])"/>
		    </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="concat('ScilabGraphic',position()-1)"/>
            </xsl:otherwise>
        </xsl:choose>
      </description>
      <command>
        <xsl:if test="@editable='yes'">
            <raise name="{concat('window',position())}"/>
        </xsl:if>
        <scilab-eval cmd="_fig({position()-1}),_shg"/>
      </command>
    </item>
  </xsl:template>

  <xsl:template match="display|window" mode="init">
    <xsl:apply-templates mode="init"/>
  </xsl:template>

  <xsl:template match="axis2d|axis3d" mode="init">
    <xsl:apply-templates mode="init"/>
  </xsl:template>


  <xsl:template match="drawcurve2d|drawsurface" mode="init">
      <set variable="display_{generate-id()}" value="on"/>
  </xsl:template>
  
  <xsl:template match="display" mode="pagemanager">
    <xsl:apply-templates select="window[@editable='yes']" mode="pagemanager"/>
  </xsl:template>

  <xsl:template match="window" mode="pagemanager">
    
    <page name="{concat('window',position())}">
        <xsl:variable name="text">
            <xsl:choose>
                <xsl:when test="title[@lang=$lang]">
                    <xsl:value-of select="normalize-space(title[@lang=$lang])"/>
		        </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="concat('ScilabGraphic',position()-1)"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:attribute name="text">
            <xsl:value-of select="$text"/>
        </xsl:attribute>
        <xsl:if test="$widget='plain'">
        <frame packside="top" anchor="n" pady="5" >
          <label font="boldFont">
            <text>
              <xsl:value-of select="$text"/>
            </text>
          </label>
        </frame>
      </xsl:if>
        
        <frame manager="grid">
            <xsl:for-each select="axis2d|axis3d">
                <frame col="{floor((position()-1) div (floor(math:sqrt(last()))))}" row="{(position()-1) mod (floor(math:sqrt(last())))}">
                    <xsl:attribute name="text">
                        <xsl:choose>
                            <xsl:when test="title[@lang=$lang]">
                                <xsl:value-of select="normalize-space(title[@lang=$lang])"/>
		                    </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select="concat('axis ',position())"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:attribute>
                    <xsl:apply-templates select="drawcurve2d|drawsurface" mode="pagemanager"/>
                </frame>
            </xsl:for-each>
        </frame>
    </page>
  </xsl:template>

  <xsl:template match="drawcurve2d|drawsurface" mode="pagemanager">
    <xsl:if test="key('graphs',@ref)/name[@lang=$lang]">
      <frame packside="right" padx="5" pady="0" fill="x">
          <checkbutton anchor="w" variable="display_{generate-id()}">
            <command>ScilabEval xmllab_display</command>
          </checkbutton>
          <label anchor="w" expand="true" fill="x">
            <text>
                <xsl:value-of select="key('graphs',@ref)/name[@lang=$lang]"/>
            </text>
          </label>
      </frame>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="save" mode="pagemanager">

    <page name="{translate(generate-id(),'.','_')}" 
          text="{exslt:node-set($strings)/strings[@lang=$lang]/@saveresults}">

      <xsl:if test="$widget='plain'">
        <frame packside="top" anchor="n" pady="5">
          <label font="boldFont">
            <text>
              <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@saveresults"/>
            </text>
          </label>
        </frame>
      </xsl:if>

      <xsl:apply-templates select="file" mode="save"/>

      <button>
        <text>
          <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@save"/>
        </text>
        <command>ScilabEval exec(save_stuff,-1)</command>
      </button>

    </page>

  </xsl:template>
  
  <xsl:template match="file" mode="save">

    <frame packside="left" padx="5" expand="true" fill="x">
      <label anchor="w" width="250" expand="true">
        <text>
          <xsl:value-of select="translate(@labels,' ',',')"/>
        </text>
      </label>
      <label anchor="w" expand="true">
        <text>(Format : <xsl:value-of select="@format"/>)</text>
      </label>
      <entry variable="file{position()}" state="normal" width="20">
        <value>$env(HOME)/<xsl:value-of select="@href"/></value>
      </entry>
    </frame>

  </xsl:template>
  
  <xsl:template match="headers|notes|title|display|save|image"/>

</xsl:stylesheet>
